"""An API to hit Amazon scope.

It's heavily adapted from bottlenose code, particulary not using
urllib2.urlopen but centralized urlread.
"""

import cStringIO as StringIO
import gzip
import hmac
import time
import urllib

from base64 import b64encode
from hashlib import sha256


SERVICE_DOMAINS = {
    'CA': 'ecs.amazonaws.ca',
    'CN': 'webservices.amazon.cn',
    'DE': 'ecs.amazonaws.de',
    'ES': 'webservices.amazon.es',
    'FR': 'ecs.amazonaws.fr',
    'IT': 'webservices.amazon.it',
    'JP': 'ecs.amazonaws.jp',
    'UK': 'ecs.amazonaws.co.uk',
    'US': 'ecs.amazonaws.com',
}

# version and corresponding namespace
AMZ_VERSION = "2011-08-01"
AMZ_NS = {'x': "http://webservices.amazon.com/AWSECommerceService/2011-08-01"}

# the signature for a gzip compressed binary
GZIP_SIGNATURE = "\x1F\x8B\x08"


class Amazon(object):
    """The interface to Amazon."""

    def __init__(self, urlread, aws_key, aws_secret, associate_tag, region, _operation=None):
        self.urlread = urlread
        self.aws_key = aws_key
        self.aws_secret = aws_secret
        self.associate_tag = associate_tag
        self.region = region
        self._operation = _operation

    def __getattr__(self, k):
        try:
            return object.__getattr__(self, k)
        except:
            return Amazon(self.urlread, self.aws_key, self.aws_secret,
                          self.associate_tag, self.region, _operation=k)

    def __call__(self, **kwargs):
        kwargs['Timestamp'] = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        kwargs['Operation'] = self._operation
        kwargs['Version'] = AMZ_VERSION
        kwargs['AWSAccessKeyId'] = self.aws_key
        kwargs['Service'] = "AWSECommerceService"
        kwargs['AssociateTag'] = self.associate_tag

        service_domain = SERVICE_DOMAINS[self.region]
        items = [(k, urllib.quote(unicode(v).encode('utf-8'), safe='~'))
                 for k, v in sorted(kwargs.items())]
        quoted_strings = "&".join("%s=%s" % x for x in items)

        data = "GET\n%s\n/onca/xml\n%s" % (service_domain, quoted_strings)

        digest = hmac.new(self.aws_secret, data, sha256).digest()
        signature = urllib.quote(b64encode(digest))

        api_string = "http://%s/onca/xml?%s&Signature=%s" % (
            service_domain, quoted_strings, signature)
        response = self.urlread(api_string, headers={"Accept-Encoding": "gzip"})

        if response.startswith(GZIP_SIGNATURE):
            gzipped_file = gzip.GzipFile(fileobj=StringIO.StringIO(response))
            response = gzipped_file.read()
        return response
