# Copyright 2014-2015 Canonical
# All Rights Reserved

"""Rest Scopes server."""

import collections
import importlib
import json
import logging
import os
import re
import time
import urlparse
import urllib2

import yaml

import utils

# this module is generated by a "make build"
from versioninfo import version_info

from translator import translator


urlread_logger = logging.getLogger('restscopes.urlread')

REVNO_HEADER = ('X-Bzr-Revision-Number', str(version_info['revno']))

X_CACHE_RE = re.compile('.*(HIT from (juju-[a-zA-Z0-9]+-u1-psearch.*|localhost))')

# load general config
with open(os.environ['REST_SCOPES_CONFIG']) as fh:
    config = yaml.load(fh)


def dict_merger(d, u):
    """Merge both dicts.

    Taken from https://stackoverflow.com/questions/3232943/
               update-value-of-a-nested-dictionary-of-varying-depth
    """
    for k, v in u.iteritems():
        if isinstance(v, collections.Mapping):
            r = dict_merger(d.get(k, {}), v)
            d[k] = r
        else:
            d[k] = u[k]
    return d


def get_scopes_config():
    """Get the info for the scopes from the config(s) file(s)."""
    # check if we have the simple case (just the name) or a list with names
    scopes_config = config['scopes_config']
    if isinstance(scopes_config, basestring):
        scopes_config = [scopes_config]

    # compose the configs, by just updating a dict
    info = {}
    for fname in scopes_config:
        with open(os.path.join(os.environ['ROOTDIR'], fname)) as fh:
            info.update(yaml.load(fh))
    return info


def get_scopes_secrets():
    """Get the secrets for the scopes from the config file.

    In the file we store the secrets grouping by partner (much simpler
    for the human), but then we need them by scope (so we can pass them to
    each one of those), so here we'll kind of transpose them.
    """
    scopes_config_file = os.path.join(os.environ['ROOTDIR'],
                                      config['scopes_secrets'])
    with open(scopes_config_file) as fh:
        secrets = yaml.load(fh) or {}

    transposed = {}
    for partner, partner_data in secrets.items():
        if partner == 'default':
            partner = None
        for scope, scope_data in partner_data.items():
            d = transposed.setdefault(scope, {})
            d[partner] = scope_data

    return transposed


def urlread(meter, prefix, url, data=None, headers=None):
    """Centralized urlopen with extra functionality."""
    if headers is None:
        headers = {}

    def _inform(response, headers=None):
        """Inform the metrics."""
        if meter is None:
            return

        # deduce the cache behaviour
        if headers is None:
            cache = None
        else:
            try:
                x_cache = headers['X-Cache']
            except KeyError:
                cache = 'no-cache'
            else:
                if X_CACHE_RE.match(x_cache):
                    cache = 'hit'
                else:
                    x_cache_lookup = headers.get('X-Cache-Lookup', '')
                    if X_CACHE_RE.match(x_cache_lookup):
                        cache = 'miss-expired'
                    else:
                        cache = 'miss'

        # build path and inform
        deltat = time.time() - tini
        domain = urlparse.urlparse(url).netloc.replace('.', '_')
        name_parts = (prefix, 'backend', domain, response, cache)
        name = '.'.join(x for x in name_parts if x)
        meter.timing(name, deltat)

    tini = time.time()
    req = urllib2.Request(url, data=data, headers=headers)
    try:
        u = urllib2.urlopen(req)
        data = u.read()
    except urllib2.HTTPError as exc:
        _inform(str(exc.code), exc.hdrs)
        if exc.fp is None:
            body = None
        else:
            body = exc.fp.read()
        urlread_logger.error("HTTPError %s (%s) when hitting %r: %r", exc.code, exc.msg, url, body)
        raise
    except Exception as exc:
        _inform(exc.__class__.__name__)
        raise

    _inform('200', u.headers)
    return data


class App(object):
    """The WSGI application.

    It can receive a dict with extra config per scope, that will overwrite
    whatever the config in disk would be.
    """

    def __init__(self, extra_scope_config=None, metrics=None, oopser=None):
        if metrics is None:
            self._meter = self._metric_prefix = scope_metrics = None
        else:
            self._meter = metrics[0]
            if metrics[1]:
                self._metric_prefix = metrics[1] + '.restscopes'
            else:
                self._metric_prefix = 'restscopes'
            scope_metrics = (self._meter, self._metric_prefix)

        # load scope info and modules
        scope_info = []
        self.scope_apps = {}
        info = get_scopes_config()
        scopes_secrets = get_scopes_secrets()
        self._real_partners = set()
        for scope_id, data in info.items():
            base_url = data['base_url']
            if base_url[0] == '/':
                # internal path
                base_url = config['server'] + base_url
                internal = True
            else:
                internal = False

            fqdn = data.get('fqdn')
            try:
                icon_fname = data['icon']
            except KeyError:
                icon_img = None
            else:
                icon_img = os.path.join(config['base_image_url'], icon_fname)
            try:
                art_fname = data['art']
            except KeyError:
                art_img = None
            else:
                art_img = os.path.join(config['base_image_url'], art_fname)

            # get appearance info, and fix logo path if needed
            appearance = data.get('appearance', {})
            _pageheader = appearance.get('page-header', {})
            if 'logo' in _pageheader:
                _pageheader['logo'] = os.path.join(config['base_image_url'],
                                                   _pageheader['logo'])

            scope_info.append({
                'id': fqdn if fqdn else scope_id,
                'name': data['name'].strip(),
                'description': data['description'].strip(),
                'base_url': base_url,
                'invisible': data['invisible'],
                'icon': icon_img,
                'art': art_img,
                'author': data['author'],
                'appearance': appearance,
                'needs_location_data': data.get('needs_location_data', False),
                'hidden': data.get('hidden', False),
                'keywords': data.get('keywords', []),
            })
            if internal:
                package = data.get('mod_package', config['scopes_package'])
                mod_path = "%s.%s" % (package, scope_id)
                mod = importlib.import_module(mod_path)

                # build the config
                secrets = scopes_secrets.get(scope_id, {})
                self._real_partners.update(secrets)
                scope_config = config.copy()
                scope_config['secrets'] = secrets
                if extra_scope_config is not None:
                    extra = extra_scope_config.get(scope_id)
                    if extra is not None:
                        scope_config = dict_merger(scope_config, extra)

                self.scope_apps[scope_id] = mod.App(scope_config, scope_metrics, oopser)
        self.scope_info = {None: scope_info}

    def _generic_response(self, status, start_response,
                          message="", **extra_headers):
        """Answer a generic response."""
        response_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(message))),
            REVNO_HEADER,
        ]
        response_headers.extend(extra_headers.items())
        start_response(status, response_headers)
        return [message]

    def _error(self, status, start_response, message="",
               **extra_headers):
        """Answer an error response."""
        data = {"error": status}
        if message:
            data['message'] = message
        return self._generic_response(status, start_response,
                                      json.dumps(data, indent=4),
                                      **extra_headers)

    def _send_metric(self, name, tstart):
        """Send the metric, if available."""
        if self._meter is not None:
            delta_t = time.time() - tstart
            self._meter.timing(self._metric_prefix + '.' + name, delta_t)

    def __call__(self, environ, start_response):
        """Serve the request."""
        tini = time.time()
        path = environ["PATH_INFO"].rstrip("/")
        if path.startswith("/_static"):
            return self.serve_static(environ, start_response, path)

        # before serving do an inspection to maybe locate a special partner and
        # transform it as a deep seer; as a side effect leave the UA expanded
        useragent_raw = environ.pop("HTTP_USER_AGENT", "")
        useragent = dict(urlparse.parse_qsl(useragent_raw,
                                            keep_blank_values=True))
        environ["USER_AGENT"] = useragent
        partner = useragent.get('partner')
        if partner is not None:
            partner = partner.strip()
        if partner not in self._real_partners:
            del useragent['partner']
            environ['DEEP_SIGHT'] = partner

        # the list of configured scopes
        if path == "/remote-scopes":
            info = self.list_remote_scopes(environ, start_response)
            self._send_metric('remote-scopes.200', tini)
            return info

        # may be a scope!
        scope_name = path.split("/")[1]
        try:
            scope = self.scope_apps[scope_name]
        except KeyError:
            self._send_metric('404', tini)
            return self._error("404 NOT FOUND", start_response)

        scope_path = path[len(scope_name) + 1:]  # +1 because of initial /
        if len(scope_path) < 2:
            msg = "Malformed URL: need an action after the scope name"
            self._send_metric('400', tini)
            return self._error("400 BAD REQUEST", start_response, msg)

        # fix environment and serve
        environ["PATH_INFO"] = scope_path
        return scope(environ, start_response)

    def curate_list(self, country_blacklist, scopes):
        """Remove blacklisted scopes."""
        curated_list = []
        for scope in scopes:
            if scope['id'] in country_blacklist:
                continue
            curated_list.append(scope)
        return curated_list

    def list_remote_scopes(self, environ, start_response):
        """List all the configured scopes."""
        qs = environ.get("QUERY_STRING", "")
        qd = dict(urlparse.parse_qsl(qs, keep_blank_values=True))
        locale = utils.sanitize_locale(qd.get("locale"))
        country = qd.get('country')
        deep_sight = environ.get("DEEP_SIGHT")

        try:
            scope_list = self.scope_info[locale]
        except KeyError:
            # not built yet, let's do it
            scope_info = []
            for item in self.scope_info[None]:
                loc = item.copy()
                loc['description'] = translator(item['description'], locale)
                loc['name'] = translator(item['name'], locale)
                scope_info.append(loc)
            self.scope_info[locale] = scope_list = scope_info

        # remove hidden scopes, if should
        if config['hide_scopes']:
            scope_list = [x for x in scope_list
                          if not x['hidden'] or x['hidden'] == deep_sight]

        # Remove blacklisted scopes
        country_blacklist = config['blacklist'].get(country)
        if country_blacklist:
            curated_list = self.curate_list(country_blacklist, scope_list)
        else:
            curated_list = scope_list
        json_list = json.dumps(curated_list)
        return self._generic_response("200 OK", start_response, json_list, Vary='User-Agent')

    def serve_static(self, environ, start_response, path):
        """Serve static files from inside the project."""
        filepath = os.path.join(os.environ["ROOTDIR"], path[1:])
        if not os.access(filepath, os.R_OK):
            return self._error("404 NOT FOUND", environ, start_response)
        img_size = os.stat(filepath).st_size

        def stream():
            """Generator to return the data streamed."""
            with open(filepath, 'rb') as fh:
                while True:
                    chunk = fh.read(4096)
                    if not chunk:
                        break
                    yield chunk

        response_headers = [
            ('Content-Length', str(img_size)),
        ]
        start_response('200 OK', response_headers)
        return stream()
