# Copyright 2014-2015 Canonical
# All Rights Reserved

"""A base Simple Application to provide common functionality to scopes.

When building a new builtin scope, just create a Python class (inheriting
this SimpleApp) and put it in a file in the scopes directory. Check the
src/scopes/wikipedia.py scope as a fully working example and the SimpleApp
docstring for more info.

Of course, if you want more complex needs and want to write the whole
stack, you can do it.
"""

import httplib
import logging
import urllib2
import urlparse
import json
import time

from .. import utils, rest_scopes

# some defaults for the requests
DEFAULT_LIMIT = 10
DEFAULT_COUNTRY = 'US'

# to separate the result chunks
CHUNK_SEPARATOR = '\r\n'

logger = logging.getLogger('restscopes')


class BadRequestError(Exception):
    """The received request from the client has missing/wrong information.

    This exception is useful to flag a problem with the received request
    inside a scope after it started serving it.
    """


class BackendDataError(Exception):
    """The response received from a backend is garbage.

    This normally is raised by scopes when failing to parse backend
    responses.
    """
    def __init__(self, reason, url):
        if isinstance(reason, Exception):
            self.msg = "%s('%s')" % (reason.__class__.__name__, reason)
        else:
            self.msg = reason
        self.url = url
        super(BackendDataError, self).__init__(self.msg)


class SimpleApp(object):
    """A simple wsgi app that serves the two mandatory endpoints.

    Subclass at will, feel free to not use it at all if have more
    complex needs.

    Then overwrite the 'search' and 'preview' methods. These will be
    automatically called, on requests. Both methods will receive the request
    parameters; 'search' needs to 'yield' the categories/results/etc,
    and 'preview' needs to 'yield' the different previews. The
    specification of what these methods will receive and what they
    must yield/return is in FIXME.
    """

    surface_headers = [('Content-type', 'application/json')]

    def __init__(self, config, metrics=None, oopser=None):
        self.config = config
        self.default_limit = DEFAULT_LIMIT
        self.scope_module_name = self.__class__.__module__.split('.')[-1]
        if metrics is None:
            self._meter = self._metric_prefix = None
        else:
            self._meter = metrics[0]
            self._metric_prefix = metrics[1] + '.' + self.scope_module_name
        self.oopser = oopser

    def urlread(self, url, data=None, headers=None):
        """Centralized urlopen/read with extra functionality."""
        return rest_scopes.urlread(self._meter, self._metric_prefix, url, data, headers)

    def error(self, status, start_response, error_msg=None):
        """Answer an error to the client."""
        data = {"error": status}
        if error_msg is not None:
            data['error_msg'] = error_msg
        message = json.dumps(data, indent=4)

        response_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(message))),
        ]
        start_response(status, response_headers)
        return [message]

    def _get_search_args(self, environ):
        """Get the parameters for the 'search', checking all is ok."""
        # get the query string, decode, build the query dictionary
        qs = environ.get("QUERY_STRING", "")
        qd = dict(urlparse.parse_qsl(qs, keep_blank_values=True))

        # same for the user agent
        useragent = environ.get("USER_AGENT", {})

        # latitude and longitude must be paired
        lat = float(qd['latitude']) if 'latitude' in qd else None
        lng = float(qd['longitude']) if 'longitude' in qd else None
        if lat is None and lng is not None or lng is None and lat is not None:
            msg = "latitude and longitude must be paired, both present or none"
            raise ValueError(msg)

        # get a valid locale
        locale = utils.sanitize_locale(qd.get('locale'))

        # dissasemble the filters, if present
        filters = qd.get('filters')
        if filters is not None:
            filters = json.loads(urlparse.unquote(filters))

        try:
            x_fwd_for = environ['HTTP_X_FORWARDED_FOR'].split(",")
            client_ip = x_fwd_for[0].strip()
        except Exception:
            # handle all errors, we don't want to fail if there is no IP
            client_ip = None
        # get all the rest of parameters, checking also type
        args = {
            'query': qd['q'].decode('utf8'),
            'session_id': environ.get("HTTP_X_REST_SCOPES_SESSION_ID"),
            'query_id': int(qd['query_id']) if 'query_id' in qd else None,
            'locale': locale,
            'platform': qd['platform'],
            'country': qd.get('country', DEFAULT_COUNTRY),
            'latitude': lat,
            'longitude': lng,
            'limit': int(qd.get('limit', self.default_limit)),
            'client_ip': client_ip,
            'department': qd.get('department'),
            'filters': filters,
            'partner': useragent.get('partner'),
        }
        return args

    def _get_preview_args(self, environ):
        """Get the parameters for the 'preview', checking all is ok."""
        # get the query string, decode, build the query dictionary
        qs = environ.get("QUERY_STRING", "")
        qd = dict(urlparse.parse_qsl(qs))

        # same for the user agent
        useragent = environ.get("USER_AGENT", {})

        # get a valid locale
        locale = utils.sanitize_locale(qd.get('locale'))

        try:
            x_fwd_for = environ['HTTP_X_FORWARDED_FOR'].split(",")
            client_ip = x_fwd_for[0].strip()
        except Exception:
            # handle all errors, we don't want to fail if there is no IP
            client_ip = None
        # get all the rest of parameters, checking also type
        args = {
            'session_id': environ.get("HTTP_X_REST_SCOPES_SESSION_ID"),
            'locale': locale,
            'platform': qd['platform'],
            'country': qd.get('country', DEFAULT_COUNTRY),
            'widgets_api_version': int(qd['widgets_api_version']),
            'result': json.loads(qd['result']),
            'client_ip': client_ip,
            'partner': useragent.get('partner'),
        }
        return args

    def search(self, **kwargs):
        """To be overwritten by subclass.

        Will receive the request parameters, needs to 'yield'
        the categories/results/etc.
        """
        raise NotImplementedError("Need to overwrite 'search' method.")

    def preview(self, **kwargs):
        """To be overwritten by subclass.

        Will receive the request parameters; needs to 'yield' the
        columns info and widgets.
        """
        raise NotImplementedError("Need to overwrite 'preview' method.")

    def _send_metric(self, name, tstart):
        """Send the metric, if available."""
        if self._meter is not None:
            delta_t = time.time() - tstart
            self._meter.timing(self._metric_prefix + '.' + name, delta_t)

    def __call__(self, environ, start_response):
        """Serve the request."""
        tini = time.time()
        path = environ["PATH_INFO"]

        # select the methods to serve the request
        meter_path = path[1:]
        if path == '/search':
            args_checker = self._get_search_args
            serving_method = self.search
            log_msg = ("Start search [%(scope)s]: session_id=%(session_id)r "
                       "locale=%(locale)r country=%(country)s "
                       "partner=%(partner)s query=%(query)r")
        elif path == '/preview':
            args_checker = self._get_preview_args
            serving_method = self.preview
            log_msg = ("Start preview [%(scope)s]: session_id=%(session_id)r "
                       "locale=%(locale)r country=%(country)s "
                       "partner=%(partner)s")
        else:
            self._send_metric('404', tini)
            return self.error("404 NOT FOUND", start_response)

        # get parameters
        try:
            args = args_checker(environ)
        except ValueError as err:
            msg = "Problem in parameters: " + str(err)
            self._send_metric(meter_path + '.400', tini)
            return self.error("400 BAD REQUEST", start_response, msg)
        except KeyError as err:
            msg = "Missing parameter " + str(err)
            self._send_metric(meter_path + '.400', tini)
            return self.error("400 BAD REQUEST", start_response, msg)

        if path == '/search' and args['query'] == '':
            headers = self.surface_headers[:]
        else:
            headers = [('Content-type', 'application/json')]

        # just return empty if we receive a silly query
        if path == '/search' and utils.avoidable_query(args['query']):
            logger.info("Avoiding query %r", args['query'])
            self._send_metric(meter_path + '.200', tini)
            start_response("200 OK", [])
            return [""]

        log_args = dict(scope=self.scope_module_name)
        log_args.update(args)
        logger.info(log_msg % log_args)

        def chunk_generator():
            """Send encoded chunks to the user."""
            error_code = None
            first = True
            try:
                for result in serving_method(**args):
                    if first:
                        first = False
                        # first chunk to send, start the response ok
                        start_response("200 OK", headers)
                    msg = json.dumps(result)
                    yield msg + CHUNK_SEPARATOR
            except urllib2.HTTPError as exc:
                if exc.code < 499 and exc.code != 429:
                    raise
                error_code = str(exc.code)
                error_message = str(exc)
            except urllib2.URLError as exc:
                error_code = 'urlerror'
                error_message = "URLError: " + str(exc.reason)
            except httplib.HTTPException as exc:
                error_code = 'httpexception'
                error_message = "HTTPException: " + str(exc)
            except BadRequestError as exc:
                msg = "Bad request: " + str(exc)
                self._send_metric(meter_path + '.400', tini)
                yield self.error("400 BAD REQUEST", start_response, msg)[0]
                return
            except BackendDataError as exc:
                self._send_metric(meter_path + '.backend-data-error', tini)
                logger.debug("Backend data error for %r: %s", exc.url, exc.msg)

            if error_code:
                if first:
                    # nothing sent before, start the response with the error
                    msg = "Original error: " + error_message
                    yield self.error("502 BAD GATEWAY", start_response, msg)[0]
                self._send_metric(meter_path + '.' + error_code, tini)
            else:
                if first:
                    # nothing sent before, send an empty 200
                    start_response("200 OK", [])
                self._send_metric(meter_path + '.200', tini)

        return chunk_generator()
