# Copyright 2014-2015 Canonical
# All Rights Reserved

"""The eBay scope."""

import urllib
import json
import re
import logging

from urllib import urlencode
from random import choice
from babel import numbers

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category
from ..utils import markup_cleaner

logger = logging.getLogger("restscopes.ebay")

SEARCH_URI = ('https://svcs.ebay.com/services/search/FindingService/v1'
              '?SECURITY-APPNAME=%(key)s&OPERATION-NAME=findItemsByKeywords'
              '&SERVICE-VERSION=1.0.0&RESPONSE-DATA-FORMAT=JSON'
              '&GLOBAL-ID=%(site)s&REST-PAYLOAD'
              '&keywords=%(query)s&paginationInput.entriesPerPage=%(limit)s'
              '&outputSelector(0)=PictureURLSuperSize'
              '&outputSelector(1)=PictureURLLarge'
              '&affiliate.networkId=9&affiliate.trackingId=%(campaign)s')

SORTING_URI = ('https://svcs.ebay.com/services/search/FindingService/v1'
               '?SECURITY-APPNAME=%(key)s'
               '&OPERATION-NAME=findItemsByCategory'
               '&SERVICE-VERSION=1.0.0&RESPONSE-DATA-FORMAT=JSON'
               '&GLOBAL-ID=%(site)s&REST-PAYLOAD'
               '&categoryId=%(dep)s&sortOrder=%(filters)s'
               '&paginationInput.entriesPerPage=%(limit)s'
               '&outputSelector(0)=PictureURLSuperSize'
               '&outputSelector(1)=PictureURLLarge'
               '&itemFilter.name=ListingType&itemFilter.value=Auction'
               '&affiliate.networkId=9&affiliate.trackingId=%(campaign)s')

SURFACING_URI = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
                 '&responseencoding=JSON&appid=%(key)s&siteid=%(site)s'
                 '&version=531&CategoryID=%(dep)s&MaxEntries=%(limit)s'
                 '&trackingpartnercode=9&trackingid=%(campaign)s')

DEPINFO_URI = ('http://open.api.ebay.com/shopping?callname=GetCategoryInfo'
               '&responseencoding=JSON&appid=%(key)s&siteid=%(site)s'
               '&version=531&CategoryID=%(dep)s'
               '&IncludeSelector=ChildCategories')

ITEMINFO_URI = ('http://open.api.ebay.com/shopping?callname=GetSingleItem'
                '&responseencoding=JSON&appid=%(key)s'
                '&ItemID=%(item)s&version=531'
                '&IncludeSelector=Details,Description,ShippingCosts')

SITE_IDS = {
    'AT': {'de_AT': (16, 'EBAY-AT')},
    'AU': {'en_AU': (15, 'EBAY-AU')},
    'CH': {'de_CH': (193, 'EBAY-CH')},
    'DE': {'de_DE': (77, 'EBAY-DE')},
    'CA': {'en_CA': (2, 'EBAY-ENCA'),
           'fr_CA': (210, 'EBAY-FRCA')},
    'ES': {'es_ES': (186, 'EBAY-ES')},
    'FR': {'fr_FR': (71, 'EBAY-FR')},
    'BE': {'nl_BE': (123, 'EBAY-NLBE'),
           'fr_BE': (23, 'EBAY-FRBE')},
    'GB': {'en_GB': (3, 'EBAY-GB')},
    'HK': {'zh_HK': (201, 'EBAY-HK')},
    'IE': {'en_IE': (205, 'EBAY-IE')},
    'IN': {'en_IN': (203, 'EBAY-IN')},
    'IT': {'it_IT': (101, 'EBAY-IT')},
    'MY': {'en_MY': (207, 'EBAY-MY')},
    'NL': {'nl_NL': (146, 'EBAY-NL')},
    'PH': {'en_PH': (211, 'EBAY-PH')},
    'PL': {'pl_PL': (212, 'EBAY-PL')},
    'SG': {'en_SG': (216, 'EBAY-SG')},
    'US': {'en_US': (0, 'EBAY-US')},
    }

FILTERS = {'Condition', 'MaxPrice', 'MinPrice', 'SellerBusinessType',
           'EndTimeTo', 'TopRatedSellerOnly'}

SORTING_ID = 'sorting_filter'
SORTING_FEATURED = '__featured__'
SORTING_OPTIONS = [
    {"id": SORTING_FEATURED, "label": translator("Featured")},
    {"id": "BestMatch", "label": translator("Best Match")},
    {"id": "BidCountFewest", "label": translator("Bid count fewest")},
    {"id": "BidCountMost", "label": translator("Bid count most")},
    {"id": "CurrentPriceHighest",
     "label": translator("Current price highest")},
    {"id": "EndTimeSoonest", "label": translator("End time soonest")},
    {"id": "PricePlusShippingHighest",
     "label": translator("Price plus shipping highest")},
    {"id": "PricePlusShippingLowest",
     "label": translator("Price plus shipping lowest")},
    {"id": "StartTimeNewest", "label": translator("Start time newest")}
]


# Override category renderer
RENDERER_LIST = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'large',
                 'card-layout': 'horizontal',
                 'collapsed-rows': 0},
    'components': {'title': 'title',
                   'art': 'art',
                   'attributes': {'field': 'attributes', 'max-count': 4}},
}
PRODUCTS_CATEGORY = {
    'id': 'cat_0',
    'title': None,
    'render_template': json.dumps(RENDERER_LIST),
}

RENDERER_SURFACING = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'small',
                 'collapsed-rows': 0},
    'components': {'title': 'title',
                   'art': 'art',
                   'subtitle': 'price',
                   'attributes': {'field': 's_attributes', 'max-count': 1}},
}
SURFACING_CATEGORY = {
    'id': 'cat_0',
    'title': None,
    'render_template': json.dumps(RENDERER_SURFACING),
}


def _get_svcs_api_content(json_response, call):
    """Get search content from response."""
    results = []
    search_results = json_response[call][0].get('searchResult')
    if search_results:
        count = int(search_results[0]['@count'])
        if count > 0:
            results = search_results[0]['item']
    try:
        cnt = json_response[call][0]['paginationOutput'][0]['totalEntries'][0]
    except KeyError:
        total_count = None
    else:
        total_count = int(cnt)
    return (results, total_count)


def _parse_searched_item(item, locale):
    """Parse searched items."""
    res = {}
    res['item_id'] = item['itemId'][0]
    res['icon_hint'] = item.get('galleryURL', [None])[0]
    if item.get('pictureURLSuperSize'):
        res['image_hint'] = item['pictureURLSuperSize'][0]
    elif item.get('pictureURLLarge'):
        res['image_hint'] = item['pictureURLLarge'][0]
    else:
        res['image_hint'] = res['icon_hint']
    res['title'] = item['title'][0]
    res['uri'] = item['viewItemURL'][0]
    if 'condition' in item:
        res['condition'] = item['condition'][0]['conditionDisplayName'][0]
    else:
        res['condition'] = None
    res['shipping_to'] = ', '.join(item[
        'shippingInfo'][0]['shipToLocations'])
    try:
        res['payment_method'] = ', '.join(item['paymentMethod'])
    except KeyError:
        res['payment_method'] = None
    res['currency_code'] = item[
        'sellingStatus'][0]['currentPrice'][0]['@currencyId']
    price = item[
        'sellingStatus'][0]['currentPrice'][0]['__value__']
    res['price'] = numbers.format_currency(float(price),
                                           res['currency_code'],
                                           locale=locale)
    try:
        res['time_left'] = item['sellingStatus'][0]['timeLeft'][0]
    except KeyError:
        logger.warning("timeLeft key missing on searched item: %r",
                       res['item_id'])
        res['time_left'] = None
    try:
        bid_count = item['sellingStatus'][0]['bidCount'][0]
    except KeyError:
        res['bid_count'] = None
    else:
        if bid_count == 1:
            bid_count_str = translator("%s bid" % (bid_count), locale)
        else:
            bid_count_str = translator("%s bids" % (bid_count), locale)
        res['bid_count'] = bid_count_str
    try:
        shipping_cost = item[
            'shippingInfo'][0]['shippingServiceCost'][0]['__value__']
    except KeyError:
        logger.warning("shippingServiceCost key missing on searched item: %r",
                       res['item_id'])
        res['shipping'] = None
    else:
        shipping_currency = item[
            'shippingInfo'][0]['shippingServiceCost'][0]['@currencyId']
        if float(shipping_cost) == 0:
            res['shipping'] = translator('Free', locale)
        else:
            res['shipping'] = numbers.format_currency(float(shipping_cost),
                                                      shipping_currency,
                                                      locale=locale)
    return res


def _parse_surfaced_item(item, locale):
    """Parse surfaced items."""
    try:
        s_thumb = item['GalleryURL']
    except KeyError:
        return None
    res = {}
    res['title'] = item['Title']
    res['item_id'] = item['ItemID']
    # The PopularItems API doesn't provide thumbnails larger than 80px
    # but we can get them using the item id from the returned thumb
    thumb = s_thumb.replace('8080_', '_').replace('/pict/', '/d/l150/pict/')
    l_thumb = thumb.replace('/l150/', '/l300/')
    res['icon_hint'] = thumb
    res['image_hint'] = l_thumb
    res['uri'] = item['ViewItemURLForNaturalSearch']
    res['currency_code'] = item['ConvertedCurrentPrice']['CurrencyID']
    res['price'] = numbers.format_currency(
        item['ConvertedCurrentPrice']['Value'], res['currency_code'],
        locale=locale)
    res['payment_method'] = None
    res['shipping_to'] = None
    res['condition'] = None
    try:
        bid_count = item['BidCount']
    except KeyError:
        res['bid_count'] = None
    else:
        if bid_count == 1:
            bid_count_str = translator("%d bid" % (bid_count), locale)
        else:
            bid_count_str = translator("%d bids" % (bid_count), locale)
        res['bid_count'] = bid_count_str
    try:
        shipping_cost = item[
            'ShippingCostSummary']['ShippingServiceCost']['Value']
    except KeyError:
        logger.warning("ShippingServiceCost key missing on searched item: %r",
                       res['item_id'])
        res['shipping'] = None
    else:
        shipping_currency = item[
            'ShippingCostSummary']['ShippingServiceCost']['CurrencyID']
        if shipping_cost == 0:
            res['shipping'] = translator('Free', locale)
        else:
            res['shipping'] = numbers.format_currency(shipping_cost,
                                                      shipping_currency,
                                                      locale=locale)
    try:
        res['time_left'] = item['TimeLeft']
    except KeyError:
        logger.warning("TimeLeft key missing on surfacing item: %r",
                       res['item_id'])
        res['time_left'] = None
    return res


def _parse_department_item(item, locale):
    """Parse department items."""
    res = {}
    res['title'] = item['Title']
    res['item_id'] = item['ItemID']
    s_thumb = item['GalleryURL']
    # The PopularItems API doesn't provide thumbnails larger than 80px
    # but we can get them using the item id from the returned thumb
    thumb = s_thumb.replace('8080_', '_').replace('/pict/', '/d/l150/pict/')
    l_thumb = thumb.replace('/l150/', '/l300/')
    res['icon_hint'] = thumb
    res['image_hint'] = l_thumb
    res['uri'] = item['ViewItemURLForNaturalSearch']
    res['currency_code'] = item['ConvertedCurrentPrice']['CurrencyID']
    res['price'] = numbers.format_currency(
        item['ConvertedCurrentPrice']['Value'], res['currency_code'],
        locale=locale)
    res['payment_method'] = None
    res['shipping_to'] = None
    res['condition'] = None
    try:
        bid_count = item['bidCount']
    except KeyError:
        res['bid_count'] = None
    else:
        if bid_count == 1:
            bid_count_str = translator("%d bid" % (bid_count), locale)
        else:
            bid_count_str = translator("%d bids" % (bid_count), locale)
        res['bid_count'] = bid_count_str
    res['shipping'] = ''
    res['cat_id'] = PRODUCTS_CATEGORY['id']
    try:
        res['time_left'] = item['TimeLeft']
    except KeyError:
        logger.warning("TimeLeft key missing on department item: %r",
                       res['item_id'])
        res['time_left'] = None
    return res


def _parse_filters(filters):
    """Parse and validate filters."""
    # we're building a list and then concatenating it with '&'
    # (instead of just urlencoding a big dict) because order
    # is important for ebay
    filters_parsed = []
    nb_filters = 0
    for filtr in filters:
        filters_args = {}
        if filtr in FILTERS:
            filter_name = 'itemFilter(%i).name' % nb_filters
            filters_args[filter_name] = filtr
            filters_parsed.append(urlencode(filters_args))
            if isinstance(filters[filtr], list):
                for nb_opt, opt in enumerate(filters[filtr]):
                    options_args = {}
                    option_name = 'itemFilter(%i).value(%i)' % (nb_filters,
                                                                nb_opt)
                    options_args[option_name] = opt
                    filters_parsed.append(urlencode(options_args))
            else:
                option_name = 'itemFilter(%i).value(0)' % (nb_filters)
                options_args = {option_name: filters[filtr]}
                filters_parsed.append(urlencode(options_args))
        nb_filters += 1
    return "&".join(filters_parsed)


def _build_canned(dep=None, filters=None):
    """Build a canned query for the department tree."""
    params = []
    if dep:
        params.append('dep=%s' % (dep,))
    if filters:
        params.append('filters=%s' % (filters,))
    if params:
        cq = 'scope://com.canonical.scopes.ebay?' + '&'.join(params)
    else:
        cq = 'scope://com.canonical.scopes.ebay'
    return cq


def _get_localized_filters(locale):
    """Localize and build the filters structure"""
    options = []
    sorting_filters = [
        {"filter_type": "option_selector",
         "display_hints": "primary",
         "id": SORTING_ID,
         "label": "Sorting",
         "multi_select": False,
         "options": options,
         }
    ]
    for option in SORTING_OPTIONS:
        options.append({
            "id": option['id'],
            "label": translator(option['label'], locale),
        })
    return sorting_filters


def _get_item_details(item, locale):
    """Get preview details for a single product"""
    details = {'id': item}
    details['seller'] = item['Seller']
    details['description'] = item.get('Description')
    details['gallery'] = item.get('PictureURL', None)
    details['time_left'] = _parse_time_left(item.get('TimeLeft'), locale)
    details['ships_to'] = ', '.join(item.get('ShipToLocations', []))
    details['location'] = item.get('Location')
    details['return_policy'] = item.get('ReturnPolicy')
    details['handling_time'] = item.get('HandlingTime')
    details['shipping_cost'] = item.get('ShippingCostSummary')
    details['payment_methods'] = ', '.join(item.get('PaymentMethods', []))
    return details


def _parse_time_left(tl_iso, locale):
    """"Get a string from ebay's take on ISO8601"""
    if not tl_iso:
        return None
    time_left_regex = re.compile('(?P<sign>-?)P(?:(?P<years>\d+)Y)?'
                                 '(?:(?P<months>\d+)M)?(?:(?P<days>\d+)D)?'
                                 '(?:T(?:(?P<hours>\d+)H)?'
                                 '(?:(?P<minutes>\d+)M)?'
                                 '(?:(?P<seconds>\d+)S)?)?')
    # Fetch the match groups with default value of 0 (not None)
    match = time_left_regex.match(tl_iso)
    if not match:
        logger.warning("Remaining time not parsed ok: %r", tl_iso)
        return None
    duration = match.groupdict(0)
    days = int(duration['days']) + int(duration['months']) * 30 + int(duration['years'] * 365)
    hours = int(duration['hours'])
    minutes = int(duration['minutes'])

    if days > 0 and hours > 0:
        # move around the {days_marker} and {hours_marker} to accomodate days/hours
        # to the order needed in the language, but leave them as-is; you can change
        # the rest of the string as much as you want
        tpl = translator("{days_marker}d {hours_marker}h", locale)
        formatted_time = tpl.format(days_marker=days, hours_marker=hours)

    elif days > 0 and hours == 0:
        # move around the {days_marker} to accomodate days
        # to the order needed in the language, but leave it as-is; you can change
        # the rest of the string as much as you want
        tpl = translator("{days_marker}d", locale)
        formatted_time = tpl.format(days_marker=days)

    elif days == 0 and hours > 0 and minutes > 0:
        # move around the {hours_marker} and {minutes_marker} to accomodate hours/minutes
        # to the order needed in the language, but leave them as-is; you can change
        # the rest of the string as much as you want
        tpl = translator("{hours_marker}h {minutes_marker}m", locale)
        formatted_time = tpl.format(hours_marker=hours, minutes_marker=minutes)

    elif days == 0 and hours > 0 and minutes == 0:
        # move around the {hours_marker} to accomodate hours
        # to the order needed in the language, but leave it as-is; you can change
        # the rest of the string as much as you want
        tpl = translator("{hours_marker}h", locale)
        formatted_time = tpl.format(hours_marker=hours)

    else:
        # move around the {minutes_marker} to accomodate minutes
        # to the order needed in the language, but leave it as-is; you can change
        # the rest of the string as much as you want
        tpl = translator("{minutes_marker}m", locale)
        formatted_time = tpl.format(minutes_marker=minutes)

    return formatted_time


def _build_surfacing_category(label, locale):
    """Build a category to host surfacing results"""
    category = SURFACING_CATEGORY.copy()
    category['title'] = label
    return category


def _build_shipping_and_postage_text(details, locale):
    """"Build the Shipping and Postage text of the preview"""
    shipping_and_postage_info = []

    # Build postage info text
    postage_info = [translator('Postage', locale)]
    if details['location']:
        item_loc_str = translator('Item location: %s', locale)
        postage_info.append(item_loc_str % (details['location'],))
    if details['ships_to']:
        item_posts_str = translator('Ships to: %s', locale)
        postage_info.append(item_posts_str % (details['ships_to'],))
    if len(postage_info) > 1:
        postage_str = '\n'.join(postage_info)
        shipping_and_postage_info.append(postage_str)

    # Build packaging info text
    packaging_info = [translator('Packaging', locale)]
    if details['handling_time']:
        item_dispatch_str = translator('Dispatch time: %s business day',
                                       locale)
        packaging_info.append(item_dispatch_str % (details['handling_time'],))
    if len(packaging_info) > 1:
        packaging_str = '\n'.join(packaging_info)
        shipping_and_postage_info.append(packaging_str)

    # Build payment details text
    payment_info = [translator('Payment details', locale)]
    if details['payment_methods']:
        item_payment_str = translator('Payment method: %s', locale)
        payment_info.append(item_payment_str % (details['payment_methods'],))
    if len(payment_info) > 1:
        payment_str = '\n'.join(payment_info)
        shipping_and_postage_info.append(payment_str)

    # Build returns policy info text
    returns_info = [translator('Returns policy', locale)]
    if details['return_policy']:

        if details['return_policy'].get('ReturnsAccepted'):
            policy_str = translator('Policy: %s', locale)
            returns_info.append(policy_str % (
                details['return_policy']['ReturnsAccepted'].capitalize(),))

        if details['return_policy'].get('Refund'):
            refunds_str = translator('Refund: %s', locale)
            returns_info.append(refunds_str % (
                details['return_policy']['Refund'].capitalize(),))

        if details['return_policy'].get('ReturnsWithin'):
            returns_within_str = translator('Returns within: %s', locale)
            returns_info.append(returns_within_str % (
                details['return_policy']['ReturnsWithin'].capitalize(),))

        if details['return_policy'].get('ShippingCostsPaidBy'):
            shipping_paid_str = translator('Shipping paid by: %s', locale)
            returns_info.append(shipping_paid_str % (
                details['return_policy']['ShippingCostsPaidBy'].capitalize(),))

        if details['return_policy'].get('Description'):
            returns_info.append('\n%s' % (
                details['return_policy']['Description'],))

    if len(returns_info) > 1:
        returns_str = '\n'.join(returns_info)
        shipping_and_postage_info.append(returns_str)

    return '\n\n'.join(shipping_and_postage_info)


def _build_counted_cat(cat, locale, count):
    """Build a category with the number of results in the title."""
    category = cat.copy()
    if not count:
        category['title'] = translator('Results', locale)
    elif count == 1:
        category['title'] = translator('1 result', locale)
    else:
        category['title'] = translator('%d results', locale) % (count,)
    return category


class App(SimpleApp):
    """The eBay app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self.auth = self.config['secrets']

    def _get_products(self, url):
        """Call the eBay API."""
        response = self.urlread(url)
        results = response.decode('utf8')
        return results

    def _get_available_departments(self, dep, site, key, locale, filters=None):
        """Get details, parents and children for the current department"""
        if not dep:
            dep = '-1'
        url = DEPINFO_URI % dict(key=key, dep=dep, site=site)
        results = self._get_products(url)
        parsed_results = json.loads(results)
        current = {}
        parent = {}
        parent['subdepartments'] = []
        current['subdepartments'] = []
        select_one = []
        encfilt = urllib.quote(json.dumps(filters)) if filters else None
        for d in parsed_results['CategoryArray']['Category']:
            if d['CategoryID'] == dep:
                current['canned_query'] = _build_canned(filters=encfilt)
                if dep == '-1':
                    current['label'] = translator('All departments', locale)
                else:
                    current['label'] = d['CategoryName']
                    current['canned_query'] = _build_canned(dep=dep, filters=encfilt)
                if d['CategoryParentID'] != '0':
                    parent['canned_query'] = _build_canned(filters=encfilt)
                    if d['CategoryParentID'] == '-1':
                        parent['label'] = translator('All departments', locale)
                    else:
                        parent['label'] = d['CategoryNamePath'].split(':')[-2]
                        parent['canned_query'] = _build_canned(dep=d['CategoryParentID'],
                                                               filters=encfilt)
                    parent['subdepartments'].append(current)
                else:
                    parent = current
            else:
                sub = {}
                sub['label'] = d['CategoryName']
                sub['canned_query'] = _build_canned(dep=d['CategoryID'], filters=encfilt)
                sub['has_subdepartments'] = not d['LeafCategory']
                current['subdepartments'].append(sub)
                select_one.append((d['CategoryID'], d['CategoryName']))

        selected = choice(select_one) if select_one else None
        return parent, selected

    def search(self, **kwargs):
        """Do the search."""
        limit = kwargs['limit']
        locale = kwargs['locale']
        country = kwargs.get('country')
        query = urllib.quote(kwargs['query'].encode('utf-8'))
        department = kwargs.get('department')
        auth = self.auth[kwargs['partner']]
        filters = kwargs['filters']
        if filters:
            filters_args = filters.get(SORTING_ID, [None])[0]
            filters_state = {SORTING_ID: [filters_args]}
        else:
            filters_args = ''
            filters_state = None

        results = []
        shop_loc = SITE_IDS.get(country.upper(), SITE_IDS['US'])
        site_id, global_id = shop_loc.get(locale, shop_loc.itervalues().next())
        if not query.strip():
            # Surfacing API calls
            available_deps, selected_dep = self._get_available_departments(
                department, site_id, auth["api_key"], locale,
                filters=filters_state)
            yield dict(departments=available_deps)
            if not department:
                department, dep_label = selected_dep
                category = _build_surfacing_category(dep_label, locale)
            else:
                yield dict(filters=_get_localized_filters(locale))
                if filters_state is None:
                    yield dict(filter_state={SORTING_ID: [SORTING_FEATURED]})
                else:
                    yield dict(filter_state=filters_state)
                category = localized_category(PRODUCTS_CATEGORY, locale)

            if filters_args and filters_args != SORTING_FEATURED:
                # One department and a useful sorting filter
                search_url = SORTING_URI % dict(
                    key=auth["api_key"], limit=limit, dep=department,
                    filters=filters_args, site=global_id,
                    campaign=auth['campaign'])
                json_response = json.loads(self._get_products(search_url))
                parsed_response, total_count = _get_svcs_api_content(
                    json_response, "findItemsByCategoryResponse")
                for item in parsed_response:
                    results.append(_parse_searched_item(item, locale))
            else:
                # One department and no sorting filter
                search_url = SURFACING_URI % dict(
                    key=auth["api_key"], limit=limit, dep=department,
                    site=site_id, campaign=auth['campaign'])
                json_response = json.loads(self._get_products(search_url))
                item_array = json_response.get('ItemArray')
                if item_array is not None:
                    for item in item_array['Item']:
                        results.append(_parse_surfaced_item(item, locale))
        else:
            # with query no department/sorting is used
            search_url = SEARCH_URI % dict(
                key=auth["api_key"], query=query, limit=limit,
                site=global_id, campaign=auth['campaign'])
            json_response = json.loads(self._get_products(search_url))
            parsed_response, total_count = _get_svcs_api_content(
                json_response, "findItemsByKeywordsResponse")
            category = _build_counted_cat(PRODUCTS_CATEGORY,
                                          locale=locale, count=total_count)
            for item in parsed_response:
                results.append(_parse_searched_item(item, locale))

        # send the category if have any results, then those
        if results:
            yield dict(category=category)
        for res in results:
            if res:
                time_hint = _parse_time_left(res['time_left'], locale)
                yield dict(result={
                    'uri': res['uri'],
                    'dnd_uri': res['uri'],
                    'art': res['icon_hint'],
                    'cat_id': category['id'],
                    'title': res['title'],
                    'currency_code': res['currency_code'],
                    'price': res['price'],
                    'condition': res['condition'],
                    'shipping_to': res['shipping_to'],
                    'payment_methods': res['payment_method'],
                    'image_hint': res['image_hint'],
                    'item_id': res['item_id'],
                    'attributes': [{'value': res['price']},
                                   {'value': res['bid_count']},
                                   {'value': res['shipping']},
                                   {'value': time_hint}],
                    's_attributes': [{'value': time_hint}],
                })

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']
        auth = self.auth[kwargs['partner']]
        details_url = ITEMINFO_URI % dict(key=auth["api_key"],
                                          item=result['item_id'])
        json_response = json.loads(self._get_products(details_url))
        details = _get_item_details(json_response['Item'], locale)
        # yield a 'gallery' widget
        if details['gallery']:
            yield dict(widget={
                'id': 'imagegallery',
                'type': 'gallery',
                'sources': details['gallery'],
            })
        if result['price']:
            price_str = translator('Current bid: %s', locale)
        else:
            price_str = None
        shipping_cost = None
        if details['shipping_cost']:
            if details['shipping_cost'].get('ShippingServiceCost'):
                sc_details = details['shipping_cost']['ShippingServiceCost']
                shipping_cost_str = translator('Shipping: %s', locale)
                shipping_cost_val = numbers.format_currency(
                    float(sc_details['Value']),
                    sc_details['CurrencyID'],
                    locale=locale)
                if sc_details['Value'] > 0:
                    shipping_cost = shipping_cost_str % (shipping_cost_val,)
                else:
                    shipping_cost = translator('Free', locale)
        # yield a 'header' widget
        yield dict(widget={
            'id': 'header',
            'type': 'header',
            'title': result['title'],
            'subtitle': price_str % (result['price'],),
            'attributes': [{'value': shipping_cost}],
        })

        # add an 'actions' widget
        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'label': translator('View on eBay', locale),
                         'uri': result['uri'],
                         'id': 'action1'}]
        })
        seller_name_str = translator('Sold by %(name)s (%(fb)i)', locale)
        seller_name_text = seller_name_str % dict(
            name=details['seller']['UserID'],
            fb=details['seller']['FeedbackScore'])
        seller_fb_str = translator('Feedback: %(fb_p).1f%% positive', locale)
        seller_fb_text = seller_fb_str % dict(
            fb_p=details['seller']['PositiveFeedbackPercent'])
        info_details = []
        if details['time_left']:
            time_left_str = translator('Time left: %s', locale)
            info_details.append(time_left_str % (details['time_left'],))
        if details['location']:
            item_loc_str = translator('Item location: %s', locale)
            info_details.append(item_loc_str % (details['location'],))
        if len(details['ships_to']) < 1:
            ships_to_str = translator('Ships to: %s', locale)
            info_details.append(ships_to_str % (details['ships_to'],))
        if result['condition']:
            condition_str = translator('Condition: %s', locale)
            info_details.append(condition_str % (result['condition'],))
        info_text = '\n'.join(info_details)
        description = markup_cleaner(details['description']).replace(
            '&nbsp_place_holder', '')
        seller = '%(seller)s\n%(feedback)s' % dict(seller=seller_name_text,
                                                   feedback=seller_fb_text)
        # yield a 'text' widget
        yield dict(widget={
            'id': 'info',
            'type': 'text',
            'title': translator('Info', locale),
            'text': info_text,
        })
        # yield a 'text' widget
        yield dict(widget={
            'id': 'description',
            'type': 'text',
            'title': '',
            'text': description,
        })
        # yield a 'text' widget
        yield dict(widget={
            'id': 'seller',
            'type': 'text',
            'title': '',
            'text': seller,
        })

        shipping_and_postage = _build_shipping_and_postage_text(details,
                                                                locale)
        # yield a 'text' widget
        yield dict(widget={
            'id': 'shipping_and_postage',
            'type': 'text',
            'title': translator('Shipping and postage', locale),
            'text': shipping_and_postage,
        })
