# Copyright 2014-2015 Canonical
# All Rights Reserved

"""The Reddit scope."""

import urllib
import json

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category

# this module is generated by a "make build"
from versioninfo import version_info


# the different URIs to get info from Reddit
POPULAR_SUBREDDITS = 'http://www.reddit.com/subreddits/popular.json?limit=50'
POST_SEARCH_URI = 'http://www.reddit.com/search.json?q=%(query)s&limit=%(limit)i&show=all'
SUBREDDIT_SEARCH_URI = 'http://www.reddit.com/subreddits/search.json?q=%(query)s&limit=%(limit)i'
SURFACING_DEP_URI = 'http://www.reddit.com/r/%(dep)s.json?limit=%(limit)i'
SURFACING_URI = 'http://www.reddit.com/.json?limit=%(limit)i'

RENDERER_THREAD = {
    "schema-version": 1,
    "template": {
        "category-layout": "grid",
        "card-size": "large",
        "card-layout": "horizontal"},
    "components": {
        "title": "title",
        "mascot": "mascot",
        "subtitle": "subreddit",
        "attributes": "attributes"},
}

RENDERER_SUBREDDIT = {
    "schema-version": 1,
    "template": {
        "category-layout": "grid",
        "card-size": "medium",
        "card-layout": "vertical"
    },
    "components": {
        "subtitle": "subscribers",
        "title": "title"
    }
}

THREAD_CATEGORY = {
    'id': 'posts',
    'title': '',
    'render_template': json.dumps(RENDERER_THREAD),
    'icon': None,
}

S_THREAD_CATEGORY = {
    'id': 'posts',
    'title': translator('Posts'),
    'render_template': json.dumps(RENDERER_THREAD),
    'icon': None,
}

SUBREDDIT_CATEGORY = {
    'id': 'subreddits',
    'title': translator('Subreddits'),
    'render_template': json.dumps(RENDERER_SUBREDDIT),
    'icon': None,
}


def _build_uri(permalink, platform):
    """Build thread uri, mobile when appropriate"""
    uri = 'http://reddit.com%s' % (permalink,)
    # We assume unknown form-factors are mobile
    if platform and platform != "desktop":
        uri = "%s.compact" % (uri)
    return uri


def _parse_post(d, query, locale, platform):
    """Parse post data."""
    # We need both media info or neither to have proper previews.
    try:
        media_type = d['data']['media']['oembed']['type']
        media_preview = d['data']['media']['oembed']['thumbnail_url']
    except (TypeError, KeyError):
        media_type, media_preview = None, None
    if not d['data']['is_self']:
        art = d['data']['thumbnail']
    else:
        art = None
    if not art and media_preview:
        art = media_preview
    if d['data']['domain'] == 'i.imgur.com':
        if not d['data']['url'].endswith('.gif'):
            media_preview = d['data']['url']
            media_type = "rich"
    if d['data']['over_18'] is True and "#nsfw" not in query:
        art = None
        media_preview = None
    uri = _build_uri(d['data']['permalink'], platform)
    if d['data']['num_comments'] == 1:
        comments_str = translator('%i comment', locale)
    else:
        comments_str = translator('%i comments', locale)
    comments = comments_str % (d['data']['num_comments'],)
    post = {
        'uri': uri,
        'dnd_uri': uri,
        'mascot': art,
        'cat_id': 'posts',
        'title': d['data']['title'],
        'subreddit': d['data']['subreddit'],
        'author': d['data']['author'],
        'comments': comments,
        'domain': d['data']['domain'],
        'summary': d['data']['selftext'],
        'target': d['data']['url'],
        'media_type': media_type,
        'media_preview': media_preview,
        'attributes': [{'style': 'highlighted',
                        'value': str(d['data']['score'])},
                       {'style': 'default',
                        'value': comments}]
    }
    return post


def _parse_subreddit(d, query, locale):
    """Parse subreddit data."""
    subreddit = d['data']['display_name']
    cq = _get_canned_query(dep=subreddit)
    subscribers = d['data']['subscribers']
    if subscribers is None:
        subscribers_str = translator('no subscribers', locale)
    elif subscribers == 1:
        subscribers_str = translator('1 subscriber', locale)
    else:
        subscribers_str = translator('%i subscribers', locale) % (subscribers,)
    subreddit = {
        'uri': cq,
        'dnd_uri': d['data']['url'],
        'art': d['data']['header_img'],
        'cat_id': 'subreddits',
        'title': subreddit,
        'subreddit': subreddit,
        'summary': d['data']['public_description'],
        'header_title': d['data']['header_title'],
        'subscribers': subscribers_str,
    }
    return subreddit


def _get_canned_query(dep=None, filters=None):
    """Generate the canned query."""
    params = []
    if dep:
        params.append('dep=%s' % (dep,))
    if filters:
        params.append('filters=%s' % (filters,))
    if params:
        cq = 'scope://com.canonical.scopes.reddit?' + '&'.join(params)
    else:
        cq = 'scope://com.canonical.scopes.reddit'
    return cq


class App(SimpleApp):
    """The Reddit app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self.default_limit = 25

    def _get_results(self, url):
        """Call the Reddit API."""
        hdr = {'User-Agent': "Reddit Ubuntu Rest Scope " + str(version_info['revno'])}
        response = self.urlread(url, headers=hdr)
        results = json.loads(response.decode('utf-8'))
        return results

    def _get_available_departments(self, department, locale):
        """Use popular subreddits to build the departments list."""
        current = {}
        current['subdepartments'] = []
        current['canned_query'] = _get_canned_query()
        current['label'] = translator('Front page', locale)
        current['has_subdepartments'] = True
        # Departments for reddit are a flat list, always display them
        for r in self._get_results(POPULAR_SUBREDDITS)['data']['children']:
            sub = {}
            name = r['data']['display_name']
            sub['label'] = name
            sub['canned_query'] = _get_canned_query(dep=name)
            sub['has_subdepartments'] = False
            current['subdepartments'].append(sub)
        # If the user navigates to a non-default department via search
        # we put it at the top of the list
        if department:
            sub = {}
            sub['canned_query'] = _get_canned_query(dep=department)
            sub['label'] = department
            sub['has_subdepartments'] = False
            if sub not in current['subdepartments']:
                current['subdepartments'].insert(0, sub)
        return current

    def search(self, **kwargs):
        """Do the search."""
        limit = kwargs['limit']
        locale = kwargs['locale']
        platform = kwargs.get('platform')
        query = kwargs['query'].encode('utf-8')
        department = kwargs.get('department')
        if not query.strip() or len(query) < 3:
            available_deps = self._get_available_departments(department,
                                                             locale)
            yield dict(departments=available_deps)
            if department:
                uri = SURFACING_DEP_URI % dict(dep=department, limit=limit)
                yield dict(category=localized_category(THREAD_CATEGORY,
                                                       locale))
                for post in self._get_results(uri)['data']['children']:
                    yield dict(result=_parse_post(post, query,
                                                  locale, platform))
            else:
                uri = SURFACING_URI % dict(limit=limit)
                yield dict(category=localized_category(THREAD_CATEGORY,
                                                       locale))
                for post in self._get_results(uri)['data']['children']:
                    yield dict(result=_parse_post(post, query,
                                                  locale, platform))
        else:
            query_cleaned = urllib.quote(query.split('#')[0])

            # Search subreddits
            subs_uri = SUBREDDIT_SEARCH_URI % dict(query=query_cleaned,
                                                   limit=limit)
            yield dict(category=localized_category(SUBREDDIT_CATEGORY, locale))
            for subreddit in self._get_results(subs_uri)['data']['children']:
                yield dict(result=_parse_subreddit(subreddit, query, locale))

            # Search posts
            posts_uri = POST_SEARCH_URI % dict(query=query_cleaned,
                                               limit=limit)
            yield dict(category=localized_category(S_THREAD_CATEGORY, locale))
            for post in self._get_results(posts_uri)['data']['children']:
                yield dict(result=_parse_post(post, query,
                                              locale, platform))

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']
        mascot = None
        if result['cat_id'] == "posts":
            if result['media_type'] == "rich" \
                    and result['media_preview'] \
                    and not result['media_preview'].endswith('.gif'):
                yield dict(widget={
                    'id': 'image',
                    'type': 'image',
                    'source': result['media_preview'],
                })
            elif result['media_type'] == "video":
                yield dict(widget={
                    'id': 'video',
                    'type': 'video',
                    'source': result['target'],
                    'screenshot': result['media_preview'],
                })
            else:
                mascot = result['mascot']
            # yield a 'header' widget
            yield dict(widget={
                'id': 'header',
                'type': 'header',
                'title': result['title'],
                'subtitle': result['domain'],
                'mascot': mascot,
            })
            # add an 'actions' widget
            actions = [{'label': result['comments'],
                        'uri': result['uri'],
                        'id': 'action1'}]
            if not result['target'].startswith("http://www.reddit.com/r/"):
                actions.append({'label': translator('View', locale),
                                'uri': result['target'],
                                'id': 'action2'})
            yield dict(widget={
                'id': 'buttons',
                'type': 'actions',
                'actions': actions,
            })
        else:
            if result['art']:
                # yield an 'image' widget
                yield dict(widget={
                    'id': 'image',
                    'type': 'image',
                    'source': result['art'],
                    })
            # yield a 'header' widget
            yield dict(widget={
                'id': 'header',
                'type': 'header',
                'title': result['title'],
                'subtitle': result['header_title'],
            })
            # add an 'actions' widget
            actions = [{'label': '/r/' + result['subreddit'],
                        'uri': result['uri'],
                        'id': 'action1'}]
            yield dict(widget={
                'id': 'buttons',
                'type': 'actions',
                'actions': actions,
            })
        # yield a 'text' widget
        yield dict(widget={
            'id': 'text',
            'type': 'text',
            'text': result['summary'],
        })
