# Copyright 2014-2015 Canonical
# All Rigths Reserved

"""The 7digital scope."""

import collections
import datetime
import json
import operator
import urllib

from oauth import oauth

from lxml import etree

from ..scopes import SimpleApp, BackendDataError
from ..translator import translator

# how many of the genres to show for the 'genres' department
TOP_GENRES = 5

# all the genres to show in the departments structure
GENRES = [
    ('genre-pop', translator('Pop')),
    ('genre-dance', translator('Dance')),
    ('genre-electronic', translator('Electronic')),
    ('genre-rock', translator('Rock')),
    ('genre-jazz', translator('Jazz')),
    ('genre-world', translator('World')),
    ('genre-compilation', translator('Compilation')),
    ('genre-classical', translator('Classical')),
    ('genre-house', translator('House')),
    ('genre-alternative', translator('Alternative')),
    ('genre-instrumental', translator('Instrumental')),
    ('genre-country', translator('Country')),
    ('genre-latin', translator('Latin')),
    ('genre-folk', translator('Folk')),
    ('genre-techno', translator('Techno')),
    ('genre-reggae', translator('Reggae')),
    ('genre-hip hop/rap', translator('Hip Hop and Rap')),
    ('genre-trance', translator('Trance')),
    ('genre-blues', translator('Blues')),
    ('genre-soundtrack', translator('Soundtrack')),
    ('genre-christian', translator('Christian')),
    ('genre-r&b/soul', translator('R&B and Soul')),
    ('genre-new age', translator('New Age')),
    ('genre-hard rock/metal', translator('Hard Rock and Metal')),
    ('genre-tech house', translator('Tech House')),
    ('genre-easy listening', translator('Easy Listening')),
    ('genre-christmas', translator('Christmas')),
    ('genre-electro', translator('Electro')),
    ('genre-holiday', translator('Holiday')),
    ('genre-drum & bass', translator('Drum & Bass')),
    ('genre-pop rock', translator('Pop Rock')),
    ('genre-karaoke', translator('Karaoke')),
    ('genre-punk', translator('Punk')),
    ('genre-electronica', translator('Electronica')),
    ('genre-live', translator('Live')),
    ('genre-dubstep', translator('Dubstep')),
    ('genre-deep house', translator('Deep House')),
    ('genre-spoken word', translator('Spoken Word')),
    ('genre-inspirational', translator('Inspirational')),
    ('genre-children', translator('Children')),
    ('genre-alternative/indie', translator('Alternative/Indie')),
    ('genre-2000s', translator('2000s')),
    ('genre-gospel', translator('Gospel')),
    ('genre-chamber music', translator('Chamber Music')),
    ('genre-orchestral', translator('Orchestral')),
    ('genre-ambient', translator('Ambient')),
    ('genre-french', translator('French')),
    ('genre-indie rock', translator('Indie Rock')),
    ('genre-african', translator('African')),
    ('genre-dancehall', translator('Dancehall')),
]

# the base departments... id, label, and if it has subdepartments
BASE_DEPARTMENTS = [
    ('newreleases', translator('New releases'), False),
    ('topalbums', translator('Top albums'), False),
    ('toptracks', translator('Top tracks'), False),
    ('genres', translator('Genres'), True),
]

# the label of the departments root
ROOT_DEPARTMENT_LABEL = translator('All categories')

# titles
TITLE_ARTISTS = "Artists"
TITLE_ALBUMS = "Albums"
TITLE_TRACKS = "Tracks"

# list of countries that we can not send to 7digital
UNAUTHORIZED_COUNTRIES = set("AR BR CO DK ID IS MX TH UY ZA".split())
DEFAULT_COUNTRY = 'GB'

# the categories
CATEGORY_GRID_LARGE = {
    'id': 'cat-grid-large',
    'title': '',
    'render_template': json.dumps({
        "schema-version": 1,
        'template': {'category-layout': 'grid',
                     'card-size': 'large',
                     'collapsed-rows': 0},
        'components': {'title': 'title',
                       'art': 'art'},
    }),
}
CATEGORY_GRID_MEDIUM = {
    'id': 'cat-grid-medium',
    'title': '',
    'render_template': json.dumps({
        "schema-version": 1,
        'template': {'category-layout': 'grid',
                     'card-size': 'medium',
                     'collapsed-rows': 0},
        'components': {'title': 'title',
                       'art': 'art',
                       'subtitle': 'subtitle',
                       'attributes': 'attributes'},
    }),
}
CATEGORY_GRID_SMALL = {
    'id': 'cat-grid-small',
    'title': '',
    'render_template': json.dumps({
        "schema-version": 1,
        'template': {'category-layout': 'grid',
                     'card-size': 'small',
                     'collapsed-rows': 0},
        'components': {'title': 'title',
                       'art': 'art',
                       'subtitle': 'subtitle',
                       'attributes': 'attributes'},
    }),
}
CATEGORY_LIST_SHORT = {
    'id': 'cat-list-short',
    'title': '',
    'render_template': json.dumps({
        "schema-version": 1,
        'template': {'category-layout': 'grid',
                     'card-size': 'large',
                     'card-layout': 'horizontal',
                     'collapsed-rows': 3},
        'components': {'title': 'title',
                       'art': 'art',
                       'subtitle': 'subtitle',
                       'attributes': 'attributes'},
    }),
}
CATEGORY_LIST_LONG = {
    'id': 'cat-list-long',
    'title': '',
    'render_template': json.dumps({
        "schema-version": 1,
        'template': {'category-layout': 'grid',
                     'card-size': 'large',
                     'card-layout': 'horizontal',
                     'collapsed-rows': 0},
        'components': {'title': 'title',
                       'art': 'art',
                       'subtitle': 'subtitle',
                       'attributes': 'attributes'},
    }),
}

Album = collections.namedtuple(
    "Album", "id title price url image artist release_date"
    " price_amount price_currency")
Track = collections.namedtuple(
    "Track", "id title price url image artist release_date length"
    " price_amount price_currency")
Artist = collections.namedtuple("Artist", "id name url image bio")


def url_to_ascii(url):
    """Encode URL to ASCII."""
    try:
        url = url.encode('ascii')
    except (UnicodeEncodeError, UnicodeDecodeError):
        url = "".join(urllib.quote(x.encode("utf8")) if ord(x) > 127 else
                      x for x in url).encode('ascii')
    return url


class SevenDigitalInterface(object):
    """Intermediate to talk to 7digital."""
    _api_base = 'http://api.7digital.com/1.2/'
    _preview_base = "http://previews.7digital.com/clip/"

    def __init__(self, oauth_key, oauth_secret, country, urlread):
        self.oauth_key = oauth_key
        self.oauth_secret = oauth_secret
        if country is None or country in UNAUTHORIZED_COUNTRIES:
            country = DEFAULT_COUNTRY
        self.country = country
        self.urlread = urlread

    def build_preview_track_source(self, track_id):
        """Build the signed URL to preview the specific track."""
        # build the URL
        params = {'country': self.country}
        url = self._preview_base + track_id + '?' + urllib.urlencode(params)

        consumer = oauth.OAuthConsumer(self.oauth_key, self.oauth_secret)
        req = oauth.OAuthRequest.from_consumer_and_token(
            consumer, http_url=url, parameters=params)
        sig_method = oauth.OAuthSignatureMethod_HMAC_SHA1()
        req.sign_request(sig_method, consumer, token=None)
        return req.to_url()

    def _hit_backend(self, path, **params):
        """Hit 7digital backend and get the response already as XML."""
        # build the URL
        d = dict(oauth_consumer_key=self.oauth_key, pageSize=20, imageSize=500,
                 country=self.country)
        d.update(params)
        url = self._api_base + path + "?" + urllib.urlencode(d)

        # hit the service, read the data and convert to XML
        raw = self.urlread(url)
        try:
            xml = etree.XML(raw)
        except etree.XMLSyntaxError as exc:
            raise BackendDataError(exc, url)
        return xml

    def _extract_albums(self, xml):
        """Get the albums and related info from a XML result."""
        albums = []
        included = set()
        for item in xml.xpath("//release"):
            if item.findtext("explicitContent") == 'true':
                continue
            if item.findtext("type") == "Album":
                id_ = item.get("id")
                if id_ in included:
                    continue
                included.add(id_)

                title = item.findtext("title")
                url = url_to_ascii(item.findtext("url"))
                image = item.findtext("image")
                price = item.findtext("price/formattedPrice")
                price_amount = item.findtext("price/value")
                price_currency = item.findtext("price/currency")
                rd = item.findtext("releaseDate").split("T")[0]
                artist = self._build_artist(item.find("artist"))
                album = Album(id=id_, title=title, url=url, price=price,
                              image=image, artist=artist, release_date=rd,
                              price_amount=price_amount,
                              price_currency=price_currency)
                albums.append(album)
        return albums

    def _extract_tracks(self, xml):
        """Get the tracks and related info from a XML result."""
        tracks = []
        included = set()
        for item in xml.xpath("//track"):
            if item.findtext("explicitContent") == 'true':
                continue
            id_ = item.get("id")
            if id_ in included:
                continue
            included.add(id_)

            title = item.findtext("title")
            url = url_to_ascii(item.findtext("url"))
            image = item.findtext("release/image")
            price = item.findtext("price/formattedPrice")
            price_amount = item.findtext("price/value")
            price_currency = item.findtext("price/currency")
            rd = item.findtext("release/releaseDate").split("T")[0]
            length = int(item.findtext("duration"))
            artist = self._build_artist(item.find("artist"))
            track = Track(id=id_, title=title, url=url, price=price,
                          image=image, artist=artist,
                          release_date=rd, length=length,
                          price_amount=price_amount,
                          price_currency=price_currency)
            tracks.append(track)
        return tracks

    def _build_artist(self, item):
        """Build an artist after the typical sub-xml-item info."""
        artist = Artist(id=item.get('id'), image=None, bio=None,
                        name=item.findtext("name"),
                        url=url_to_ascii(item.findtext("url")))
        return artist

    def get_album_latest(self):
        """Get the latest released albums."""
        oneweekago = datetime.date.today() - datetime.timedelta(days=7)
        xml = self._hit_backend("release/bydate",
                                fromDate=oneweekago.strftime("%Y%m%d"))
        albums = self._extract_albums(xml)
        albums.sort(key=operator.attrgetter('release_date'), reverse=True)
        return albums

    def get_album_topseller(self, image_size=None, page_size=None):
        """Get the more selled albums."""
        params = {}
        if image_size is not None:
            params['imageSize'] = image_size
        if page_size is not None:
            params['pageSize'] = page_size
        xml = self._hit_backend("release/chart", **params)
        return self._extract_albums(xml)

    def get_track_topseller(self, image_size=None, page_size=None):
        """Get the more selled tracks."""
        params = {}
        if image_size is not None:
            params['imageSize'] = image_size
        if page_size is not None:
            params['pageSize'] = page_size
        xml = self._hit_backend('track/chart', **params)
        return self._extract_tracks(xml)

    def get_album_tagged(self, tags):
        """Get the more selled tracks."""
        xml = self._hit_backend('release/bytag/new', tags=tags)
        return self._extract_albums(xml)

    def search_artists(self, query, image_size=None):
        """Search some artists."""
        params = dict(q=query.encode('utf8'))
        if image_size is not None:
            params['imageSize'] = image_size
        xml = self._hit_backend('artist/search', **params)
        count = int(xml.findtext("searchResults/totalItems"))
        artists = []
        for item in xml.xpath("//artist"):
            id_ = item.get("id")
            name = item.findtext("name")
            url = url_to_ascii(item.findtext("url"))
            image = item.findtext("image")
            artist = Artist(id=id_, name=name, url=url, image=image, bio=None)
            artists.append(artist)
        return count, artists

    def search_albums(self, query, image_size=None, page_size=None):
        """Search some albums."""
        params = dict(q=query.encode('utf8'), type='album')
        if image_size is not None:
            params['imageSize'] = image_size
        if page_size is not None:
            params['pageSize'] = page_size
        xml = self._hit_backend('release/search', **params)
        count = int(xml.findtext("searchResults/totalItems"))
        albums = self._extract_albums(xml)
        return count, albums

    def get_album_details(self, album_id):
        """Get the album details."""
        xml = self._hit_backend('release/details', releaseId=album_id)
        album = self._extract_albums(xml)
        if not album:
            return None
        return album[0]

    def search_tracks(self, query, image_size=None, page_size=None):
        """Search some tracks."""
        params = dict(q=query.encode('utf8'))
        if image_size is not None:
            params['imageSize'] = image_size
        if page_size is not None:
            params['pageSize'] = page_size
        xml = self._hit_backend('track/search', **params)
        count = int(xml.findtext("searchResults/totalItems"))
        tracks = self._extract_tracks(xml)
        return count, tracks

    def get_track_details(self, track_id):
        """Get the track details."""
        xml = self._hit_backend('track/details', trackId=track_id)
        track = self._extract_tracks(xml)
        if not track:
            return None
        return track[0]

    def get_artist_albums(self, artist_id):
        """Get the albums for a specific artist."""
        # per artist, get albums
        xml = self._hit_backend('artist/releases',
                                artistId=artist_id, type='album')
        return self._extract_albums(xml)

    def get_artist_tracks(self, artist_id):
        """Get the tracks for a specific artist."""
        xml = self._hit_backend('artist/toptracks', artistId=artist_id)
        return self._extract_tracks(xml)

    def get_artist_details(self, artist_id, image_size=None):
        """Get the artist details."""
        params = dict(artistId=artist_id)
        if image_size is not None:
            params['imageSize'] = image_size
        xml = self._hit_backend('artist/details', **params)
        item = xml.xpath("//artist")[0]
        id_ = item.get("id")
        name = item.findtext("name")
        url = url_to_ascii(item.findtext("url"))
        image = item.findtext("image")
        bio = item.findtext("bio/text").strip()
        artist = Artist(id=id_, name=name, url=url, image=image, bio=bio)
        return artist

    def get_album_tracks(self, album_id):
        """Get the tracks for a specific album."""
        xml = self._hit_backend('release/tracks', releaseId=album_id)
        tracks = []
        for item in xml.xpath("//track"):
            if item.findtext("explicitContent") == 'true':
                continue
            id_ = item.get("id")
            title = item.findtext("title")
            url = url_to_ascii(item.findtext("url"))
            image = item.findtext("release/image")
            price = item.findtext("price/formattedPrice")
            price_amount = item.findtext("price/value")
            price_currency = item.findtext("price/currency")
            length = int(item.findtext("duration"))
            number = item.findtext("trackNumber")
            artist = self._build_artist(item.find("artist"))
            track = Track(id=id_, title=title, url=url, price=price,
                          image=image, artist=artist,
                          release_date=None, length=length,
                          price_amount=price_amount,
                          price_currency=price_currency)
            tracks.append((number, track))
        tracks = [x[1] for x in sorted(tracks)]
        return tracks


class App(SimpleApp):
    """The 7digital app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self.auth = self.config['secrets']

        # convert ordered structures to dict for better search
        self._d_base_deps = {x[0]: x[1] for x in BASE_DEPARTMENTS}
        self._d_genres = dict(GENRES)

    def _get_departments_tree(self, current, locale):
        """Build the tree of departments, centered in current node."""
        if current in self._d_base_deps:
            # any of the base departments
            subdeps = []
            if current == 'genres':
                for dep_id, dep_label in GENRES:
                    subdeps.append({
                        'label': translator(dep_label, locale),
                        'canned_query': ('scope://com.canonical.scopes.'
                                         'sevendigital?dep=' + dep_id),
                        'has_subdepartments': False,
                    })
                subdeps.sort(key=operator.itemgetter('label'))
            tree = {
                'label': translator(ROOT_DEPARTMENT_LABEL, locale),
                'canned_query': 'scope://com.canonical.scopes.sevendigital',
                'subdepartments': [{
                    'label': translator(self._d_base_deps[current], locale),
                    'canned_query': ('scope://com.canonical.scopes.'
                                     'sevendigital?dep=' + current),
                    'subdepartments': subdeps,
                }]
            }
        elif current in self._d_genres:
            # any genre
            tree = {
                'label': translator(self._d_base_deps['genres'], locale),
                'canned_query': ('scope://com.canonical.scopes.'
                                 'sevendigital?dep=genres'),
                'subdepartments': [{
                    'label': translator(self._d_genres[current], locale),
                    'canned_query': ('scope://com.canonical.scopes.'
                                     'sevendigital?dep=' + current),
                    'subdepartments': [],
                }]
            }
        else:
            # root, or at least return root if received garbage
            subdeps = []
            for dep_id, dep_label, has_subdeps in BASE_DEPARTMENTS:
                subdeps.append({
                    'label': translator(dep_label, locale),
                    'canned_query': ('scope://com.canonical.scopes.'
                                     'sevendigital?dep=' + dep_id),
                    'has_subdepartments': has_subdeps,
                })
            tree = {
                'label': translator(ROOT_DEPARTMENT_LABEL, locale),
                'canned_query': 'scope://com.canonical.scopes.sevendigital',
                'subdepartments': subdeps,
            }
        return tree

    def _yield_albums(self, cat, albums):
        """Yield the result information for albums."""
        yield dict(category=cat)
        for album in albums:
            yield dict(result={
                'cat_id': cat['id'],
                'uri': album.url,
                'art': album.image,
                'title': album.title,
                'subtitle': album.artist.name,
                'attributes': [{'style': 'default', 'value': album.price}],
                'preview_info': {
                    'type': 'album',
                    'album_id': album.id,
                    'release_date': album.release_date,
                    'price': album.price,
                    'artist': album.artist.name,
                },
            })

    def _yield_tracks(self, cat, tracks):
        """Yield the result information for tracks."""
        yield dict(category=cat)
        for track in tracks:
            yield dict(result={
                'cat_id': cat['id'],
                'uri': track.url,
                'art': track.image,
                'title': track.title,
                'subtitle': track.artist.name,
                'attributes': [{'style': 'default',
                                'value': track.price}],
                'preview_info': {
                    'type': 'track',
                    'track_id': track.id,
                    'release_date': track.release_date,
                    'price': track.price,
                    'artist': track.artist.name,
                    'length': track.length,
                },
            })

    def _search_surfacing(self, sdi, department, locale):
        """Produce the results for a surfacing search."""
        tree = self._get_departments_tree(department, locale)
        yield dict(departments=tree)

        if not department:
            albums_latest = sdi.get_album_latest()
            cat = CATEGORY_GRID_MEDIUM.copy()
            cat['title'] = translator("New releases", locale)
            for r in self._yield_albums(cat, albums_latest[:2]):
                yield r

            for r in self._yield_albums(CATEGORY_GRID_SMALL,
                                        albums_latest[2:5]):
                yield r

            albums_top = sdi.get_album_topseller()
            cat = CATEGORY_LIST_LONG.copy()
            cat['title'] = translator("Best selling albums", locale)
            for r in self._yield_albums(cat, albums_top):
                yield r

        elif department == 'newreleases':
            albums_latest = sdi.get_album_latest()
            for r in self._yield_albums(CATEGORY_GRID_MEDIUM,
                                        albums_latest[:2]):
                yield r

            for r in self._yield_albums(CATEGORY_GRID_SMALL,
                                        albums_latest[2:]):
                yield r

        elif department == 'topalbums':
            albums_top = sdi.get_album_topseller()
            for r in self._yield_albums(CATEGORY_LIST_LONG, albums_top):
                yield r

        elif department == 'toptracks':
            tracks_top = sdi.get_track_topseller()
            for r in self._yield_tracks(CATEGORY_LIST_LONG, tracks_top):
                yield r

        elif department == 'genres':
            for genre, label in GENRES[:TOP_GENRES]:
                tag = genre[6:]
                albums = sdi.get_album_tagged(tag)
                cat = CATEGORY_GRID_SMALL.copy()
                cat['id'] = 'cat-genre-' + tag
                cat['title'] = translator(label, locale)
                for r in self._yield_albums(cat, albums):
                    yield r

        elif department.startswith('genre-'):
            tag = department[6:]
            albums = sdi.get_album_tagged(tag)
            for r in self._yield_albums(CATEGORY_GRID_SMALL, albums):
                yield r

    def _build_count(self, prefix, count, locale):
        """Build the count string for categories headers."""
        t_prefix = translator(prefix, locale)
        if count == 1:
            # used for the specific case of one result only
            result = translator("(1 result)", locale)
        else:
            # the {} will be replaced with a number: the results quantity
            result = translator("({} results)", locale)
            result = result.format(count)
        return t_prefix + ' ' + result

    def _search_normal(self, sdi, query, locale):
        """Produce the results for a normal search."""
        c_artists, artists = sdi.search_artists(query)
        c_albums, albums = sdi.search_albums(query)
        c_tracks, tracks = sdi.search_tracks(query)

        cat = CATEGORY_LIST_SHORT.copy()
        cat['id'] = 'artists'
        cat['title'] = self._build_count(TITLE_ARTISTS, c_artists, locale)
        yield dict(category=cat)
        for artist in artists:
            params = dict(q=query.encode('utf8'),
                          dep='specialsearch-' + artist.id)
            url_encoded = urllib.urlencode(params)
            uri_encoded = url_encoded.replace('+', '%20')
            cq = 'scope://com.canonical.scopes.sevendigital?' + uri_encoded
            yield dict(result={
                'cat_id': cat['id'],
                'uri': cq,
                'art': artist.image,
                'title': artist.name,
            })

        cat = CATEGORY_LIST_SHORT.copy()
        cat['id'] = 'albums'
        cat['title'] = self._build_count(TITLE_ALBUMS, c_albums, locale)
        for r in self._yield_albums(cat, albums):
            yield r

        cat = CATEGORY_LIST_SHORT.copy()
        cat['id'] = 'tracks'
        cat['title'] = self._build_count(TITLE_TRACKS, c_tracks, locale)
        for r in self._yield_tracks(cat, tracks):
            yield r

    def _search_special(self, sdi, artist_id, locale):
        """Produce the results showing a specific artist."""
        artist = sdi.get_artist_details(artist_id)
        yield dict(category=CATEGORY_GRID_LARGE)
        yield dict(result={
            'cat_id': CATEGORY_GRID_LARGE['id'],
            'uri': artist.url,
            'art': artist.image,
            'title': artist.name,
        })

        albums = sdi.get_artist_albums(artist_id)
        cat = CATEGORY_LIST_SHORT.copy()
        cat['id'] = 'albums'
        cat['title'] = translator("Albums", locale)
        for r in self._yield_albums(cat, albums):
            yield r

        tracks = sdi.get_artist_tracks(artist_id)
        cat = CATEGORY_LIST_SHORT.copy()
        cat['id'] = 'tracks'
        cat['title'] = translator("Tracks", locale)
        for r in self._yield_tracks(cat, tracks):
            yield r

    def search(self, **kwargs):
        """Do the search."""
        query = kwargs['query'].strip()
        locale = kwargs['locale']
        department = kwargs['department']
        country = kwargs['country']

        # build the interface
        auth = self.auth[kwargs['partner']]
        _oauth_key = auth['oauth_consumer_key']
        _oauth_secret = auth['oauth_consumer_secret']
        sdi = SevenDigitalInterface(_oauth_key, _oauth_secret, country, self.urlread)

        if query:
            # search!
            if department and department.startswith('specialsearch-'):
                # special not-really-search, showing info for a specifc artist
                artist_id = department[14:]
                for r in self._search_special(sdi, artist_id, locale):
                    yield r
            else:
                # normal search
                for r in self._search_normal(sdi, query, locale):
                    yield r
        else:
            # surfacing
            for r in self._search_surfacing(sdi, department, locale):
                yield r

    def _preview_album(self, sdi, locale, result, previnfo):
        """The preview of an album."""
        art_image = result['art']
        uri = result['uri']
        title = result['title']
        album_id = previnfo['album_id']
        release_date = dict(zip('year month day'.split(),
                                previnfo['release_date'].split("-")))
        price = previnfo['price']
        artist = previnfo['artist']

        # the album image
        yield dict(widget={
            'id': 'art',
            'type': 'image',
            'source': art_image,
        })

        # a header with the album info
        # the {} with be replaced by the artist who did the album
        artist_ownership = translator(u'by {}', locale)
        released = u" ".join((
            translator(u'Released:', locale),
            '{day}/{month}/{year}'.format(**release_date)))
        yield dict(widget={
            'id': 'header',
            'type': 'header',
            'uri': uri,
            'title': title,
            'subtitle': artist_ownership.format(artist),
            'attributes': [{'style': 'default',
                            'value': released}],
        })

        # the button with the price
        button_text_base = translator(u'{} on 7digital', locale)
        button_text = button_text_base.format(price)
        yield dict(widget={
            'id': 'button',
            'type': 'actions',
            'actions': [{'id': uri, 'label': button_text}],
        })

        # the tracks
        tracks = sdi.get_album_tracks(album_id)
        widget_tracks = []
        for i, track in enumerate(tracks, 1):
            title = u"{:02d}. {}".format(i, track.title)
            widget_tracks.append({
                'title': title,
                'source': sdi.build_preview_track_source(track.id),
                'length': track.length,
            })
        if widget_tracks:
            yield dict(widget={
                'id': 'tracks',
                'type': 'audio',
                'tracks': widget_tracks
            })

    def _preview_track(self, sdi, locale, result, previnfo):
        """The preview of an track."""
        art_image = result['art']
        uri = result['uri']
        title = result['title']
        track_id = previnfo['track_id']
        release_date = dict(zip('year month day'.split(),
                                previnfo['release_date'].split("-")))
        price = previnfo['price']
        artist = previnfo['artist']
        length = previnfo['length']

        # a header with the album info
        # the {} with be replaced by the artist who did the album
        artist_ownership = translator(u'by {}', locale)
        released = u" ".join((
            translator(u'Released:', locale),
            '{day}/{month}/{year}'.format(**release_date)))
        yield dict(widget={
            'id': 'header',
            'type': 'header',
            'uri': uri,
            'title': title,
            'subtitle': artist_ownership.format(artist),
            'attributes': [{'style': 'default',
                            'value': released}],
            'mascot': art_image,
        })

        # the button with the price
        button_text_base = translator(u'{} on 7digital', locale)
        button_text = button_text_base.format(price)
        yield dict(widget={
            'id': 'button',
            'type': 'actions',
            'actions': [{'id': uri, 'label': button_text}],
        })

        # the track itself
        yield dict(widget={
            'id': 'tracks',
            'type': 'audio',
            'tracks': [{
                'title': title,
                'source': sdi.build_preview_track_source(track_id),
                'length': length,
            }],
        })

    def preview(self, **kwargs):
        """Do the preview."""
        locale = kwargs['locale']
        result = kwargs['result']
        previnfo = result.get('preview_info', {})
        country = kwargs['country']

        # build the interface
        auth = self.auth[kwargs['partner']]
        _oauth_key = auth['oauth_consumer_key']
        _oauth_secret = auth['oauth_consumer_secret']
        sdi = SevenDigitalInterface(_oauth_key, _oauth_secret, country, self.urlread)

        try:
            preview_type = previnfo['type']
        except KeyError:
            # if we don't have preview type, it means the client just asked
            # for a preview of a result that it shouldn't have, or that we
            # return results not ready for preview, which is a kind-of valid
            # case
            return []

        if preview_type == 'album':
            meth = self._preview_album
        elif preview_type == 'track':
            meth = self._preview_track
        else:
            return []

        return meth(sdi, locale, result, previnfo)
