# -*- coding: utf8 -*-

# Copyright 2014-2015 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import json
import urllib

from contextlib import nested
from unittest import TestCase

from lxml import etree
from mock import patch, Mock, call
from urllib2 import HTTPError

from src.scopes import amazon
from src.scopes.tests import FakeAmazon, get_fixture

CONFIG = {'secrets': {None: {'key': '', 'secret': '',
                             'shops': {'US': 'usshopcode',
                                       'CN': 'cnshopcode',
                                       'ES': 'esshopcode',
                                       'UK': 'ukshopcode'}}},
          'scopes_config_dir': 'configs/scopes', }


class AmazonTestCase(TestCase):
    """Amazon search tests."""
    maxDiff = None

    def test_normalize_shop_upper(self):
        app = amazon.App(CONFIG)
        shop = app._normalize_shop('es', CONFIG['secrets'][None]['shops'])
        self.assertEqual(shop, 'ES')

    def test_normalize_shop_gb(self):
        app = amazon.App(CONFIG)
        shop = app._normalize_shop('GB', CONFIG['secrets'][None]['shops'])
        self.assertEqual(shop, 'UK')

    def test_normalize_shop_fallback(self):
        app = amazon.App(CONFIG)
        shop = app._normalize_shop('not a valid country',
                                   CONFIG['secrets'][None]['shops'])
        self.assertEqual(shop, 'US')

    def test_search(self):
        app = amazon.App(CONFIG)
        fake_amazon = FakeAmazon("search.xml")
        with patch.object(amazon, 'Amazon') as mock:
            mock.return_value = fake_amazon
            response = list(app.search(country='US', query='t_query', limit=10,
                                       filters=None, locale='en_US',
                                       department=None, session_id='',
                                       partner=None))
        category = response[0]['category']
        self.assertEqual(category['id'], "generic")
        self.assertEqual(category['title'], "Results")

        mock.assert_called_with(app.urlread, '', '', 'usshopcode', 'US')
        self.assertEqual({
            'result': {
                'price': '11.09F',
                'art': u'fake:large_image',
                'dnd_uri': 'fake:details',
                'title': 'Title',
                'subtitle': 'TestBrand',
                'cat_id': 'generic',
                'uri': 'fake:details',
                'preview_info': {
                    'item_id': 'ASIN',
                    'root_code': None,
                },
            }},
            response[1])

        amz_call = fake_amazon.history[0]
        self.assertEqual(amz_call[0], 'ItemSearch')
        should = dict(
            Keywords='t_query', SearchIndex='All',
            ResponseGroup='Images,ItemAttributes,OfferSummary,Offers')
        self.assertEqual(amz_call[1], should)

    def test_search_geostore_using_locale(self):
        app = amazon.App(CONFIG)
        with patch.object(amazon, 'Amazon') as mock:
            mock.return_value = FakeAmazon("search.xml")
            list(app.search(country='CN', query='fake', limit=10, locale='C',
                            filters=None, session_id='', department=None,
                            partner=None))

        mock.assert_called_with(app.urlread, '', '', 'cnshopcode', 'CN')

    def test_search_geostore_using_country(self):
        app = amazon.App(CONFIG)
        with patch.object(amazon, 'Amazon') as mock:
            mock.return_value = FakeAmazon("search.xml")
            list(app.search(country='es', query='fake', limit=10,
                            filters=None, locale='en', department=None,
                            session_id='', partner=None))

        mock.assert_called_with(app.urlread, '', '', 'esshopcode', 'ES')

    def test_landpage(self):
        """Check the landpage response, for a req. with no query and no dep."""
        app = amazon.App(CONFIG)

        # put custom structures for the departments map so we can predict what
        # is randomly selected (which needs to have a 'salesrank' sorting)
        app.departments_map = dict(US={
            123456: (u'DepartmentRootCode', u'Human text ñá£'),
            456789: (u'Bogus', u'Foo Bar'),
        })
        app.sorting_info = dict(US={
            'DepartmentRootCode': ['sorting1', 'salesrank', 'sorting2'],
            'Bogus': ['sorting1', 'sorting2'],
        })
        app.subtitle_attribs['DepartmentRootCode'] = []

        fake_amazon = FakeAmazon("search.xml")
        with patch.object(amazon, 'Amazon') as mock_amz:
            with patch.object(amazon, '_get_department_tree') as mock_dtree:
                mock_amz.return_value = fake_amazon
                mock_dtree.return_value = 'fake department tree'
                response = list(app.search(country='US', query='', limit=10,
                                           filters=None, locale='en_US',
                                           department=None, session_id='',
                                           partner=None))

        mock_amz.assert_called_with(app.urlread, '', '', 'usshopcode', 'US')
        root_depts = app.departments_map['US']
        mock_dtree.assert_called_with(fake_amazon, root_depts,
                                      None, None, 'en_US', {})
        self.assertEqual(response[0], {'departments': 'fake department tree'})

        # the category is the landpage one, with a custom title
        categ_with_title = amazon.LANDPAGE_CATEGORY.copy()
        categ_with_title['title'] = u"Human text ñá£, best sellers"
        self.assertEqual(response[1], {'category': categ_with_title})

        self.assertEqual(response[2].keys(), ['result'])

        amz_call = fake_amazon.history[0]
        self.assertEqual(amz_call[0], 'ItemSearch')
        should = dict(
            Keywords=' ', SearchIndex='DepartmentRootCode',
            BrowseNode='123456', Sort='salesrank',
            ResponseGroup='Images,ItemAttributes,OfferSummary,Offers')
        self.assertEqual(amz_call[1], should)

    def test_search_with_department(self):
        app = amazon.App(CONFIG)
        fake_amazon = FakeAmazon("search.xml")
        with patch.object(amazon, 'Amazon') as mock:
            mock.return_value = fake_amazon
            args = dict(country='FAKE', query='testquery', limit=10,
                        department='Shoes:1234', filters=None, locale='C',
                        session_id='', partner=None)
            response = list(app.search(**args))
        category = response[0]['category']
        self.assertEqual(category['id'], "generic")
        self.assertEqual(category['title'], "Results")
        amz_call = fake_amazon.history[0]
        self.assertEqual(amz_call[0], 'ItemSearch')
        should = dict(
            Keywords='testquery', BrowseNode='1234', SearchIndex='Shoes',
            ResponseGroup='Images,ItemAttributes,OfferSummary,Offers')
        self.assertEqual(amz_call[1], should)

    def test_surfacing_with_department(self):
        app = amazon.App(CONFIG)
        fake_amazon = FakeAmazon("search.xml")
        with patch.object(amazon, 'Amazon') as mock_amz:
            with patch.object(amazon, '_get_department_tree') as mock_dtree:
                mock_amz.return_value = fake_amazon
                mock_dtree.return_value = 'fake department tree'
                args = dict(country='FAKE', query='', limit=10, session_id='',
                            department='Shoes:1234', filters=None, locale='C',
                            partner=None)
                response = list(app.search(**args))

        mock_amz.assert_called_with(app.urlread, '', '', 'usshopcode', 'US')
        root_depts = app.departments_map['US']
        mock_dtree.assert_called_with(fake_amazon, root_depts,
                                      'Shoes', '1234', 'C', {})
        self.assertEqual(response[0], {'departments': 'fake department tree'})
        self.assertEqual(response[1].keys(), ['filters'])
        self.assertEqual(response[2].keys(), ['filter_state'])
        category = response[3]['category']
        self.assertEqual(category['id'], "generic")
        self.assertEqual(category['title'], "")
        self.assertEqual(response[4].keys(), ['result'])

        amz_call = fake_amazon.history[0]
        self.assertEqual(amz_call[0], 'ItemSearch')
        should = dict(
            Keywords=' ', SearchIndex='Shoes', BrowseNode='1234',
            ResponseGroup='Images,ItemAttributes,OfferSummary,Offers')
        self.assertEqual(amz_call[1], should)

    def test_filters(self):
        filters = {"MaximumPrice": 5000,
                   "foo": "bar"}
        should_filters = {"MaximumPrice": 5000}
        validate = amazon.parse_filters(filters)

        # Ensure only valid Amazon filters are kept
        self.assertEqual(should_filters, validate)

    def test_surfacing_with_sorting(self):
        app = amazon.App(CONFIG)
        fake_amazon = FakeAmazon("search.xml")
        with nested(patch.object(amazon, 'Amazon'),
                    patch.object(amazon, '_get_department_tree'),
                    patch.object(amazon, '_get_sorting_filters'),
                    ) as (mock_amz, mock_dtree, mock_srtf):
            mock_amz.return_value = fake_amazon
            mock_dtree.return_value = 'fake department tree'
            mock_srtf.return_value = ('new_sort_list', 'new_filter_state')
            sorting_filter = {amazon.SORTING_FILTER_ID: ['testsrt']}
            args = dict(country='FAKE', query='', department='DVD:1234',
                        limit=10, filters=sorting_filter, locale='C',
                        session_id='', partner=None)
            response = list(app.search(**args))

        mock_amz.assert_called_with(app.urlread, '', '', 'usshopcode', 'US')
        root_depts = app.departments_map['US']
        mock_dtree.assert_called_with(fake_amazon, root_depts,
                                      'DVD', '1234', 'C', sorting_filter)
        allowed = app.sorting_info['US']['DVD']
        mock_srtf.assert_called_with(allowed, 'C', sorting_filter)
        self.assertEqual(response[0].keys(), ['departments'])
        self.assertEqual(response[1], {'filters': ['new_sort_list']})
        self.assertEqual(response[2], {'filter_state': 'new_filter_state'})
        self.assertEqual(response[3].keys(), ['category'])
        self.assertEqual(response[4].keys(), ['result'])

        amz_call = fake_amazon.history[0]
        self.assertEqual(amz_call[0], 'ItemSearch')
        should = dict(
            Keywords=' ', SearchIndex='DVD', BrowseNode='1234', Sort='testsrt',
            ResponseGroup='Images,ItemAttributes,OfferSummary,Offers')
        self.assertEqual(amz_call[1], should)

    def test_preview(self):
        app = amazon.App(CONFIG)
        result = {
            'art': u'arttest',
            'cat_id': 'cat_0',
            'dnd_uri': u'http://amazontest',
            'price': u'$42',
            'title': u'Amazon',
            'uri': u'http://amazontest',
            'preview_info': {
                'item_id': u'test:test',
                'root_code': None,
            },
        }
        with patch.object(amazon, 'Amazon') as mock_amz:
            with patch.object(amazon, 'markup_cleaner') as mock_cleaner:
                mock_amz.return_value = FakeAmazon(lookup="details.xml")
                mock_cleaner.return_value = "Cleaned test text"
                response = list(app.preview(result=result, locale='C',
                                            country='US', partner=None))

        mock_amz.assert_called_with(app.urlread, '', '', 'usshopcode', 'US')
        mock_cleaner.assert_called_with("Test text for EditorialReview")

        r = response[0]
        self.assertEqual(r, dict(widget={
            'id': 'imagegallery',
            'sources': ['fake:large_image'],
            'type': 'gallery'
        }))

        r = response[1]
        self.assertEqual(r, dict(widget={
            'id': 'header1',
            'type': 'header',
            'attribute-1': None,
            'attribute-2': None,
            'attribute-3': None,
            'subtitle': u'$42',
            'title': u'Amazon',
            'emblem': None,
            'uri': u'http://amazontest',
        }))

        r = response[2]
        self.assertEqual(r, dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'label': 'View on Amazon',
                         'uri': 'http://amazontest',
                         'id': 'action1'}]
        }))

        r = response[3]
        self.assertEqual(r, dict(widget={
            'id': 'text1',
            'type': 'text',
            'title': 'Description',
            'text': 'Cleaned test text',
            'uri': u'http://amazontest',
        }))

    def test_preview_with_features(self):
        app = amazon.App(CONFIG)
        result = {
            'art': u'arttest',
            'cat_id': 'cat_0',
            'dnd_uri': u'http://amazontest',
            'price': u'$42',
            'title': u'Amazon',
            'uri': u'http://amazontest',
            'preview_info': {
                'item_id': u'test:test',
                'root_code': None,
            },
        }
        with patch.object(amazon, 'Amazon') as mock_amz:
            mock_amz.return_value = FakeAmazon(lookup="features.xml")
            response = list(app.preview(result=result, locale='C',
                                        country='US', partner=None))

        mock_amz.assert_called_with(app.urlread, '', '', 'usshopcode', 'US')

        r = response[0]
        self.assertEqual(r, dict(widget={
            'id': 'imagegallery',
            'sources': ['http://img.amazon.com/images/I/412AzvqE6gL.jpg'],
            'type': 'gallery'
        }))

        r = response[1]
        self.assertEqual(r, dict(widget={
            'id': 'header1',
            'type': 'header',
            'attribute-1': None,
            'attribute-2': None,
            'attribute-3': None,
            'subtitle': u'$42',
            'title': u'Amazon',
            'emblem': None,
            'uri': u'http://amazontest',
        }))

        r = response[2]
        self.assertEqual(r, dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'label': 'View on Amazon',
                         'uri': 'http://amazontest',
                         'id': 'action1'}]
        }))

        r = response[3]
        self.assertEqual(r, dict(widget={
            'id': 'text2',
            'type': 'text',
            'title': 'Features and Details',
            'text': (u' * Test feature text 1\n'
                     u' * Test feature text 2\n'
                     u" * Test feature text 3"),
            'uri': u'http://amazontest',
        }))

    def test_preview_no_details(self):
        app = amazon.App(CONFIG)
        result = {
            'art': u'arttest',
            'cat_id': 'cat_0',
            'dnd_uri': u'http://amazontest',
            'price': u'$42',
            'title': u'Amazon',
            'uri': u'http://amazontest',
            'preview_info': {
                'item_id': u'test:test',
                'root_code': None,
            },
        }
        with patch.object(amazon, 'details') as mock:
            mock.return_value = None
            response = list(app.preview(result=result, locale='C',
                                        country='US', partner=None))

        self.assertEqual(response, [])


class ConfigTestCase(TestCase):
    """Check some configs are ok."""

    def test_shops(self):
        app = amazon.App(CONFIG)
        generated_from_web_page = set(app.sorting_info['__descriptions__'])
        curated_by_hand = set(amazon.SORTING_INFO)
        self.assertEqual(generated_from_web_page, curated_by_hand)

    def test_subtitles_info_for_nodep(self):
        app = amazon.App(CONFIG)
        # None is built, and do a sanity check
        self.assertIn(None, app.subtitle_attribs)
        self.assertIn('x:Brand', app.subtitle_attribs[None])

    def test_all_roots_have_sorting(self):
        app = amazon.App(CONFIG)
        for shop, shop_data in app.departments_map.items():
            root_cats = set(x[0] for x in shop_data.values())
            missing = root_cats - set(app.sorting_info[shop])
            self.assertFalse(missing, "Missing in %r: %s" % (shop, missing))

    def test_all_roots_have_subtitles(self):
        app = amazon.App(CONFIG)
        for shop, shop_data in app.departments_map.items():
            root_cats = set(x[0] for x in shop_data.values())
            missing = root_cats - set(app.subtitle_attribs)
            self.assertFalse(missing, "Missing in %r: %s" % (shop, missing))


class HelpersTestCase(TestCase):
    """Test some helping functions."""

    def test_get_department_info_ok(self):
        mock_amz = Mock()
        mock_amz.BrowseNodeLookup = mock_bnl = Mock()
        mock_bnl.side_effect = [
            get_fixture("amazon_nodebrowse_base.xml"),
            get_fixture("amazon_nodebrowse_1.xml"),
            get_fixture("amazon_nodebrowse_2.xml"),
        ]
        parent, current, children = amazon._get_department_info(mock_amz,
                                                                'dep_id')
        self.assertEqual(mock_bnl.call_args_list, [
            call(BrowseNodeId='dep_id', ResponseGroup='BrowseNodeInfo'),
            call(BrowseNodeId='2619526011', ResponseGroup='BrowseNodeInfo'),
            call(BrowseNodeId='2645269011', ResponseGroup='BrowseNodeInfo'),
        ])

        self.assertEqual(parent, None)
        self.assertEqual(current, {'name': 'Appliances', 'id': '2619525011'})
        self.assertEqual(children, [
            {'has_children': True, 'name': 'Categories', 'id': '2619526011'},
            {'has_children': True, 'name': 'Featured', 'id': '2645269011'},
        ])

    def test_get_department_info_fail(self):
        mock_amz = Mock()
        mock_amz.BrowseNodeLookup = mock_bnl = Mock()
        mock_bnl.side_effect = [
            get_fixture("amazon_nodebrowse_base.xml"),
            HTTPError(None, 503, "Service Unavailable", None, None),
            get_fixture("amazon_nodebrowse_2.xml"),
        ]
        parent, current, children = amazon._get_department_info(mock_amz,
                                                                'dep_id')
        # the has_children=True of first node is hardcoded after the 503 fail
        self.assertEqual(children, [
            {'has_children': True, 'name': 'Categories', 'id': '2619526011'},
            {'has_children': True, 'name': 'Featured', 'id': '2645269011'},
        ])

    def test_get_department_tree_allroot(self):
        root_dep = {
            'id1': ('code1', 'name1'),
            'id2': ('code2', 'name2'),
        }
        filters = {'filter_id': ['stuff']}
        encf = urllib.quote(json.dumps(filters))
        resp = amazon._get_department_tree('amz', root_dep, None, None,
                                           'te_ST', filters)
        self.assertEqual(resp, {
            'canned_query': 'scope://com.canonical.scopes.amazon?'
                            'filters=' + encf,
            'subdepartments': [{
                'canned_query': 'scope://com.canonical.scopes.amazon?'
                                'dep=code1:id1&filters=' + encf,
                'has_subdepartments': True,
                'label': 'name1',
            }, {
                'canned_query': 'scope://com.canonical.scopes.amazon?'
                                'dep=code2:id2&filters=' + encf,
                'has_subdepartments': True,
                'label': 'name2',
            }],
            'label': 'All departments',
        })

    def test_get_department_tree_specific_node_middle(self):
        parent = dict(name='parentname', id='parentid')
        current = dict(name='currentname', id='currentid')
        children = [
            dict(name='childname1', id='childid1', has_children=True),
            dict(name='childname2', id='childid2', has_children=False),
        ]
        filters = {'filter_id': ['stuff']}
        encf = urllib.quote(json.dumps(filters))
        with patch.object(amazon, '_get_department_info') as mock:
            mock.return_value = (parent, current, children)
            resp = amazon._get_department_tree(
                'amz', 'roots', 'dep_code', 'dep_id', 'te_ST', filters)

        mock.assert_called_with('amz', 'dep_id')
        self.assertEqual(resp, {
            'canned_query': 'scope://com.canonical.scopes.amazon?'
                            'dep=dep_code:parentid&filters=' + encf,
            'subdepartments': [{
                'canned_query': 'scope://com.canonical.scopes.amazon?'
                                'dep=dep_code:currentid&filters=' + encf,
                'subdepartments': [{
                    'canned_query': 'scope://com.canonical.scopes.amazon?'
                                    'dep=dep_code:childid1&filters=' + encf,
                    'has_subdepartments': True,
                    'label': 'childname1',
                }, {
                    'canned_query': 'scope://com.canonical.scopes.amazon?'
                                    'dep=dep_code:childid2&filters=' + encf,
                    'has_subdepartments': False,
                    'label': 'childname2',
                }],
                'label': 'currentname',
            }],
            'label': 'parentname',
        })

    def test_buildcanned_empty(self):
        r = amazon._build_canned()
        self.assertEqual(r, 'scope://com.canonical.scopes.amazon')
        r = amazon._build_canned(dep_code='notenough')
        self.assertEqual(r, 'scope://com.canonical.scopes.amazon')
        r = amazon._build_canned(dep_id='notenough')
        self.assertEqual(r, 'scope://com.canonical.scopes.amazon')

    def test_buildcanned_only_dep(self):
        r = amazon._build_canned('dep_code', 'dep_id')
        self.assertEqual(
            r, 'scope://com.canonical.scopes.amazon?dep=dep_code:dep_id')

    def test_buildcanned_only_filter(self):
        r = amazon._build_canned(filters='somefilter')
        self.assertEqual(
            r, 'scope://com.canonical.scopes.amazon?filters=somefilter')

    def test_buildcanned_everything(self):
        r = amazon._build_canned('dep_code', 'dep_id', 'somefilter')
        self.assertEqual(
            r,
            'scope://com.canonical.scopes.amazon?'
            'dep=dep_code:dep_id&filters=somefilter')

    def test_getsortingfilters_structure(self):
        allowed = ['price', 'runtime']
        new_srt, new_state = amazon._get_sorting_filters(allowed, 'en_US', {})
        self.assertEqual(new_srt, {
            'filter_type': 'option_selector',
            'id': 'sorting_primary_filter',
            'label': 'Sort option',
            'display_hints': 'primary',
            'multi_select': False,
            'options': [
                {'id': amazon.SORTING_DEFAULT, 'label': 'Featured'},
                {'id': 'price', 'label': amazon.SORTING_INFO['price']},
                {'id': 'runtime', 'label': amazon.SORTING_INFO['runtime']},
            ],
        })
        self.assertEqual(new_state,
                         {'sorting_primary_filter': [amazon.SORTING_DEFAULT]})

    def test_getsortingfilters_order_by_label(self):
        allowed = ['price', 'salesrank', 'runtime']
        assert allowed != sorted(allowed, key=lambda k: amazon.SORTING_INFO[k])
        new_srt, _ = amazon._get_sorting_filters(allowed, 'en_US', {})
        self.assertEqual(new_srt['options'], [
            {'id': 'salesrank', 'label': amazon.SORTING_INFO['salesrank']},
            {'id': amazon.SORTING_DEFAULT, 'label': 'Featured'},
            {'id': 'price', 'label': amazon.SORTING_INFO['price']},
            {'id': 'runtime', 'label': amazon.SORTING_INFO['runtime']},
        ])

    def test_getsortingfilters_current_state_untouched(self):
        allowed = ['price', 'salesrank', 'runtime']
        current = {amazon.SORTING_FILTER_ID: ['price'], 'otherid': ['foo']}
        _, new_state = amazon._get_sorting_filters(allowed, 'es_ES', current)
        self.assertEqual(new_state, current)

    def test_getsortingfilters_current_state_fixed(self):
        allowed = ['price', 'salesrank', 'runtime']
        current = {amazon.SORTING_FILTER_ID: ['different'], 'otherid': ['foo']}
        _, new_state = amazon._get_sorting_filters(allowed, 'es_ES', current)
        self.assertEqual(new_state, {'otherid': ['foo']})

    def test_parse_item_subtitle_first(self):
        raw = get_fixture("amazon_search.xml")
        xml = etree.XML(raw)
        item = xml.xpath("//x:Item", namespaces=amazon.AMZ_NS)[0]

        # check passing first Brand
        resp = amazon._parse_item(item, 'US', ['x:Brand', 'x:Binding'])
        self.assertEqual(resp['subtitle'], 'TestBrand')

        # then check passing first Binding
        resp = amazon._parse_item(item, 'US', ['x:Binding', 'x:Brand'])
        self.assertEqual(resp['subtitle'], 'TestBinding')

    def test_parse_item_subtitle_default(self):
        raw = get_fixture("amazon_search.xml")
        xml = etree.XML(raw)
        item = xml.xpath("//x:Item", namespaces=amazon.AMZ_NS)[0]

        # check with something that won't be in the XML
        resp = amazon._parse_item(item, 'US', ['x:WontFind'])
        self.assertEqual(resp['subtitle'], '')

    def test_parse_item_subtitle_several(self):
        raw = get_fixture("amazon_search.xml")
        xml = etree.XML(raw)
        item = xml.xpath("//x:Item", namespaces=amazon.AMZ_NS)[0]

        # check with something that is repeated
        resp = amazon._parse_item(item, 'US', ['x:Author'])
        self.assertEqual(resp['subtitle'],
                         'TestAuthor1, TestAuthor2, TestAuthor3')

    def test_pickimage_simple(self):
        sources = {
            '120x120': ['image1'],
            '80x80': ['image2'],
        }
        r = amazon._pick_images(sources, (100, 100))
        self.assertEqual(r, ['image1'])

    def test_pickimage_several(self):
        sources = {
            '120x120': ['image1'],
            '80x80': ['image2'],
            '200x200': ['image3', 'image4'],
            '150x150': ['image5'],
        }
        r = amazon._pick_images(sources, (100, 100))
        self.assertEqual(r, ['image3', 'image4', 'image5', 'image1'])

    def test_pickimage_allsmaller(self):
        sources = {
            '80x80': ['image1'],
            '70x70': ['image2', 'image3'],
        }
        r = amazon._pick_images(sources, (100, 100))
        self.assertEqual(r, ['image1'])

    def test_pickimage_empty(self):
        r = amazon._pick_images({}, (100, 100))
        self.assertEqual(r, [])

    def test_pickimage_onlyone(self):
        sources = {
            '80x80': ['image1'],
        }
        r = amazon._pick_images(sources, (100, 100))
        self.assertEqual(r, ['image1'])

    def test_pickimage_greater_than_both(self):
        sources = {
            '80x80': ['image1'],
            '70x80': ['image2'],
            '80x70': ['image3'],
            '70x70': ['image4'],
        }
        r = amazon._pick_images(sources, (75, 75))
        self.assertEqual(r, ['image1', 'image3', 'image2'])

    def test_process_department_nodepartment(self):
        dep_root, dep_id = amazon._process_department(None, {})
        self.assertEqual(dep_root, None)
        self.assertEqual(dep_id, None)

    def test_process_department_normal(self):
        dep_root, dep_id = amazon._process_department("Foo:Bar", {})
        self.assertEqual(dep_root, "Foo")
        self.assertEqual(dep_id, "Bar")

    def test_process_department_simple_existant(self):
        dep_map = {'1234': ('Foo', "Bar baz, etc")}
        dep_root, dep_id = amazon._process_department("Foo", dep_map)
        self.assertEqual(dep_root, "Foo")
        self.assertEqual(dep_id, "1234")

    def test_process_department_simple_nonexistant(self):
        dep_map = {'1234': ('Foo', "Bar baz, etc")}
        dep_root, dep_id = amazon._process_department("NotThere", dep_map)
        self.assertEqual(dep_root, None)
        self.assertEqual(dep_id, None)
