# -*- coding: utf8 -*-

# Copyright 2014-2015 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import json
import socket
import types
import urllib

from httplib import HTTPException
from unittest import TestCase
from urllib2 import HTTPError, URLError

from mock import Mock, ANY, patch

from src import scopes, utils
from src.scopes.tests import SetupLogChecker


class CallTestCase(TestCase):
    """Test when calling the class."""

    def setUp(self):
        self.meter = Mock()
        self.app = scopes.SimpleApp(config={}, metrics=(self.meter, 'base'))
        self.app.scope_module_name = 'test scope name'
        self.app._get_search_args = check_mock = Mock()
        validated_args = dict(some='args', foo='bar', query='testquery',
                              session_id='test_session', locale='testlocale',
                              country='testcountry', partner='test partner')
        check_mock.return_value = validated_args

        self.start_response = Mock()
        SetupLogChecker(self, 'restscopes')

    def test_endpoints(self):
        self.app.error = mock = Mock()
        mock.return_value = 'error response'

        environ = {
            'PATH_INFO': '/bad_endpoint',  # 'search' or 'preview' only
        }
        resp = self.app(environ, 'start_response')
        self.assertEqual(resp, 'error response')
        mock.assert_called_with('404 NOT FOUND', 'start_response')
        self.meter.timing.assert_called_with('base.scopes.404', ANY)

    def test_endpoints_search(self):

        class TestApp(scopes.SimpleApp):
            """App class with surface_headers."""
            surface_headers = [('foo', 'bar')]

        app = TestApp(config={}, metrics=(self.meter, 'base'))
        app.scope_module_name = 'test scope name'
        app._get_search_args = check_mock = Mock()
        validated_args = dict(some='args', foo='bar', query='testquery',
                              session_id='test_session', locale='testlocale',
                              country='testcountry', partner='test partner')
        check_mock.return_value = validated_args
        app.search = action_mock = Mock()
        action_mock.return_value = iter([{'resp': 'data'}, [1, 2]])

        environ = {
            'PATH_INFO': '/search',
            'FOO': 'bar',
        }
        chunked = app(environ, self.start_response)
        self.assertTrue(isinstance(chunked, types.GeneratorType))
        response = list(chunked)
        check_mock.assert_called_with(environ)
        action_mock.assert_called_with(**validated_args)
        self.assertEqual(response, ['{"resp": "data"}\r\n', '[1, 2]\r\n'])
        self.start_response.assert_called_with("200 OK", [
            ('Content-type', 'application/json')])
        self.assertLogged("Start search", "testquery", "session", "testlocale",
                          "testcountry", 'test scope name', 'test partner')
        self.meter.timing.assert_called_with('base.test_base.search.200', ANY)

    def test_endpoints_surface_headers(self):

        class TestApp(scopes.SimpleApp):
            """App class with surface_headers."""
            surface_headers = [('foo', 'bar')]

        app = TestApp(config={}, metrics=(self.meter, 'base'))
        app.scope_module_name = 'test scope name'
        app._get_search_args = check_mock = Mock()
        validated_args = dict(some='args', foo='bar', query='',
                              session_id='test_session', locale='testlocale',
                              country='testcountry', partner='test partner')
        check_mock.return_value = validated_args
        app.search = action_mock = Mock()
        action_mock.return_value = iter([{'resp': 'data'}, [1, 2]])

        environ = {
            'PATH_INFO': '/search',
            'FOO': 'bar',
        }
        chunked = app(environ, self.start_response)
        self.assertTrue(isinstance(chunked, types.GeneratorType))
        response = list(chunked)
        check_mock.assert_called_with(environ)
        action_mock.assert_called_with(**validated_args)
        self.assertEqual(response, ['{"resp": "data"}\r\n', '[1, 2]\r\n'])
        self.start_response.assert_called_with("200 OK", [
            ('foo', 'bar')])
        self.assertLogged("Start search", "session", "testlocale",
                          "testcountry", 'test scope name', 'test partner')
        self.meter.timing.assert_called_with('base.test_base.search.200', ANY)

    def test_endpoints_preview(self):
        app = scopes.SimpleApp(config={}, metrics=(self.meter, 'base'))
        app.scope_module_name = 'test scope name'
        app._get_preview_args = check_mock = Mock()
        validated_args = dict(some='args', foo='bar', country='testcountry',
                              session_id='test_session', locale='testlocale',
                              partner='test partner')
        check_mock.return_value = validated_args
        app.preview = action_mock = Mock()
        action_mock.return_value = iter([{'resp': 'data'}, [1, 2]])

        environ = {
            'PATH_INFO': '/preview',
            'FOO': 'bar',
        }
        chunked = app(environ, self.start_response)
        self.assertTrue(isinstance(chunked, types.GeneratorType))
        response = list(chunked)
        check_mock.assert_called_with(environ)
        action_mock.assert_called_with(**validated_args)
        self.assertEqual(response, ['{"resp": "data"}\r\n', '[1, 2]\r\n'])
        self.start_response.assert_called_with("200 OK", [
            ('Content-type', 'application/json')])
        self.assertLogged("Start preview", "session", "testlocale",
                          "testcountry", 'test scope name', 'test partner')
        self.meter.timing.assert_called_with('base.scopes.preview.200', ANY)

    def test_endpoints_bad_parameters(self):
        self.app._get_search_args = check_mock = Mock()
        check_mock.side_effect = ValueError("Test message")

        environ = {
            'PATH_INFO': '/search',
            'FOO': 'bar',
        }
        resp = self.app(environ, self.start_response)
        self.assertEqual(json.loads("".join(resp))['error_msg'],
                         "Problem in parameters: Test message")
        self.assertEqual(self.start_response.call_args[0][0],
                         "400 BAD REQUEST")
        self.meter.timing.assert_called_with('base.scopes.search.400', ANY)

    def test_endpoints_missing_parameters(self):
        self.app._get_preview_args = check_mock = Mock()
        check_mock.side_effect = KeyError("someparameter")

        environ = {
            'PATH_INFO': '/preview',
            'FOO': 'bar',
        }
        resp = self.app(environ, self.start_response)
        self.assertEqual(json.loads("".join(resp))['error_msg'],
                         "Missing parameter 'someparameter'")
        self.assertEqual(self.start_response.call_args[0][0],
                         "400 BAD REQUEST")
        self.meter.timing.assert_called_with('base.scopes.preview.400', ANY)

    def test_failure_immediate(self):
        def _f(*a, **k):
            """Fake function emulating the backend."""
            exc = HTTPError(None, 503, "Service Unavailable", None, None)
            raise exc
            yield  # this yield is because we want _f to be a generator!
        self.app.search = _f

        chunked = self.app({'PATH_INFO': '/search'}, self.start_response)
        self.assertTrue(isinstance(chunked, types.GeneratorType))
        response = list(chunked)
        self.assertEqual(json.loads(response[0]), {
            "error_msg": "Original error: HTTP Error 503: Service Unavailable",
            "error": "502 BAD GATEWAY",
        })
        self.assertEqual(self.start_response.call_args[0][0],
                         "502 BAD GATEWAY")
        self.assertLogged("Start search", "testquery", "session", "testlocale",
                          "testcountry", 'test scope name', 'test partner')
        self.meter.timing.assert_called_with('base.scopes.search.503', ANY)

    def test_failure_delayed(self):
        def _f(*a, **k):
            """Fake function emulating the backend."""
            for r in ["somedata", HTTPError(None, 503, None, None, None)]:
                if isinstance(r, Exception):
                    raise r
                else:
                    yield r
        self.app.search = _f

        chunked = self.app({'PATH_INFO': '/search'}, self.start_response)
        self.assertTrue(isinstance(chunked, types.GeneratorType))
        response = list(chunked)
        self.assertEqual(response, [json.dumps("somedata") + "\r\n"])
        self.start_response.assert_called_with("200 OK", [
            ('Content-type', 'application/json')])
        self.meter.timing.assert_called_with('base.scopes.search.503', ANY)

    def test_empty_response(self):
        self.app.search = lambda *a, **k: iter([])
        chunked = self.app({'PATH_INFO': '/search'}, self.start_response)
        self.assertTrue(isinstance(chunked, types.GeneratorType))
        response = list(chunked)
        self.assertEqual(response, [])
        self.start_response.assert_called_with("200 OK", [])
        self.meter.timing.assert_called_with('base.scopes.search.200', ANY)

    def test_delayed_finding_of_bad_request(self):
        self.app.search = check_mock = Mock()
        check_mock.side_effect = scopes.BadRequestError("Missing parameter X")

        chunked = self.app({'PATH_INFO': '/search'}, self.start_response)
        self.assertTrue(isinstance(chunked, types.GeneratorType))
        response = list(chunked)
        self.assertEqual(json.loads("".join(response))['error_msg'],
                         "Bad request: Missing parameter X")
        self.assertEqual(self.start_response.call_args[0][0],
                         "400 BAD REQUEST")
        self.meter.timing.assert_called_with('base.scopes.search.400', ANY)

    def test_avoidable_query(self):
        # use a silly query
        test_query = 'e' * 10
        assert utils.avoidable_query(test_query)

        # set up the app to return that query when getting args
        app = scopes.SimpleApp(config={}, metrics=(self.meter, 'base'))
        app._get_search_args = check_mock = Mock()
        check_mock.return_value = dict(query=test_query)

        # call and check everything
        response = app({'PATH_INFO': '/search'}, self.start_response)
        self.assertEqual(response, [''])
        self.start_response.assert_called_with("200 OK", [])
        self.assertLogged("Avoiding query", test_query)
        self.meter.timing.assert_called_with('base.scopes.search.200', ANY)

    def test_backend_data_error_exception(self):
        self.app.search = check_mock = Mock()
        check_mock.side_effect = scopes.BackendDataError(ValueError("pumba"), 'test url')
        response = list(self.app({'PATH_INFO': '/search'}, self.start_response))
        self.assertEqual(response, [])
        self.start_response.assert_called_with("200 OK", [])
        self.meter.timing.assert_any_call('base.scopes.search.200', ANY)
        self.meter.timing.assert_any_call('base.scopes.search.backend-data-error', ANY)
        self.assertLogged("Backend data error", "ValueError", "pumba", "test url")

    def test_backend_data_error_message(self):
        self.app.search = check_mock = Mock()
        check_mock.side_effect = scopes.BackendDataError('All is broken', 'test url')
        response = list(self.app({'PATH_INFO': '/search'}, self.start_response))
        self.assertEqual(response, [])
        self.start_response.assert_called_with("200 OK", [])
        self.meter.timing.assert_any_call('base.scopes.search.200', ANY)
        self.meter.timing.assert_any_call('base.scopes.search.backend-data-error', ANY)
        self.assertLogged("Backend data error", "All is broken", "test url")


class SearchParamsValidationTestCase(TestCase):
    """Test the validation of parameters needed in the search."""

    def setUp(self):
        self.app = scopes.SimpleApp(config={})

    def test_simple_ok(self):
        q = 'q=foo&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_query_unicode(self):
        q = 'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'moño',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_latitude_longitude(self):
        q = (
            'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es_AR'
            '&latitude=1.2312&longitude=3.13123'
        )
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'moño',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': 1.2312,
            'longitude': 3.13123,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_latitude_only(self):
        q = (
            'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es_AR'
            '&latitude=1.2312'
        )
        msg = "latitude and longitude must be paired, both present or none"
        with self.assertRaises(ValueError) as cm:
            self.app._get_search_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), msg)

    def test_longitude_only(self):
        q = (
            'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es_AR'
            '&longitude=1.2312'
        )
        msg = "latitude and longitude must be paired, both present or none"
        with self.assertRaises(ValueError) as cm:
            self.app._get_search_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), msg)

    def test_no_locale_no_country_no_limit(self):
        q = 'q=mo\xc3\xb1o&platform=desktop'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'moño',
            'session_id': None,
            'query_id': None,
            'locale': utils.DEFAULT_LOCALE,
            'platform': 'desktop',
            'country': scopes.DEFAULT_COUNTRY,
            'latitude': None,
            'longitude': None,
            'limit': scopes.DEFAULT_LIMIT,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_bad_locale(self):
        q = 'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=es|AR'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'moño',
            'session_id': None,
            'query_id': None,
            'locale': utils.DEFAULT_LOCALE,
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_locale_C(self):
        q = 'q=mo\xc3\xb1o&platform=desktop&country=ar&limit=4&locale=C'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'moño',
            'session_id': None,
            'query_id': None,
            'locale': 'en_US',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_empty_query(self):
        q = 'q=&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_query_decode(self):
        q = 'q=cami%C3%B3n&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'camión',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_with_client_ip(self):
        q = 'q=foo&platform=desktop&country=ar&limit=4&locale=es_AR'
        resp = self.app._get_search_args(
            {'QUERY_STRING': q,
             'HTTP_X_FORWARDED_FOR': '192.168.1.1'})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': '192.168.1.1',
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_default_limit_override(self):
        q = 'q=foo&platform=desktop'
        self.app.default_limit = 25
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'platform': 'desktop',
            'locale': 'en',
            'country': 'US',
            'latitude': None,
            'longitude': None,
            'limit': 25,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_department(self):
        q = 'q=foo&platform=desktop&department=shoes'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'platform': 'desktop',
            'locale': 'en',
            'country': 'US',
            'latitude': None,
            'longitude': None,
            'limit': 10,
            'client_ip': None,
            'department': 'shoes',
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_filters(self):
        q = 'q=foo&platform=desktop&filters=%7B%22f1%22%3A%5B%22o1%22%5D%7D%0A'
        resp = self.app._get_search_args({'QUERY_STRING': q})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'platform': 'desktop',
            'locale': 'en',
            'country': 'US',
            'latitude': None,
            'longitude': None,
            'limit': 10,
            'client_ip': None,
            'department': None,
            'filters': {"f1": ["o1"]},
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_session_id(self):
        q = 'q=foo&platform=desktop&country=ar&limit=4&locale=es_AR'
        args = {'QUERY_STRING': q, 'HTTP_X_REST_SCOPES_SESSION_ID': 'test'}
        resp = self.app._get_search_args(args)
        should = {
            'query': u'foo',
            'session_id': 'test',
            'query_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'country': 'ar',
            'latitude': None,
            'longitude': None,
            'limit': 4,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_partner(self):
        q = 'q=foo&platform=desktop'
        resp = self.app._get_search_args({'QUERY_STRING': q,
                                          'USER_AGENT': {'partner': 'gruk'}})
        should = {
            'query': u'foo',
            'session_id': None,
            'query_id': None,
            'platform': 'desktop',
            'locale': 'en',
            'country': 'US',
            'latitude': None,
            'longitude': None,
            'limit': 10,
            'client_ip': None,
            'department': None,
            'filters': None,
            'partner': 'gruk',
        }
        self.assertEqual(resp, should)


class PreviewParamsValidationTestCase(TestCase):
    """Test the validation of parameters needed in the preview."""

    def setUp(self):
        self.app = scopes.SimpleApp(config={})
        self.prev_res_dict = dict(example='foo', stuff=23)
        self.prev_res = urllib.quote(json.dumps(self.prev_res_dict))

    def test_simple_ok(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'client_ip': None,
            'session_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'result': self.prev_res_dict,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_no_locale_no_country(self):
        q = 'platform=desktop&widgets_api_version=1&result=' + self.prev_res
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'client_ip': None,
            'session_id': None,
            'locale': utils.DEFAULT_LOCALE,
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': scopes.DEFAULT_COUNTRY,
            'result': self.prev_res_dict,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_bad_locale(self):
        q = (
            'platform=desktop&locale=es|AR&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'client_ip': None,
            'session_id': None,
            'locale': utils.DEFAULT_LOCALE,
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'result': self.prev_res_dict,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_locale_C(self):
        q = (
            'platform=desktop&locale=C&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'client_ip': None,
            'session_id': None,
            'locale': 'en_US',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'result': self.prev_res_dict,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_broken_prev_result(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=|no|json|'
        )
        with self.assertRaises(ValueError) as cm:
            self.app._get_preview_args({'QUERY_STRING': q})
            self.assertEqual(str(cm.exception), 'Broken previous result')

    def test_decode_prev_result(self):
        prev_res_dict = dict(example=u'moño', stuff=23)
        prev_res = urllib.quote(json.dumps(prev_res_dict))
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=' + prev_res
        )
        resp = self.app._get_preview_args({'QUERY_STRING': q})
        should = {
            'session_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'result': prev_res_dict,
            'client_ip': None,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_with_client_ip(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        resp = self.app._get_preview_args(
            {'QUERY_STRING': q,
             'HTTP_X_FORWARDED_FOR': '192.168.1.1'})
        should = {
            'client_ip': '192.168.1.1',
            'session_id': None,
            'locale': 'es_AR',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'result': self.prev_res_dict,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_session_id(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        args = {'QUERY_STRING': q, 'HTTP_X_REST_SCOPES_SESSION_ID': 'test'}
        resp = self.app._get_preview_args(args)
        should = {
            'client_ip': None,
            'session_id': 'test',
            'locale': 'es_AR',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'result': self.prev_res_dict,
            'partner': None,
        }
        self.assertEqual(resp, should)

    def test_partner(self):
        q = (
            'platform=desktop&locale=es_AR&country=ar&widgets_api_version=1'
            '&result=' + self.prev_res
        )
        args = {'QUERY_STRING': q, 'HTTP_X_REST_SCOPES_SESSION_ID': 'test',
                'USER_AGENT': {'partner': 'grok'}}

        resp = self.app._get_preview_args(args)
        should = {
            'client_ip': None,
            'session_id': 'test',
            'locale': 'es_AR',
            'platform': 'desktop',
            'widgets_api_version': 1,
            'country': 'ar',
            'result': self.prev_res_dict,
            'partner': 'grok',
        }
        self.assertEqual(resp, should)


class HelpersTestCase(TestCase):
    """Test some helper methods."""

    def setUp(self):
        self.app = scopes.SimpleApp(config={})
        self.start_response = Mock()

    def test_error_sanity(self):
        resp = self.app.error("400 test", self.start_response,
                              "test message")
        should_error_message = json.dumps(
            {'error': '400 test', 'error_msg': 'test message'}, indent=4)

        self.assertEqual(resp, [should_error_message])
        should_status = "400 test"
        should_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(should_error_message))),
        ]
        self.start_response.assert_called_with(should_status, should_headers)

    def test_urlread(self):
        app = scopes.SimpleApp(config={}, metrics=('meter', 'base'))
        with patch.object(scopes.rest_scopes, 'urlread') as mock_urlread:
            mock_urlread.return_value = 'response'
            resp = app.urlread('http://blah/', data='data', headers='headers')

        self.assertEqual(resp, 'response')
        mock_urlread.assert_called_with('meter', 'base.scopes', 'http://blah/', 'data', 'headers')


class NetworkHandlingtCase(TestCase):
    """Check all the network failure handling."""

    def setUp(self):
        self.meter = Mock()
        self.app = scopes.SimpleApp(config={}, metrics=(self.meter, 'base'))
        self.start_response = Mock()
        self.app._get_search_args = check_mock = Mock()
        validated_args = dict(some='args', foo='bar', query='testquery',
                              session_id='test_session', locale='testlocale',
                              country='testcountry', partner='test partner')
        check_mock.return_value = validated_args

    def _test_failure(self, exc, msg, code):
        """Help to check a network failure handling."""
        def _f(*a, **k):
            """Fake function emulating the backend."""
            raise exc
            yield  # this yield is because we want _f to be a generator!
        self.app.search = _f

        resp = list(self.app({'PATH_INFO': '/search'}, self.start_response))
        self.assertEqual(json.loads(resp[0]), {
            "error_msg": "Original error: " + msg,
            "error": "502 BAD GATEWAY",
        })
        self.assertEqual(self.start_response.call_args[0][0],
                         "502 BAD GATEWAY")
        self.meter.timing.assert_called_with('base.scopes.search.' + code, ANY)

    def _test_error(self, exc):
        """Help to check a network failure that shouldn't be handled."""
        def _f(*a, **k):
            """Fake function emulating the backend."""
            raise exc
            yield  # this yield is because we want _f to be a generator!
        self.app.search = _f

        try:
            list(self.app({'PATH_INFO': '/search'}, self.start_response))
        except Exception as real_exc:
            self.assertEqual(real_exc, exc)

    def test_httperror_400(self):
        exc = HTTPError(None, 400, "Bad Request", None, None)
        self._test_error(exc)

    def test_httperror_429(self):
        exc = HTTPError(None, 429, "Rate-Limited Non Standard Err", None, None)
        msg = "HTTP Error 429: Rate-Limited Non Standard Err"
        self._test_failure(exc, msg, '429')

    def test_httperror_500(self):
        exc = HTTPError(None, 500, "Internal Server Error", None, None)
        msg = "HTTP Error 500: Internal Server Error"
        self._test_failure(exc, msg, '500')

    def test_httperror_501(self):
        exc = HTTPError(None, 501, "Not Implemented", None, None)
        msg = "HTTP Error 501: Not Implemented"
        self._test_failure(exc, msg, '501')

    def test_httperror_502(self):
        exc = HTTPError(None, 502, "Bad Gateway", None, None)
        msg = "HTTP Error 502: Bad Gateway"
        self._test_failure(exc, msg, '502')

    def test_httperror_503(self):
        exc = HTTPError(None, 503, "Service Unavailable", None, None)
        msg = "HTTP Error 503: Service Unavailable"
        self._test_failure(exc, msg, '503')

    def test_httperror_504(self):
        exc = HTTPError(None, 504, "Gateway Time-out", None, None)
        msg = "HTTP Error 504: Gateway Time-out"
        self._test_failure(exc, msg, '504')

    def test_urlerror_simple(self):
        exc = URLError("Spurious network issue")
        msg = "URLError: Spurious network issue"
        self._test_failure(exc, msg, 'urlerror')

    def test_urlerror_socket(self):
        error = socket.error("Spurious network issue")
        exc = URLError(error)
        msg = "URLError: Spurious network issue"
        self._test_failure(exc, msg, 'urlerror')

    def test_httpexception(self):
        exc = HTTPException("Spurious network issue")
        msg = "HTTPException: Spurious network issue"
        self._test_failure(exc, msg, 'httpexception')

    def test_crazy_httperrors(self):
        exc = HTTPError(None, 666, "What?", None, None)
        msg = "HTTP Error 666: What?"
        self._test_failure(exc, msg, '666')
        exc = HTTPError(None, 756, "Seriously?", None, None)
        msg = "HTTP Error 756: Seriously?"
        self._test_failure(exc, msg, '756')
