# -*- coding: utf8 -*-

# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import json
import urlparse

from contextlib import nested
from unittest import TestCase

from mock import patch

from src.scopes import ebay
from src.scopes.tests import get_fixture

CONFIG = {'secrets': {None: {'api_key': 'hush', 'campaign': 'campid'}}}

TEST_SEARCH_URI = ('https://svcs.ebay.com/services/search/FindingService/v1'
                   '?SECURITY-APPNAME=hush&OPERATION-NAME=findItemsByKeywords'
                   '&SERVICE-VERSION=1.0.0&RESPONSE-DATA-FORMAT=JSON'
                   '&GLOBAL-ID=EBAY-US&REST-PAYLOAD'
                   '&keywords=paris&paginationInput.entriesPerPage=10'
                   '&outputSelector(0)=PictureURLSuperSize'
                   '&outputSelector(1)=PictureURLLarge'
                   '&affiliate.networkId=9&affiliate.trackingId=campid')

TEST_SURFACING_URI = (
    'http://open.api.ebay.com/shopping?callname=FindPopularItems'
    '&responseencoding=JSON&appid=hush&siteid=0&version=531&'
    'CategoryID=550&MaxEntries=10&trackingpartnercode=9&trackingid=campid')

TEST_SORTING_URI = (
    'https://svcs.ebay.com/services/search/FindingService/v1'
    '?SECURITY-APPNAME=hush&OPERATION-NAME=findItemsByCategory'
    '&SERVICE-VERSION=1.0.0&RESPONSE-DATA-FORMAT=JSON'
    '&GLOBAL-ID=EBAY-US&REST-PAYLOAD'
    '&categoryId=550&sortOrder=SomeSorting&paginationInput.entriesPerPage=10'
    '&outputSelector(0)=PictureURLSuperSize&outputSelector(1)=PictureURLLarge'
    '&itemFilter.name=ListingType&itemFilter.value=Auction'
    '&affiliate.networkId=9&affiliate.trackingId=campid')


# dict as a base for normal search calls
STD_KWARGS = dict(query='paris', limit=10, platform='desktop', filters=None,
                  locale='en', country='us', department='', partner=None)


class SearchTestCase(TestCase):
    """Tests for the search function."""
    maxDiff = None

    def test_product(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-results.json')
                response = list(app.search(**STD_KWARGS))

        # got the info as it should
        mock.assert_called_with(TEST_SEARCH_URI)

        # category
        category_mock = ebay.PRODUCTS_CATEGORY.copy()
        category_mock['title'] = '271775 results'
        should_cat = {'category': category_mock}
        self.assertEqual(response[0], should_cat)

        # the result
        should_res = {'result': {
            'condition': u'Pre-owned',
            'currency_code': u'USD',
            'art': u'http://thumbs3.ebaystatic.com/m/' +
            'mA8ILqmlSWokT6vyEN1KFyw/140.jpg',
            'cat_id': 'cat_0',
            'dnd_uri': u'http://www.ebay.com/itm/Paris-Eiffel-' +
            'Tower-Hook-Black-Pink-EUC-/321313863006?pt=Decals' +
            '_Stickers_Vinyl_Art',
            'image_hint': u'http://i.ebayimg.com/00/s/MTAwMFg3NTA=' +
            '/z/a4cAAOxyVLNS4Ts7/$_3.JPG',
            'payment_methods': u'PayPal',
            'price': u'$4.00',
            's_attributes': [{'value': '1h 30m'}],
            'shipping_to': u'US',
            'title': u'Paris Eiffel Tower Hook Black/Pink EUC',
            'uri': u'http://www.ebay.com/itm/Paris-Eiffel-Tower-Hook-' +
            'Black-Pink-EUC-/321313863006?pt=Decals_Stickers_Vinyl_Art',
            'item_id': u'321313863006',
            'attributes': [{'value': u'$4.00'},
                           {'value': u'1 bids'},
                           {'value': u'$5.00'},
                           {'value': '1h 30m'}],
        }}
        self.assertEqual(response[1], should_res)

    def test_surfacing(self):
        app = ebay.App(CONFIG)
        with nested(
                patch.object(app, '_get_products'),
                patch.object(app, '_get_available_departments'),
                ) as (mock, mdep):
            mdep.return_value = {}, ('550', 'Foo')
            mock.return_value = get_fixture('ebay-surfacing.json')
            kwargs = STD_KWARGS.copy()
            kwargs['query'] = ''
            response = list(app.search(**kwargs))

        # got the info as it should
        mock.assert_called_with(TEST_SURFACING_URI)

        # department and filter
        self.assertEqual(response[0].keys()[0], 'departments')

        # category
        self.assertEqual(response[1]['category']['title'], "Foo")

        # the result
        self.assertEqual(response[2], dict(result={
            'condition': None,
            'currency_code': u'USD',
            'art': u'http://thumbs2.ebaystatic.com/d/l150/pict/' +
            '350682346349808000000001_11.jpg',
            'cat_id': 'cat_0',
            'dnd_uri': u'http://www.ebay.com/itm/Every-Day-Carry-Tactical-' +
            'Assault-Bag-EDC-Day-Pack-Backpack-Molle-Webbing-/' +
            '350682346349?var=620083903946',
            'image_hint': u'http://thumbs2.ebaystatic.com/d/l300/pict/' +
            '350682346349808000000001_11.jpg',
            'payment_methods': None,
            'price': u'$36.92',
            'shipping_to': None,
            'title': u'Every Day Carry Tactical Assault Bag EDC Day ' +
            'Pack Backpack with Molle Webbing',
            'uri': u'http://www.ebay.com/itm/Every-Day-Carry-Tactical-' +
            'Assault-Bag-EDC-Day-Pack-Backpack-Molle-Webbing-/' +
            '350682346349?var=620083903946',
            'item_id': u'350682346349',
            'attributes': [{'value': u'$36.92'},
                           {'value': '3302 bids'},
                           {'value': 'Free'},
                           {'value': '4d 2h'}],
            's_attributes': [{'value': '4d 2h'}],

        }))

    def test_surfacing_no_img(self):
        app = ebay.App(CONFIG)
        with nested(
                patch.object(app, '_get_products'),
                patch.object(app, '_get_available_departments'),
                ) as (mock, mdep):
            mdep.return_value = {}, ('550', 'Bar')
            mock.return_value = get_fixture('ebay-surfacing-no-img.json')
            kwargs = STD_KWARGS.copy()
            kwargs['query'] = ''
            response = list(app.search(**kwargs))

        # got the info as it should
        mock.assert_called_with(TEST_SURFACING_URI)

        # department and filter
        self.assertEqual(response[0].keys()[0], 'departments')

        # category
        self.assertEqual(response[1]['category']['title'], "Bar")

        # Ensure nothing else is there (items without images are not surfaced)
        self.assertEqual(len(response), 2)

    def test_surfacing_with_department(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-surfacing.json')
                kwargs = STD_KWARGS.copy()
                kwargs.update(query='', department='550')
                list(app.search(**kwargs))

        # Fake a surfacing by department uri
        uri = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
               '&responseencoding=JSON&appid=hush&siteid=0&version=531&'
               'CategoryID=550&MaxEntries=10'
               '&trackingpartnercode=9&trackingid=campid')
        mock.assert_called_with(uri)

    def test_surfacing_with_department_noresults(self):
        app = ebay.App(CONFIG)
        empty_response = json.dumps({"Timestamp": "2014-02-20T13:09:46.167Z",
                                     "Build": "E849_CORE_APILW2_16526615_R1",
                                     "Version": "849"})   # no ItemArray!

        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = empty_response
                kwargs = STD_KWARGS.copy()
                kwargs.update(query='', department='550')
                resp = list(app.search(**kwargs))

        self.assertNotIn('result', [r.keys()[0] for r in resp])

    def test_surfacing_filters(self):
        app = ebay.App(CONFIG)
        filters = {ebay.SORTING_ID: ['SomeSorting']}
        with nested(
                patch.object(app, '_get_products'),
                patch.object(app, '_get_available_departments'),
                ) as (mock, mdep):
            mdep.return_value = {}, ('550', 'Bar')
            mock.return_value = get_fixture('ebay-results-sorting.json')
            kwargs = STD_KWARGS.copy()
            kwargs.update(query='', filters=filters)
            response = list(app.search(**kwargs))

        # got the info as it should
        mock.assert_called_with(TEST_SORTING_URI)

        # department and filter
        self.assertEqual(response[0].keys()[0], 'departments')

        # category and more results
        self.assertEqual(response[1]['category']['title'], "Bar")
        self.assertEqual(len(response), 7)

    def test_filters(self):
        filters = {"MaxPrice": 5000,
                   "MinPrice": [10, 30],
                   "foo": "bar"}
        should_filters = {"itemFilter(0).name": ["MinPrice"],
                          "itemFilter(1).value(0)": ['5000'],
                          "itemFilter(1).name": ["MaxPrice"],
                          "itemFilter(0).value(0)": ['10'],
                          "itemFilter(0).value(1)": ['30'], }
        validate = ebay._parse_filters(filters)

        # Ensure only valid Ebay filters are kept and correctly parsed
        self.assertEqual(should_filters, urlparse.parse_qs(validate))

    def test_valid_country(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-surfacing.json')
                kwargs = STD_KWARGS.copy()
                kwargs.update(query='', department='550', country='fr')
                list(app.search(**kwargs))

        # Fake a surfacing by department uri
        uri = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
               '&responseencoding=JSON&appid=hush&siteid=71&version=531&'
               'CategoryID=550&MaxEntries=10'
               '&trackingpartnercode=9&trackingid=campid')
        mock.assert_called_with(uri)

    def test_invalid_country(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-surfacing.json')
                kwargs = STD_KWARGS.copy()
                kwargs.update(query='', department='550', country='foo')
                list(app.search(**kwargs))

        # Fake a surfacing by department uri
        uri = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
               '&responseencoding=JSON&appid=hush&siteid=0&version=531&'
               'CategoryID=550&MaxEntries=10'
               '&trackingpartnercode=9&trackingid=campid')
        mock.assert_called_with(uri)

    def test_valid_country_with_multiple_locales(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-surfacing.json')
                kwargs = STD_KWARGS.copy()
                kwargs.update(query='', department='550', country='be')
                list(app.search(**kwargs))

        # Fake a surfacing by department uri
        uri = ('http://open.api.ebay.com/shopping?callname=FindPopularItems'
               '&responseencoding=JSON&appid=hush&siteid=23&version=531&'
               'CategoryID=550&MaxEntries=10'
               '&trackingpartnercode=9&trackingid=campid')
        mock.assert_called_with(uri)

    def test_getsvcsapicontent_simple(self):
        response = dict(call=[{
            'searchResult': [{
                '@count': 2,
                'item': ['result1', 'result2'],
            }],
            'paginationOutput': [{
                'totalEntries': ['123'],
            }],
        }])
        results, total_count = ebay._get_svcs_api_content(response, 'call')
        self.assertEqual(results, ['result1', 'result2'])
        self.assertEqual(total_count, 123)

    def test_getsvcsapicontent_no_total(self):
        response = dict(call=[{
            'searchResult': [{
                '@count': 2,
                'item': ['result1', 'result2'],
            }],
        }])
        results, total_count = ebay._get_svcs_api_content(response, 'call')
        self.assertEqual(results, ['result1', 'result2'])
        self.assertEqual(total_count, None)


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_preview(self):
        # fake a previous result
        result = {
            'condition': u'Pre-owned',
            'currency_code': u'USD',
            'art': u'http://thumbs3.ebaystatic.com/m/' +
            'mA8ILqmlSWokT6vyEN1KFyw/140.jpg',
            'cat_id': 'cat_0',
            'dnd_uri': u'http://www.ebay.com/itm/Paris-Eiffel-' +
            'Tower-Hook-Black-Pink-EUC-/321313863006?pt=Decals' +
            '_Stickers_Vinyl_Art',
            'image_hint': u'http://i.ebayimg.com/00/s/MTAwMFg3NTA=' +
            '/z/a4cAAOxyVLNS4Ts7/$_3.JPG',
            'payment_methods': u'PayPal',
            'price': u'4.0',
            'shipping_to': u'US',
            'title': u'Paris Eiffel Tower Hook Black/Pink EUC',
            'uri': u'http://www.ebay.com/itm/Paris-Eiffel-Tower-Hook-' +
            'Black-Pink-EUC-/321313863006?pt=Decals_Stickers_Vinyl_Art',
            'item_id': 'foo',
        }
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-iteminfo.json')
            response = list(app.preview(result=result, locale='en',
                                        partner=None))

        # check the 'image'
        should = dict(widget={
            'id': 'imagegallery',
            'type': 'gallery',
            'sources': [(u'http://i.ebayimg.com/00/s/NTMxWDUzMQ==/'
                         'z/hlwAAOxydlFSuFyz/$_1.JPG?set_id=880000500F')],
        })
        self.assertEqual(response[0], should)

        # check the 'header'
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'title': u'Paris Eiffel Tower Hook Black/Pink EUC',
            'subtitle': u'Current bid: 4.0',
            'attributes': [{'value': u'Shipping: $0.99'}],
        })
        self.assertEqual(response[1], should)

        # check the 'actions'
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'action1',
                         'uri': 'http://www.ebay.com/itm/Paris-Eiffel-' +
                         'Tower-Hook-Black-Pink-EUC-/321313863006?pt=' +
                         'Decals_Stickers_Vinyl_Art',
                         'label': 'View on eBay'}],
        })
        self.assertEqual(response[2], should)

        # check the 'info'
        should = dict(widget={
            'id': 'info',
            'type': 'text',
            'title': 'Info',
            'text': ('Time left: 26d 22h\nItem location: '
                     'West Palm Beach, Florida\nCondition: Pre-owned'),
        })
        self.assertEqual(response[3], should)

        # check the 'description'
        should_desc = 'Brand new set of all 5 books'
        self.assertTrue(response[4]['widget']['text'].startswith(should_desc))

        # check the 'seller'
        should = dict(widget={
            'id': 'seller',
            'type': 'text',
            'title': '',
            'text': ('Sold by treebeardbooks (117300)\n'
                     'Feedback: 99.9% positive'),
        })
        self.assertEqual(response[5], should)

        # check the 'shipping and postage'
        should = dict(widget={
            'id': 'shipping_and_postage',
            'type': 'text',
            'title': 'Shipping and postage',
            'text': ('Postage\nItem location: West Palm Beach, Florida\n'
                     'Ships to: US, CA, GB, DK, RO, SK, BG, CZ, '
                     'FI, HU, LV, LT, MT, EE, AU, GR, PT, CY, SI, '
                     'JP, CN, SE, KR, TW, ZA, TH, BE, FR, HK, IE, NL, '
                     'PL, ES, IT, DE, AT, RU, IL, MX, NZ, PH, SG, NO, '
                     'SA, UA, AE, QA, KW, BH, HR, MY, TR\n\nPackaging\n'
                     'Dispatch time: 1 business day\n\nPayment details\n'
                     'Payment method: PayPal\n\nReturns policy\n'
                     'Policy: Returns accepted'
                     '\nRefund: Money back\nReturns within: 14 days\n\n'
                     'There is a restocking fee of $5 which covers our '
                     'cost of shipping the item to you.  The set must not '
                     'be opened and still sealed.'),
        })
        self.maxDiff = None
        self.assertEqual(response[6], should)

    def test_preview_no_shipping_cost(self):
        # fake a previous result without shipping info in details
        result = {
            'condition': u'Pre-owned',
            'currency_code': u'USD',
            'art': u'http://thumbs3.ebaystatic.com/m/' +
            'mA8ILqmlSWokT6vyEN1KFyw/140.jpg',
            'cat_id': 'cat_0',
            'dnd_uri': u'http://www.ebay.com/itm/Paris-Eiffel-' +
            'Tower-Hook-Black-Pink-EUC-/321313863006?pt=Decals' +
            '_Stickers_Vinyl_Art',
            'image_hint': u'http://i.ebayimg.com/00/s/MTAwMFg3NTA=' +
            '/z/a4cAAOxyVLNS4Ts7/$_3.JPG',
            'payment_methods': u'PayPal',
            'price': u'4.0',
            'shipping_to': u'US',
            'title': u'Paris Eiffel Tower Hook Black/Pink EUC',
            'uri': u'http://www.ebay.com/itm/Paris-Eiffel-Tower-Hook-' +
            'Black-Pink-EUC-/321313863006?pt=Decals_Stickers_Vinyl_Art',
            'item_id': 'foo',
        }
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-iteminfo_noshipping.json')
            response = list(app.preview(result=result, locale='en',
                                        partner=None))

        # check the 'header'
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'title': u'Paris Eiffel Tower Hook Black/Pink EUC',
            'subtitle': u'Current bid: 4.0',
            'attributes': [{'value': None}],
        })
        self.assertEqual(response[1], should)


class DepartmentsTestCase(TestCase):
    """Tests for the departments tree."""

    def test_parent_in_tree(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-depinfo.json')
            deps, _ = app._get_available_departments('77933', 16, '', '')

        self.assertEqual(deps['canned_query'],
                         'scope://com.canonical.scopes.ebay?dep=267')
        self.assertEqual(deps['label'], 'Livres, BD, revues')
        self.assertEqual(len(deps['subdepartments']), 1)

    def test_current_in_tree(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-depinfo.json')
            deps, _ = app._get_available_departments('77933', 16, '', '')

        self.assertEqual(deps['subdepartments'][0]['canned_query'],
                         'scope://com.canonical.scopes.ebay?dep=77933')
        self.assertEqual(deps['subdepartments'][0]['label'],
                         u'Romans, littérature')
        self.assertEqual(len(deps['subdepartments'][0]['subdepartments']), 21)

    def test_root_canned_query(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-root-depinfo.json')
            deps, _ = app._get_available_departments('', 16, '', '')

        self.assertEqual(deps['canned_query'],
                         'scope://com.canonical.scopes.ebay')

    def test_no_children(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-depinfo-nochildren.json')
            _, selected = app._get_available_departments('156595', 16, '', '')

        self.assertEqual(selected, None)


class SortingTestCase(TestCase):
    """Tests for sorting filters."""

    def test_filters_response(self):
        app = ebay.App(CONFIG)
        filter_state = {"sorting_filter": ["PricePlusShippingLowest"]}
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-results-sorting.json')
                response = list(app.search(query='', limit=1,
                                           platform='desktop',
                                           locale='en', country='us',
                                           department='267', partner=None,
                                           filters=filter_state))

        self.assertEqual(len(response), 9)
        self.assertEqual(response[2]['filter_state'], filter_state)
        sorting_options = response[1]['filters'][0]['options']
        self.assertEqual(len(sorting_options), 9)
        self.assertEqual(sorting_options[0]['id'], ebay.SORTING_FEATURED)

    def test_filters_canned_query(self):
        app = ebay.App(CONFIG)
        filter_state = {"sorting_filter": ["PricePlusShippingLowest"]}
        with patch.object(app, '_get_products') as mock:
            mock.return_value = get_fixture('ebay-root-depinfo.json')
            deps, _ = app._get_available_departments('', 16, '', '', filters=filter_state)

        expected_cq = ('scope://com.canonical.scopes.ebay?'
                       'filters=%7B%22sorting_filter'
                       '%22%3A%20%5B%22PricePlusShippingLowest%22%5D%7D')
        self.assertEqual(deps['canned_query'], expected_cq)


class TimeTestCase(TestCase):
    """Tests for time parsing."""

    def test_time_left(self):
        # Invalid time
        time = 'foo'
        t_should = None
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # d, h
        time = 'P2DT23H32M51S'
        t_should = '2d 23h'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # h, m
        time = 'PT23H32M51S'
        t_should = '23h 32m'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # m
        time = 'P0DT0H32M'
        t_should = '32m'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # d
        time = 'P1DT0H32M'
        t_should = '1d'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)

        # d, h
        time = 'P5DT5H0M'
        t_should = '5d 5h'
        self.assertEqual(ebay._parse_time_left(time, None), t_should)


class ShippingPostagePreviewTestCase(TestCase):
    """Tests for Postage and shipping preview text building."""

    def test_postage_only(self):
        details = {'location': 'A',
                   'ships_to': 'B',
                   'handling_time': None,
                   'payment_methods': None,
                   'return_policy': None}
        text = ebay._build_shipping_and_postage_text(details, '')
        should_text = 'Postage\nItem location: A\nShips to: B'
        self.assertEqual(text, should_text)

    def test_packaging_only(self):
        details = {'location': None,
                   'ships_to': None,
                   'handling_time': 1,
                   'payment_methods': None,
                   'return_policy': None}
        text = ebay._build_shipping_and_postage_text(details, '')
        should_text = 'Packaging\nDispatch time: 1 business day'
        self.assertEqual(text, should_text)

    def test_payment_only(self):
        details = {'location': None,
                   'ships_to': None,
                   'handling_time': None,
                   'payment_methods': 'Foo, Bar',
                   'return_policy': None}
        text = ebay._build_shipping_and_postage_text(details, '')

        should_text = 'Payment details\nPayment method: Foo, Bar'
        self.assertEqual(text, should_text)

    def test_returns_only(self):
        details = {'location': None,
                   'ships_to': None,
                   'handling_time': None,
                   'payment_methods': None,
                   'return_policy': {'ReturnsAccepted': 'Maybe',
                                     'Refund': 'Certainly Not'}}
        text = ebay._build_shipping_and_postage_text(details, '')
        should_text = 'Returns policy\nPolicy: Maybe\nRefund: Certainly not'
        self.assertEqual(text, should_text)

    def test_postage_n_returns_only(self):
        details = {'location': 'A',
                   'ships_to': None,
                   'handling_time': None,
                   'payment_methods': None,
                   'return_policy': {'ReturnsAccepted': 'Returns accepted',
                                     'Description': None}}
        text = ebay._build_shipping_and_postage_text(details, '')
        should_text = ('Postage\nItem location: A\n\nReturns policy\n'
                       'Policy: Returns accepted')
        self.assertEqual(text, should_text)


class CategoriesTestCase(TestCase):
    """Tests for building categories."""

    def test_counter_in_title(self):
        category = ebay._build_counted_cat(ebay.PRODUCTS_CATEGORY, '', None)
        self.assertEqual(category['title'], 'Results')

        category = ebay._build_counted_cat(ebay.PRODUCTS_CATEGORY, '', 1)
        self.assertEqual(category['title'], '1 result')

        category = ebay._build_counted_cat(ebay.PRODUCTS_CATEGORY, '', 2)
        self.assertEqual(category['title'], '2 results')


class AttributesTestCase(TestCase):
    """Tests for attributes"""

    def test_attributes_grid(self):
        app = ebay.App(CONFIG)
        with nested(
                patch.object(app, '_get_products'),
                patch.object(app, '_get_available_departments'),
                ) as (mock, mdep):
            mdep.return_value = {}, ('550', 'Foo')
            mock.return_value = get_fixture('ebay-surfacing.json')
            kwargs = STD_KWARGS.copy()
            kwargs.update(query='')
            response = list(app.search(**kwargs))

        should = [{'value': '4d 2h'}]
        self.assertEqual(response[2]['result']['s_attributes'], should)
        self.assertEqual(response[2]['result']['price'], u'$36.92')

    def test_attributes_list(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-results.json')
                response = list(app.search(**STD_KWARGS))

        should = [{'value': u'$4.00'},
                  {'value': u'1 bids'},
                  {'value': u'$5.00'},
                  {'value': '1h 30m'}]
        self.assertEqual(response[1]['result']['attributes'], should)


class PriceTestCase(TestCase):
    """Tests for price formatting"""

    def test_price_symbol_locale_fr(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-results.json')
                kwargs = STD_KWARGS.copy()
                kwargs.update(locale='fr')
                response = list(app.search(**kwargs))

        should = {'value': u'4,00\xa0$US'}
        self.assertEqual(response[1]['result']['attributes'][0], should)

    def test_price_symbol_locale_en(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-results.json')
                response = list(app.search(**STD_KWARGS))

        should = [{'value': u'$4.00'},
                  {'value': u'1 bids'},
                  {'value': u'$5.00'},
                  {'value': '1h 30m'}]
        self.assertEqual(response[1]['result']['attributes'], should)

    def test_price_symbol_locale_en_au(self):
        app = ebay.App(CONFIG)
        with patch.object(app, '_get_products') as mock:
            with patch.object(app, '_get_available_departments') as mdep:
                mdep.return_value = {}, ()
                mock.return_value = get_fixture('ebay-results.json')
                kwargs = STD_KWARGS.copy()
                kwargs.update(locale='en_au')
                response = list(app.search(**kwargs))

        should = [{'value': u'US$4.00'},
                  {'value': u'1 bids'},
                  {'value': u'US$5.00'},
                  {'value': '1h 30m'}]
        self.assertEqual(response[1]['result']['attributes'], should)


class ParsingTestCase(TestCase):
    """Tests for different parsing cases."""
    maxDiff = None

    def test_searched_normal(self):
        item = json.loads(get_fixture('ebay-searcheditem-ok.json'))
        resp = ebay._parse_searched_item(item, 'en')
        self.assertDictEqual(resp, {
            'title': u'Conan the Swordsman (L Sprague de Camp, L Carter, '
                     u'B Nyberg, Paperback)',
            'payment_method': u'PayPal',
            'price': u'\xa30.99',
            'time_left': u'P2DT22H0M16S',
            'uri': u'http://www.ebay.co.uk/itm/Conan-Swordsman-L-Sprague-Camp'
                   u'-L-Carter-B-Nyberg-Paperback-/151395385200?pt=Fiction',
            'shipping': u'\xa31.30',
            'bid_count': u'0 bids',
            'icon_hint': u'http://thumbs1.ebaystatic.com/m/'
                         u'mWruVLT7AcyBb5Rmei3myZg/140.jpg',
            'currency_code': u'GBP',
            'item_id': u'151395385200',
            'image_hint': u'http://i.ebayimg.com/00/s/MTIwMFgxNjAw/z/'
                          u'hKoAAOSwd4tT1oHb/$_3.JPG',
            'shipping_to': u'GB',
            'condition': u'Very Good',
        })

    def test_searched_no_gallery(self):
        item = json.loads(get_fixture('ebay-searcheditem-nogallery.json'))
        resp = ebay._parse_searched_item(item, 'en')
        self.assertDictEqual(resp, {
            'title': u'Intrepid Enchanter by L. Sprague De Camp, L.Sprague De '
                     u'Camp, Fletcher Pratt...',
            'payment_method': u'PayPal, PersonalCheck',
            'price': u'\xa32.59',
            'time_left': u'P23DT9H2M52S',
            'uri': 'http://www.ebay.co.uk/itm/Intrepid-Enchanter-L-Sprague-Ca'
                   'mp-L-Sprague-Camp-Fletcher-Pratt-/181249954348?pt=Fiction',
            'shipping': u'Free',
            'bid_count': None,
            'icon_hint': None,
            'currency_code': u'GBP',
            'item_id': u'181249954348',
            'image_hint': None,
            'shipping_to': u'Worldwide',
            'condition': u'Good',
        })

    def test_searched_no_payment(self):
        item = json.loads(get_fixture('ebay-searcheditem-nopayment.json'))
        resp = ebay._parse_searched_item(item, 'en')
        self.assertEqual(resp['payment_method'], None)
