# -*- coding: utf8 -*-

# Copyright 2014-2015 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import json

from unittest import TestCase
from mock import patch

from src.scopes import reddit
from src.scopes.tests import get_fixture

from versioninfo import version_info


CONFIG = {'secrets': {}}


class SearchTestCase(TestCase):
    """Tests for the search function."""

    def test_result_retrieval(self):
        app = reddit.App(CONFIG)
        fake_data = {'test': 123}
        with patch.object(app, 'urlread') as mock_urlread:
            mock_urlread.return_value = json.dumps(fake_data)
            resp = app._get_results('test url')
        self.assertEqual(resp, fake_data)
        ua = "Reddit Ubuntu Rest Scope " + str(version_info['revno'])
        mock_urlread.assert_called_with('test url', headers={'User-Agent': ua})

    def test_surfacing(self):
        app = reddit.App(CONFIG)
        with patch.object(app, '_get_available_departments') as mockdep:
            with patch.object(app, '_get_results') as mock:
                mock.return_value = json.loads(
                    get_fixture('reddit-front.json'))
                mockdep.return_value = {}
                response = list(app.search(query='',
                                           platform='desktop',
                                           limit=app.default_limit,
                                           locale='C'))

        # got the info as it should
        mock.assert_called_with('http://www.reddit.com/.json?limit=25')

        # category
        should_cat = {'category': reddit.THREAD_CATEGORY}
        self.assertEqual(response[1], should_cat)

        # the result
        should_res = {'result': {
            'cat_id': 'posts',
            'dnd_uri': u'http://reddit.com/r/funny/comments/1z2w0l/' +
            'woman_in_scotland_fills_out_the_wrong_form_on/',
            'uri': u'http://reddit.com/r/funny/comments/1z2w0l/' +
            'woman_in_scotland_fills_out_the_wrong_form_on/',
            'title': u'Woman in Scotland fills out the wrong form on ' +
            'TripAdvisor. Is now ranked 87 out of 168 attraction in Glasgow.',
            'author': u'chrisTMMurphy',
            'comments': '358 comments',
            'mascot': u'http://c.thumbs.redditmedia.com/U-52P34Hz9WDtid2.jpg',
            'media_type': None,
            'media_preview': None,
            'target': u'http://imgur.com/RtCUqDy',
            'summary': u'',
            'subreddit': u'funny',
            'domain': u'imgur.com',
            'attributes': [{'style': 'highlighted', 'value': '3609'},
                           {'style': 'default', 'value': '358 comments'}]
        }}
        self.maxDiff = None
        self.assertEqual(response[2], should_res)

    def test_searched_subreddit(self):
        subreddit_data = json.loads(get_fixture('reddit-subreddit-info.json'))
        response = reddit._parse_subreddit(subreddit_data, '', 'EN')

        should = {'art': (u'http://thumbs.reddit.com/t5_2qh62.png?'
                          'v=2ded9b598cbb88694ced09618592d207'),
                  'cat_id': 'subreddits',
                  'dnd_uri': u'/r/Ubuntu/',
                  'header_title': u'Linux for Human Beings',
                  'subreddit': u'Ubuntu',
                  'subscribers': '47377 subscribers',
                  'summary': u'The Ubuntu community on Reddit',
                  'title': u'Ubuntu',
                  'uri': u'scope://com.canonical.scopes.reddit?dep=Ubuntu'}
        self.assertEqual(response, should)

    def test_searched_subreddit_nosubscribres(self):
        subreddit_data = json.loads(get_fixture(
            'reddit-subreddit-nosuscribers.json'))
        response = reddit._parse_subreddit(subreddit_data, '', 'EN')

        self.assertEqual(response, {
            'art': None,
            'cat_id': 'subreddits',
            'dnd_uri': u'/r/mildlydeece/',
            'header_title': None,
            'subreddit': u'mildlydeece',
            'subscribers': 'no subscribers',
            'summary': (
                u'A place for those things which we find somewhat fascinating,'
                u' slightly interesting, and always mildly deece.'),
            'title': u'mildlydeece',
            'uri': u'scope://com.canonical.scopes.reddit?dep=mildlydeece',
        })

    def test_searched_post(self):
        post_data = json.loads(get_fixture('reddit-post-info.json'))
        response = reddit._parse_post(post_data, '', 'EN', None)

        should = {
            'cat_id': 'posts',
            'dnd_uri': (u'http://reddit.com/r/Ubuntu/comments/l9jpu/'
                        'why_does_everyone_in_the_ubuntu_subreddit_seem_to/'),
            'uri': (u'http://reddit.com/r/Ubuntu/comments/l9jpu/'
                    'why_does_everyone_in_the_ubuntu_subreddit_seem_to/'),
            'target': (u'http://www.reddit.com/r/Ubuntu/comments/l9jpu/'
                       'why_does_everyone_in_the_ubuntu_subreddit_seem_to/'),
            'subreddit': u'Ubuntu',
            'domain': u'self.Ubuntu',
            'mascot': None,
            'media_preview': None,
            'media_type': None,
            'comments': '242 comments',
            'author': u's0m3f00l',
            'summary': (u" It seems as if this is more of the 10.10 UBUNTU"
                        " only subreddit, and anything that doesn't fall in"
                        " line with that is either downvoted or dismissed"
                        " in comments. Is there a Sub-Reddit for UBUNTU"
                        " that is more open to all UBUNTU folks?"),
            'title': (u"Why does everyone in the UBUNTU sub-reddit"
                      " seem to dislike UBUNTU?"),
            'attributes': [
                {'style': 'highlighted', 'value': '171'},
                {'style': 'default', 'value': '242 comments'}]
            }
        self.assertEqual(response, should)

    def test_available_departments(self):
        app = reddit.App(CONFIG)
        with patch.object(app, '_get_results') as mock:
            mock.return_value = json.loads(
                get_fixture('reddit-departments.json'))
            response = app._get_available_departments(None, 'EN')

        should = {'canned_query': 'scope://com.canonical.scopes.reddit',
                  'has_subdepartments': True,
                  'label': 'Front page',
                  'subdepartments': [
                      {'canned_query': (u'scope://com.canonical.scopes.reddit'
                                        '?dep=funny'),
                       'has_subdepartments': False,
                       'label': u'funny'},
                      {'canned_query': (u'scope://com.canonical.scopes.reddit'
                                        '?dep=AdviceAnimals'),
                       'has_subdepartments': False,
                       'label': u'AdviceAnimals'},
                      {'canned_query': (u'scope://com.canonical.scopes.reddit'
                                        '?dep=pics'),
                       'has_subdepartments': False,
                       'label': u'pics'}]}
        self.assertEqual(response, should)

    def test_available_departments_with_new_dep(self):
        app = reddit.App(CONFIG)
        with patch.object(app, '_get_results') as mock:
            mock.return_value = json.loads(
                get_fixture('reddit-departments.json'))
            response = app._get_available_departments('halflife3', 'EN')

        should = {'canned_query': 'scope://com.canonical.scopes.reddit',
                  'has_subdepartments': True,
                  'label': 'Front page',
                  'subdepartments': [
                      {'canned_query': (u'scope://com.canonical.scopes.reddit'
                                        '?dep=halflife3'),
                       'has_subdepartments': False,
                       'label': u'halflife3'},
                      {'canned_query': (u'scope://com.canonical.scopes.reddit'
                                        '?dep=funny'),
                       'has_subdepartments': False,
                       'label': u'funny'},
                      {'canned_query': (u'scope://com.canonical.scopes.reddit'
                                        '?dep=AdviceAnimals'),
                       'has_subdepartments': False,
                       'label': u'AdviceAnimals'},
                      {'canned_query': (u'scope://com.canonical.scopes.reddit'
                                        '?dep=pics'),
                       'has_subdepartments': False,
                       'label': u'pics'}]}
        self.assertEqual(response, should)

    def test_is_mobile(self):
        platforms = [{'platform': None, 'should': 'url'},
                     {'platform': 'desktop', 'should': 'url'},
                     {'platform': 'phone', 'should': 'url.compact'},
                     {'platform': 'tablet', 'should': 'url.compact'},
                     {'platform': 'foo', 'should': 'url.compact'}]
        for p in platforms:
            response = reddit._build_uri('url', p['platform'])
            self.assertEqual(response, 'http://reddit.com%s' % (p['should'],))


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_video_preview(self):
        # fake a previous video result
        result = {
            'cat_id': 'posts',
            'dnd_uri': u'The uri',
            'uri': u'The uri',
            'title': u'The title',
            'author': u'The author',
            'comments': '2 comments',
            'mascot': u'The mascot',
            'media_type': u'video',
            'media_preview': u'some image',
            'target': u'The target',
            'summary': u'The summary',
            'subreddit': u'The subreddit',
            'domain': u'The domain',
        }
        app = reddit.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the 'video'
        should = dict(widget={
            'id': 'video',
            'type': 'video',
            'source': 'The target',
            'screenshot': 'some image',
        })
        self.assertEqual(response[0], should)

    def test_image_preview(self):
        # fake a previous image result
        result = {
            'cat_id': 'posts',
            'dnd_uri': u'The uri',
            'uri': u'The uri',
            'title': u'The title',
            'author': u'The author',
            'comments': '2 comments',
            'mascot': u'The mascot',
            'media_type': u'rich',
            'media_preview': u'some image',
            'target': u'The target',
            'summary': u'The summary',
            'subreddit': u'The subreddit',
            'domain': u'The domain',
        }
        app = reddit.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the 'image'
        should = dict(widget={
            'id': 'image',
            'type': 'image',
            'source': 'some image',
        })
        self.assertEqual(response[0], should)

    def test_generic_preview(self):
        # fake a previous generic result
        result = {
            'cat_id': 'posts',
            'dnd_uri': u'The uri',
            'uri': u'The uri',
            'title': u'The title',
            'author': u'The author',
            'comments': '2 comments',
            'mascot': u'The mascot',
            'media_type': None,
            'media_preview': None,
            'target': u'The target',
            'summary': u'The summary',
            'subreddit': u'The subreddit',
            'domain': u'The domain',
        }
        app = reddit.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the 'header'
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'mascot': 'The mascot',
            'subtitle': 'The domain',
            'title': 'The title',
        })
        self.assertEqual(response[0], should)

        # check the 'actions'
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'action1',
                         'label': '2 comments',
                         'uri': u'The uri'},
                        {'id': 'action2',
                         'label': 'View',
                         'uri': u'The target'}],
        })
        self.assertEqual(response[1], should)

        # check the 'text'
        should = dict(widget={
            'id': 'text',
            'type': 'text',
            'text': 'The summary',
        })
        self.assertEqual(response[2], should)

    def test_subreddit_preview(self):
        # fake a previous subreddit result
        result = {
            'cat_id': 'subreddits',
            'dnd_uri': u'The uri',
            'uri': u'The uri',
            'title': u'The subreddit name',
            'art': u'The art',
            'target': u'The target',
            'summary': u'The summary',
            'subreddit': u'The subreddit',
            'subscribers': u'2 subscribers',
            'header_title': u'The tagline',
        }
        app = reddit.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the 'image'
        should = dict(widget={
            'id': 'image',
            'type': 'image',
            'source': 'The art',
        })
        self.assertEqual(response[0], should)

        # check the 'header'
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'title': 'The subreddit name',
            'subtitle': 'The tagline',
        })
        self.assertEqual(response[1], should)

        # check the 'actions'
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'action1',
                         'label': '/r/The subreddit',
                         'uri': u'The uri'}],
        })
        self.assertEqual(response[2], should)

        # check the 'text'
        should = dict(widget={
            'id': 'text',
            'type': 'text',
            'text': 'The summary',
        })
        self.assertEqual(response[3], should)
