# -*- coding: utf-8 -*-

# Copyright 2014-2015 Canonical
# All Rigths Reserved

"""Tests for the Simple App base stuff."""

import datetime
import urlparse

from contextlib import nested
from unittest import TestCase

from oauth import oauth
from lxml import etree
from mock import patch, Mock

from src.scopes import sevendigital, BackendDataError
from src.scopes.tests import get_fixture

FAKE_CONFIG = {'secrets': {None: {'oauth_consumer_key': 'foo',
                                  'oauth_consumer_secret': 'bar'}}}

EMPTY_XML = etree.XML("""<response>
    <searchResults><totalItems>0</totalItems></searchResults>
</response>
""")


class SevenDigitalInterfaceTestCase(TestCase):
    """Test the interface to 7digital backend."""

    def test_build_signed_url_simple(self):
        sdi = sevendigital.SevenDigitalInterface('test_oauth_key', 'test_oauth_secret', None, None)
        with nested(
                patch.object(oauth, 'OAuthConsumer'),
                patch.object(oauth.OAuthRequest, 'from_consumer_and_token'),
                patch.object(oauth, 'OAuthSignatureMethod_HMAC_SHA1'),
                ) as (mock_consumer, mock_req_factory, mock_sign_meth):
            mock_consumer.return_value = 'consumer'
            mock_req_factory.return_value = mock_request = Mock()
            mock_request.to_url.return_value = 'signed url'
            mock_sign_meth.return_value = 'meth'
            resp = sdi.build_preview_track_source('test_track_id')

        # check the response
        self.assertEqual(resp, 'signed url')
        mock_consumer.assert_called_with('test_oauth_key', 'test_oauth_secret')
        url = 'http://previews.7digital.com/clip/test_track_id?country=GB'
        mock_req_factory.assert_called_with('consumer', http_url=url, parameters={'country': 'GB'})
        mock_request.sign_request.assert_called_with('meth', 'consumer', token=None)

    def test_build_signed_url_with_country(self):
        sdi = sevendigital.SevenDigitalInterface('test_oauth_key', 'test_oauth_secret', 'ET', None)
        with nested(
                patch.object(oauth, 'OAuthConsumer'),
                patch.object(oauth.OAuthRequest, 'from_consumer_and_token'),
                patch.object(oauth, 'OAuthSignatureMethod_HMAC_SHA1'),
                ) as (mock_consumer, mock_req_factory, mock_sign_meth):
            mock_consumer.return_value = 'consumer'
            mock_req_factory.return_value = mock_request = Mock()
            mock_request.to_url.return_value = 'signed url'
            mock_sign_meth.return_value = 'meth'
            resp = sdi.build_preview_track_source('test_track_id')

        # check the response
        self.assertEqual(resp, 'signed url')
        mock_consumer.assert_called_with('test_oauth_key', 'test_oauth_secret')
        url = 'http://previews.7digital.com/clip/test_track_id?country=ET'
        mock_req_factory.assert_called_with('consumer', http_url=url,
                                            parameters=dict(country='ET'))
        mock_request.sign_request.assert_called_with('meth', 'consumer',
                                                     token=None)

    def test_hit_backend_simple(self):
        mock_urlread = Mock()
        sdi = sevendigital.SevenDigitalInterface('test_key', 't_scret', None, mock_urlread)
        mock_urlread.return_value = '<?xml version="1.0"?><foo></foo>'
        resp = sdi._hit_backend('testpath', param1='tp1', param2='tp2')

        # check URL sent
        url = mock_urlread.call_args[0][0]
        base, params = url.split("?")
        self.assertEqual(base, "http://api.7digital.com/1.2/testpath")
        self.assertEqual(sorted(urlparse.parse_qsl(params)), [
            ('country', 'GB'),
            ('imageSize', '500'),
            ('oauth_consumer_key', 'test_key'),
            ('pageSize', '20'),
            ('param1', 'tp1'),
            ('param2', 'tp2'),
        ])

        # check result is already converted to xml
        self.assertEqual(resp.tag, 'foo')

    def test_hit_backend_broken(self):
        mock_urlread = Mock()
        sdi = sevendigital.SevenDigitalInterface('test_key', 't_scret', None, mock_urlread)
        mock_urlread.return_value = 'broken xml'
        self.assertRaises(BackendDataError, sdi._hit_backend,
                          'testpath', param1='tp1', param2='tp2')

    def test_hit_backend_country_ok(self):
        mock_urlread = Mock()
        sdi = sevendigital.SevenDigitalInterface('test_key', 't_scret', 'US', mock_urlread)
        mock_urlread.return_value = '<?xml version="1.0"?><foo></foo>'
        resp = sdi._hit_backend('testpath')

        # check URL sent
        url = mock_urlread.call_args[0][0]
        base, params = url.split("?")
        self.assertEqual(base, "http://api.7digital.com/1.2/testpath")
        self.assertEqual(sorted(urlparse.parse_qsl(params)), [
            ('country', 'US'),
            ('imageSize', '500'),
            ('oauth_consumer_key', 'test_key'),
            ('pageSize', '20'),
        ])

        # check result is already converted to xml
        self.assertEqual(resp.tag, 'foo')

    def test_hit_backend_country_unauthorized(self):
        assert 'AR' in sevendigital.UNAUTHORIZED_COUNTRIES
        mock_urlread = Mock()
        sdi = sevendigital.SevenDigitalInterface('test_key', 't_scret', 'AR', mock_urlread)
        mock_urlread.return_value = '<?xml version="1.0"?><foo></foo>'
        resp = sdi._hit_backend('testpath')

        # check URL sent
        url = mock_urlread.call_args[0][0]
        base, params = url.split("?")
        self.assertEqual(base, "http://api.7digital.com/1.2/testpath")
        self.assertEqual(sorted(urlparse.parse_qsl(params)), [
            ('country', 'GB'),
            ('imageSize', '500'),
            ('oauth_consumer_key', 'test_key'),
            ('pageSize', '20'),
        ])

        # check result is already converted to xml
        self.assertEqual(resp.tag, 'foo')

    def test_get_album_latest(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-latest.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            resp = sdi.get_album_latest()
        oneweekago = datetime.date.today() - datetime.timedelta(days=7)
        mock.assert_called_with('release/bydate',
                                fromDate=oneweekago.strftime("%Y%m%d"))
        self.assertEqual(len(resp), 3)
        obj = resp[0]  # a sample
        self.assertEqual(obj.id, u'3487738')
        self.assertEqual(obj.title, u'Turn Blue')
        self.assertEqual(obj.price, u'£5.99')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/the-black-keys/release/'
            u'turn-blue-1/?partner=7999')
        self.assertEqual(
            obj.image,
            u'http://cdn.7static.com/static/img/sleeveart/00/034/877/'
            u'0003487738_50.jpg')
        self.assertEqual(obj.release_date, u'2014-06-27')
        self.assertEqual(obj.artist.name, u'The Black Keys')
        self.assertEqual(obj.artist.id, u'30198')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/the-black-keys/?partner=7999')

        # assure order is certain
        prv = resp[0]
        for item in resp[1:]:
            self.assertGreaterEqual(prv.release_date, item.release_date)
            prv = item

    def test_get_album_topseller(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-topseller.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            resp = sdi.get_album_topseller()
        mock.assert_called_with('release/chart')
        self.assertEqual(len(resp), 3)
        obj = resp[0]  # a sample
        self.assertEqual(obj.id, u'3585165')
        self.assertEqual(obj.title, u'x')
        self.assertEqual(obj.price, u'£8.99')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/ed-sheeran/release/'
            u'x-1/?partner=7999')
        self.assertEqual(
            obj.image,
            u'http://cdn.7static.com/static/img/sleeveart/00/035/851/'
            u'0003585165_50.jpg')
        self.assertEqual(obj.release_date, u'2014-06-23')
        self.assertEqual(obj.artist.name, u'Ed Sheeran')
        self.assertEqual(obj.artist.id, u'603309')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/ed-sheeran/?partner=7999')

    def test_get_album_topseller_pagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-topseller.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.get_album_topseller(page_size=25)
        mock.assert_called_with('release/chart', pageSize=25)

    def test_get_album_topseller_imagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-topseller.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.get_album_topseller(1000)
        mock.assert_called_with('release/chart', imageSize=1000)

    def test_get_track_topseller(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-track-topseller.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            resp = sdi.get_track_topseller()
        mock.assert_called_with('track/chart')
        self.assertEqual(len(resp), 3)
        obj = resp[0]  # a sample
        self.assertEqual(obj.id, u'36817572')
        self.assertEqual(obj.title, u'Ghost')
        self.assertEqual(obj.price, u'£0.99')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/ella-henderson/release/'
            u'ghost/?partner=7999&h=01')
        self.assertEqual(
            obj.image,
            u'http://cdn.7static.com/static/img/sleeveart/00/034/974/'
            u'0003497457_50.jpg')
        self.assertEqual(obj.release_date, u'2014-06-08')
        self.assertEqual(obj.length, 215)
        self.assertEqual(obj.artist.name, u'Ella Henderson')
        self.assertEqual(obj.artist.id, u'2267033')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/ella-henderson/?partner=7999')

    def test_get_track_topseller_pagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-track-topseller.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.get_track_topseller(page_size=25)
        mock.assert_called_with('track/chart', pageSize=25)

    def test_get_track_topseller_imagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-track-topseller.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.get_track_topseller(1000)
        mock.assert_called_with('track/chart', imageSize=1000)

    def test_get_album_tagged(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-tagged.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            resp = sdi.get_album_tagged('test_tag')
        mock.assert_called_with('release/bytag/new', tags='test_tag')
        self.assertEqual(len(resp), 2)
        obj = resp[0]  # a sample
        self.assertEqual(obj.id, u'3597008')
        self.assertEqual(obj.title, u'Mega Pop Hits, Vol. 1')
        self.assertEqual(obj.price, u'£8.49')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/billy-williams/release/'
            u'mega-pop-hits-vol-1/?partner=7999')
        self.assertEqual(
            obj.image,
            u'http://cdn.7static.com/static/img/sleeveart/00/035/970/'
            u'0003597008_50.jpg')
        self.assertEqual(obj.release_date, u'2014-07-01')
        self.assertEqual(obj.artist.name, u'Billy Williams')
        self.assertEqual(obj.artist.id, u'2521')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/billy-williams/?partner=7999')

    def test_search_artists(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-artist-search.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            count, items = sdi.search_artists('test_query')
        mock.assert_called_with('artist/search', q='test_query')
        self.assertEqual(count, 72)
        self.assertEqual(len(items), 4)
        obj = items[0]  # a sample
        self.assertEqual(obj.id, u'360409')
        self.assertEqual(obj.name, u'Blonde')
        self.assertEqual(
            obj.url, u'http://www.7digital.com/artist/blonde/?partner=7999')
        self.assertEqual(
            obj.image,
            u'http://cdn.7static.com/static/img/artistimages/00/003/604/'
            u'0000360409_150.jpg')

    def test_search_artists_unicode(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = EMPTY_XML
            sdi.search_artists(u'moño')
        mock.assert_called_with('artist/search', q='mo\xc3\xb1o')

    def test_search_artists_imagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-artist-search.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.search_artists('test_query', 1000)
        mock.assert_called_with('artist/search',
                                q='test_query', imageSize=1000)

    def test_search_artists_unicode_url(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-unicode-2.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            _, artists = sdi.search_artists('test_query', 1000)
        mock.assert_called_with('artist/search',
                                q='test_query', imageSize=1000)
        self.assertEqual(
            artists[0].url,
            'http://www.7digital.com/artist/bl%C3%B3nde/?partner=7999')

    def test_search_albums(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-search.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            count, items = sdi.search_albums('test_query')
        mock.assert_called_with('release/search', q='test_query', type='album')
        self.assertEqual(count, 145)
        self.assertEqual(len(items), 3)
        obj = items[0]  # a sample
        self.assertEqual(obj.id, u'370676')
        self.assertEqual(obj.title, u'Concrete Blonde')
        self.assertEqual(obj.price, u'£7.99')
        self.assertEqual(obj.price_amount, u'7.99')
        self.assertEqual(obj.price_currency, u'£')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/concrete-blonde/release/'
            u'concrete-blonde/?partner=7999')
        self.assertEqual(
            obj.image,
            u'http://cdn.7static.com/static/img/sleeveart/00/003/706/'
            u'0000370676_50.jpg')
        self.assertEqual(obj.release_date, u'2009-02-17')
        self.assertEqual(obj.artist.name, u'Concrete Blonde')
        self.assertEqual(obj.artist.id, u'15862')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/concrete-blonde/?partner=7999')

    def test_search_albums_imagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-search.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.search_albums('test_query', 1000)
        mock.assert_called_with('release/search', q='test_query',
                                type='album', imageSize=1000)

    def test_search_albums_pagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-search.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.search_albums('test_query', page_size=5)
        mock.assert_called_with('release/search', q='test_query',
                                type='album', pageSize=5)

    def test_search_albums_unicode(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = EMPTY_XML
            sdi.search_albums(u'moño')
        mock.assert_called_with('release/search',
                                q='mo\xc3\xb1o', type='album')

    def test_get_album_details(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-details.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            obj = sdi.get_album_details(370676)
        mock.assert_called_with('release/details', releaseId=370676)
        self.assertEqual(obj.id, u'370676')
        self.assertEqual(obj.title, u'Concrete Blonde')
        self.assertEqual(obj.price, u'£7.99')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/concrete-blonde/release/'
            u'concrete-blonde/?partner=7999')
        self.assertEqual(
            obj.image,
            u'http://artwork-cdn.7static.com/static/img/sleeveart/00/003/706/'
            u'0000370676_500.jpg')
        self.assertEqual(obj.release_date, u'2009-02-17')
        self.assertEqual(obj.artist.name, u'Concrete Blonde')
        self.assertEqual(obj.artist.id, u'15862')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/concrete-blonde/?partner=7999')

    def test_get_album_details_none(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = EMPTY_XML
            obj = sdi.get_album_details(370676)
        mock.assert_called_with('release/details', releaseId=370676)
        self.assertIsNone(obj)

    def test_search_tracks(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-track-search.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            count, items = sdi.search_tracks('test_query')
        mock.assert_called_with('track/search', q='test_query')
        self.assertEqual(count, 200)
        self.assertEqual(len(items), 3)
        obj = items[0]  # a sample
        self.assertEqual(obj.id, u'10608509')
        self.assertEqual(obj.title, u'Here Sometimes')
        self.assertEqual(obj.price, u'£0.99')
        self.assertEqual(obj.price_amount, u'0.99')
        self.assertEqual(obj.price_currency, u'£')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/blonde-redhead/release/'
            u'penny-sparkle/?partner=7999&h=01')
        self.assertEqual(
            obj.image,
            u'http://cdn.7static.com/static/img/sleeveart/00/009/655/'
            u'0000965521_50.jpg')
        self.assertEqual(obj.release_date, u'2010-09-13')
        self.assertEqual(obj.length, 283)
        self.assertEqual(obj.artist.name, u'Blonde Redhead')
        self.assertEqual(obj.artist.id, u'14653')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/blonde-redhead/?partner=7999')

    def test_search_tracks_unicode(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = EMPTY_XML
            sdi.search_tracks(u'moño')
        mock.assert_called_with('track/search', q='mo\xc3\xb1o')

    def test_search_tracks_imagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-track-search.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.search_tracks('test_query', 1000)
        mock.assert_called_with('track/search', q='test_query', imageSize=1000)

    def test_search_tracks_pagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-track-search.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.search_tracks('test_query', page_size=5)
        mock.assert_called_with('track/search', q='test_query', pageSize=5)

    def test_get_track_details(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-track-details.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            obj = sdi.get_track_details(10608509)
        mock.assert_called_with('track/details', trackId=10608509)
        self.assertEqual(obj.id, u'10608509')
        self.assertEqual(obj.title, u'Here Sometimes')
        self.assertEqual(obj.price, u'£0.99')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/blonde-redhead/release/'
            u'penny-sparkle/?partner=7999&h=01')
        self.assertEqual(
            obj.image,
            u'http://artwork-cdn.7static.com/static/img/sleeveart/00/009/655/'
            u'0000965521_500.jpg')
        self.assertEqual(obj.release_date, u'2010-09-13')
        self.assertEqual(obj.artist.name, u'Blonde Redhead')
        self.assertEqual(obj.artist.id, u'14653')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/blonde-redhead/?partner=7999')

    def test_get_track_details_none(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = EMPTY_XML
            obj = sdi.get_track_details(10608509)
        mock.assert_called_with('track/details', trackId=10608509)
        self.assertIsNone(obj)

    def test_get_artist_albums(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-artist-albums.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            resp = sdi.get_artist_albums('test_artist_id')
        mock.assert_called_with('artist/releases',
                                artistId='test_artist_id', type='album')
        self.assertEqual(len(resp), 1)
        obj = resp[0]  # a sample
        self.assertEqual(obj.id, u'2178610')
        self.assertEqual(obj.title, u'Leave a Light On!! (Odille Lima)')
        self.assertEqual(obj.price, u'£1.99')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/miguel-blonde/release/'
            u'leave-a-light-on-odille-lima/?partner=7999')
        self.assertEqual(
            obj.image,
            u'http://cdn.7static.com/static/img/sleeveart/00/021/786/'
            u'0002178610_50.jpg')
        self.assertEqual(obj.release_date, u'2013-02-01')
        self.assertEqual(obj.artist.name, u'Miguel Blonde')
        self.assertEqual(obj.artist.id, u'262765')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/miguel-blonde/?partner=7999')

    def test_get_artist_tracks(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-artist-tracks.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            resp = sdi.get_artist_tracks('test_artist_id')
        mock.assert_called_with('artist/toptracks', artistId='test_artist_id')
        self.assertEqual(len(resp), 3)
        obj = resp[0]  # a sample
        self.assertEqual(obj.id, u'36139377')
        self.assertEqual(obj.title, u'Foolish (feat. Ryan Ashley)')
        self.assertEqual(obj.price, u'£0.99')
        self.assertEqual(
            obj.url,
            u'http://www.7digital.com/artist/blonde/release/'
            u'foolish/?partner=7999&h=01')
        self.assertEqual(
            obj.image,
            u'http://cdn.7static.com/static/img/sleeveart/00/034/303/'
            u'0003430387_50.jpg')
        self.assertEqual(obj.release_date, u'2014-06-01')
        self.assertEqual(obj.length, 198)
        self.assertEqual(obj.artist.name, u'Blonde')
        self.assertEqual(obj.artist.id, u'360409')
        self.assertEqual(
            obj.artist.url,
            u'http://www.7digital.com/artist/blonde/?partner=7999')

    def test_get_artist_details(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-artist-details.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            artist = sdi.get_artist_details('test_artist_id')
        mock.assert_called_with('artist/details', artistId='test_artist_id')
        self.assertEqual(artist.id, u'360409')
        self.assertEqual(artist.name, u'Blonde')
        self.assertEqual(
            artist.url, u'http://www.7digital.com/artist/blonde/?partner=7999')
        self.assertEqual(
            artist.image,
            u'http://cdn.7static.com/static/img/artistimages/00360409_150.jpg')
        self.assertEqual(artist.bio, u'Visited España')

    def test_get_artist_details_unicode(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-unicode-3.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            artist = sdi.get_artist_details('test_artist_id')
        mock.assert_called_with('artist/details', artistId='test_artist_id')
        self.assertEqual(
            artist.url,
            u'http://www.7digital.com/artist/bl%C3%B3nde/?partner=7999')

    def test_get_artist_details_imagesize(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-artist-details.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            sdi.get_artist_details('test_artist_id', 1000)
        mock.assert_called_with('artist/details', artistId='test_artist_id',
                                imageSize=1000)

    def test_get_album_tracks(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-album-tracks.xml'))
        with patch.object(sdi, '_hit_backend') as mock:
            mock.return_value = sample_xml
            resp = sdi.get_album_tracks('test_album_id')
        mock.assert_called_with('release/tracks', releaseId='test_album_id')
        self.assertEqual(len(resp), 8)
        obj = resp[0]  # a sample
        self.assertEqual(obj.id, u'37720847')
        self.assertEqual(obj.title, u'All at Once')
        self.assertEqual(obj.length, 254)
        self.assertEqual(obj.price, u'£0.99')
        self.assertEqual(obj.price_amount, u'0.99')
        self.assertEqual(obj.price_currency, u'£')

        # assert the order is ok
        self.assertEqual(resp[1].title, 'Embers')
        self.assertEqual(resp[2].title, 'If You Believe')
        self.assertEqual(resp[3].title, 'Out of Style')
        self.assertEqual(resp[4].title, 'Brother')
        self.assertEqual(resp[5].title, 'Millionaires')
        self.assertEqual(resp[6].title, 'Feel the Rain')
        self.assertEqual(resp[7].title, 'Lightning')

    def test_extract_albums_filter_repeated(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-repeated-albums.xml'))
        resp = sdi._extract_albums(sample_xml)

        self.assertEqual(len(resp), 2)
        obj = resp[0]
        self.assertEqual(obj.id, u'3487738')
        self.assertEqual(obj.title, u'Turn Blue')
        obj = resp[1]
        self.assertEqual(obj.id, u'65697')
        self.assertEqual(obj.title, u'White Pony')

    def test_extract_tracks_filter_repeated(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-repeated-tracks.xml'))
        resp = sdi._extract_tracks(sample_xml)

        self.assertEqual(len(resp), 2)
        obj = resp[0]
        self.assertEqual(obj.id, u'36817572')
        self.assertEqual(obj.title, u'Ghost')
        obj = resp[1]
        self.assertEqual(obj.id, u'36920028')
        self.assertEqual(obj.title, u'Ishq Da Rogh (Round Two)')

    def test_unicode(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-unicode.xml'))
        resp = sdi._extract_tracks(sample_xml)

        self.assertEqual(len(resp), 2)
        obj = resp[0]
        self.assertEqual(obj.id, u'36817572')
        self.assertEqual(obj.url, u'http://www.7digital.com/artist/ella'
                         '-henders%C3%B3n/release/ghost/?partner=7999&h=01')

    def test_build_artist_encodes_url(self):
        sdi = sevendigital.SevenDigitalInterface('api', 'secret', None, 'urlread')
        sample_xml = etree.XML(get_fixture('7digital-artist-search.xml'))
        with patch.object(sevendigital, 'url_to_ascii') as mock:
            mock.return_value = "foo"
            sdi._build_artist(sample_xml)
        mock.assert_called_with(None)


class SDIMock(object):
    """A mock of the 7digital interface."""

    def __init__(self, *auth):
        self.auth = auth

        # some fake structures that will be used as responses
        self._fake_albums = []
        self._fake_tracks = []
        self._fake_artists = []
        for container, struct in [
                (self._fake_albums, sevendigital.Album),
                (self._fake_tracks, sevendigital.Track),
                (self._fake_artists, sevendigital.Artist)]:
            for i in range(50):
                kwds = {f: '%s%02d' % (f, i) for f in struct._fields}
                if 'artist' in kwds:
                    a_kwds = {f: 'artist_%s%02d' % (f, i)
                              for f in sevendigital.Artist._fields}
                    kwds['artist'] = sevendigital.Artist(**a_kwds)
                container.append(struct(**kwds))
        self._fake_artist_details = sevendigital.Artist(
            id='fakeid', name='Fake Artist', image='fakeimage',
            url='fakeurl', bio='fakebio')

        # which returns what of those fake structures
        self._data_source = {
            'get_album_latest': '_fake_albums',
            'get_album_topseller': '_fake_albums',
            'get_track_topseller': '_fake_tracks',
            'get_album_tagged': '_fake_albums',
            'search_artists': '_fake_artists',
            'search_albums': '_fake_albums',
            'search_tracks': '_fake_tracks',
            'get_artist_details': '_fake_artist_details',
            'get_artist_albums': '_fake_albums',
            'get_artist_tracks': '_fake_tracks',
            'get_album_tracks': '_fake_tracks',
        }

        # to record the calls
        self.calls = []

        # to know which bunch of fakes to return
        self._counter = {}

    def build_preview_track_source(self, track_id):
        """A fake url creator."""
        return 'signed url for ' + track_id

    def __getattr__(self, name):
        def f(*args):
            """Fake function."""
            # record the call
            self.calls.append((name, args))

            src = self._data_source[name]
            data = getattr(self, src)
            if isinstance(data, list):
                # return a bunch of *something* different from other calls
                cnt = self._counter.get(src, 0)
                self._counter[src] = cnt + 1
                data = data[cnt * 10: (cnt + 1) * 10]
            if name.startswith('search'):
                return (25, data)
            else:
                return data
        return f


class AppTestCase(TestCase):
    """Test the wsgi app."""
    maxDiff = None

    def test_build_7d_interface(self):
        fake_config = dict(secrets={None: {
            'oauth_consumer_key': 'test key',
            'oauth_consumer_id': 'test id',
            'oauth_consumer_secret': 'test secret',
        }})
        app = sevendigital.App(fake_config)
        with patch.object(sevendigital, 'SevenDigitalInterface') as mock:
            list(app.search(query='', locale='en_US',
                            department='', country='US', partner=None))

        mock.assert_called_with('test key', 'test secret', 'US', app.urlread)

    def test_surfacing_not_department(self):
        app = sevendigital.App(FAKE_CONFIG)
        kwargs = dict(query='', locale='C', department='',
                      country='CO', partner=None)
        sdi = SDIMock()
        with patch.object(app, '_get_departments_tree') as mock_deptree:
            with patch.object(sevendigital, 'SevenDigitalInterface',
                              lambda *a, **k: sdi):
                mock_deptree.return_value = 'fake dep tree'
                resp = list(app.search(**kwargs))

        # check that all was called ok
        self.assertEqual(len(sdi.calls), 2)
        self.assertEqual(sdi.calls[0], ('get_album_latest', ()))
        self.assertEqual(sdi.calls[1], ('get_album_topseller', ()))
        mock_deptree.assert_called_with('', 'C')

        # check the response, first we have the departments, then the
        # category and two 'latest' albums...
        self.assertEqual(resp[0]['departments'], 'fake dep tree')
        cat = sevendigital.CATEGORY_GRID_MEDIUM.copy()
        cat['title'] = 'New releases'
        self.assertEqual(resp[1]['category'], cat)
        self.assertEqual(resp[2]['result'], {
            'cat_id': cat['id'],
            'uri': 'url00',
            'art': 'image00',
            'title': 'title00',
            'subtitle': 'artist_name00',
            'attributes': [{'style': 'default',
                            'value': 'price00'}],
            'preview_info': {'album_id': 'id00',
                             'release_date': 'release_date00',
                             'type': 'album',
                             'price': 'price00',
                             'artist': 'artist_name00'},
        })
        self.assertEqual(resp[3]['result']['title'], 'title01')

        # ...then other category (no title!) and three more 'latest' albums...
        self.assertEqual(resp[4]['category'], sevendigital.CATEGORY_GRID_SMALL)
        self.assertEqual(resp[5]['result'], {
            'cat_id': sevendigital.CATEGORY_GRID_SMALL['id'],
            'uri': 'url02',
            'art': 'image02',
            'title': 'title02',
            'subtitle': 'artist_name02',
            'attributes': [{'style': 'default',
                            'value': 'price02'}],
            'preview_info': {'album_id': 'id02',
                             'release_date': 'release_date02',
                             'type': 'album',
                             'price': 'price02',
                             'artist': 'artist_name02'},
        })
        self.assertEqual(resp[6]['result']['title'], 'title03')
        self.assertEqual(resp[7]['result']['title'], 'title04')

        # ...and finally, other category, and all the 'topseller' albums
        cat = sevendigital.CATEGORY_LIST_LONG.copy()
        cat['title'] = 'Best selling albums'
        self.assertEqual(resp[8]['category'], cat)
        self.assertEqual(resp[9]['result'], {
            'cat_id': cat['id'],
            'uri': 'url10',
            'art': 'image10',
            'title': 'title10',
            'subtitle': 'artist_name10',
            'attributes': [{'style': 'default',
                            'value': 'price10'}],
            'preview_info': {'album_id': 'id10',
                             'release_date': 'release_date10',
                             'type': 'album',
                             'price': 'price10',
                             'artist': 'artist_name10'},
        })
        self.assertEqual(resp[-1]['result']['title'], 'title19')

    def test_surfacing_newreleases(self):
        app = sevendigital.App(FAKE_CONFIG)
        kwargs = dict(query='', locale='C', department='newreleases',
                      country='CO', partner=None)
        sdi = SDIMock()
        with patch.object(app, '_get_departments_tree') as mock_deptree:
            with patch.object(sevendigital, 'SevenDigitalInterface',
                              lambda *a, **k: sdi):
                mock_deptree.return_value = 'fake dep tree'
                resp = list(app.search(**kwargs))

        # check that all was called ok
        self.assertEqual(len(sdi.calls), 1)
        self.assertEqual(sdi.calls[0], ('get_album_latest', ()))
        mock_deptree.assert_called_with('newreleases', 'C')

        # check the response, first we have the departments, then the
        # category and two 'latest' albums...
        self.assertEqual(resp[0]['departments'], 'fake dep tree')
        self.assertEqual(resp[1]['category'],
                         sevendigital.CATEGORY_GRID_MEDIUM)
        self.assertEqual(resp[2]['result'], {
            'cat_id': sevendigital.CATEGORY_GRID_MEDIUM['id'],
            'uri': 'url00',
            'art': 'image00',
            'title': 'title00',
            'subtitle': 'artist_name00',
            'attributes': [{'style': 'default',
                            'value': 'price00'}],
            'preview_info': {'album_id': 'id00',
                             'release_date': 'release_date00',
                             'type': 'album',
                             'price': 'price00',
                             'artist': 'artist_name00'},
        })
        self.assertEqual(resp[3]['result']['title'], 'title01')

        # ...then other category (no title!) and three more 'latest' albums...
        self.assertEqual(resp[4]['category'], sevendigital.CATEGORY_GRID_SMALL)
        self.assertEqual(resp[5]['result'], {
            'cat_id': sevendigital.CATEGORY_GRID_SMALL['id'],
            'uri': 'url02',
            'art': 'image02',
            'title': 'title02',
            'subtitle': 'artist_name02',
            'attributes': [{'style': 'default',
                            'value': 'price02'}],
            'preview_info': {'album_id': 'id02',
                             'release_date': 'release_date02',
                             'type': 'album',
                             'price': 'price02',
                             'artist': 'artist_name02'},
        })
        self.assertEqual(resp[6]['result']['title'], 'title03')
        self.assertEqual(resp[-1]['result']['title'], 'title09')

    def test_surfacing_topalbums(self):
        app = sevendigital.App(FAKE_CONFIG)
        kwargs = dict(query='', locale='C', department='topalbums',
                      country='CO', partner=None)
        sdi = SDIMock()
        with patch.object(app, '_get_departments_tree') as mock_deptree:
            with patch.object(sevendigital, 'SevenDigitalInterface',
                              lambda *a, **k: sdi):
                mock_deptree.return_value = 'fake dep tree'
                resp = list(app.search(**kwargs))

        # check that all was called ok
        self.assertEqual(len(sdi.calls), 1)
        self.assertEqual(sdi.calls[0], ('get_album_topseller', ()))
        mock_deptree.assert_called_with('topalbums', 'C')

        # check the response, first we have the departments, then the
        # category (no title!), and then all the albums
        self.assertEqual(resp[0]['departments'], 'fake dep tree')
        self.assertEqual(resp[1]['category'], sevendigital.CATEGORY_LIST_LONG)
        self.assertEqual(resp[2]['result'], {
            'cat_id': sevendigital.CATEGORY_LIST_LONG['id'],
            'uri': 'url00',
            'art': 'image00',
            'title': 'title00',
            'subtitle': 'artist_name00',
            'attributes': [{'style': 'default',
                            'value': 'price00'}],
            'preview_info': {'album_id': 'id00',
                             'release_date': 'release_date00',
                             'type': 'album',
                             'price': 'price00',
                             'artist': 'artist_name00'},
        })
        self.assertEqual(resp[-1]['result']['title'], 'title09')

    def test_surfacing_toptracks(self):
        app = sevendigital.App(FAKE_CONFIG)
        kwargs = dict(query='', locale='C', department='toptracks',
                      country='CO', partner=None)
        sdi = SDIMock()
        with patch.object(app, '_get_departments_tree') as mock_deptree:
            with patch.object(sevendigital, 'SevenDigitalInterface',
                              lambda *a, **k: sdi):
                mock_deptree.return_value = 'fake dep tree'
                resp = list(app.search(**kwargs))

        # check that all was called ok
        self.assertEqual(len(sdi.calls), 1)
        self.assertEqual(sdi.calls[0], ('get_track_topseller', ()))
        mock_deptree.assert_called_with('toptracks', 'C')

        # check the response, first we have the departments, then the
        # category (no title!), and then all the tracks
        self.assertEqual(resp[0]['departments'], 'fake dep tree')
        self.assertEqual(resp[1]['category'], sevendigital.CATEGORY_LIST_LONG)
        self.assertEqual(resp[2]['result'], {
            'cat_id': sevendigital.CATEGORY_LIST_LONG['id'],
            'uri': 'url00',
            'art': 'image00',
            'title': 'title00',
            'subtitle': 'artist_name00',
            'attributes': [{'style': 'default',
                            'value': 'price00'}],
            'preview_info': {'release_date': 'release_date00',
                             'track_id': 'id00',
                             'type': 'track',
                             'price': 'price00',
                             'artist': 'artist_name00',
                             'length': 'length00'},
        })
        self.assertEqual(resp[-1]['result']['title'], 'title09')

    def test_surfacing_genres_parent(self):
        app = sevendigital.App(FAKE_CONFIG)
        kwargs = dict(query='', locale='C', department='genres',
                      country='CO', partner=None)
        sdi = SDIMock()
        with patch.object(app, '_get_departments_tree') as mock_deptree:
            with patch.object(sevendigital, 'SevenDigitalInterface',
                              lambda *a, **k: sdi):
                mock_deptree.return_value = 'fake dep tree'
                resp = list(app.search(**kwargs))

        # check that all was called ok
        self.assertEqual(len(sdi.calls), 5)
        genres = [x[0][6:] for x in sevendigital.GENRES[:5]]
        for sdi_call, should_tag in zip(sdi.calls, genres):
            self.assertEqual(sdi_call, ('get_album_tagged', (should_tag,)))
        mock_deptree.assert_called_with('genres', 'C')

        # check the response, which is the department, and then the category
        # and a bunch of albums per genre
        self.assertEqual(resp[0]['departments'], 'fake dep tree')
        for i in range(5):
            genre_id, genre_label = sevendigital.GENRES[i]
            cat = sevendigital.CATEGORY_GRID_SMALL.copy()
            cat['id'] = 'cat-' + genre_id
            cat['title'] = genre_label
            self.assertEqual(resp[(i * 10) + 1 + i]['category'], cat)
            self.assertEqual(resp[(i * 10) + 2 + i]['result'], {
                'cat_id': cat['id'],
                'uri': 'url%02d' % (i * 10),
                'art': 'image%02d' % (i * 10),
                'title': 'title%02d' % (i * 10),
                'subtitle': 'artist_name%02d' % (i * 10),
                'attributes': [{'style': 'default',
                                'value': 'price%02d' % (i * 10)}],
                'preview_info': {'album_id': 'id%02d' % (i * 10),
                                 'release_date': 'release_date%02d' % (i * 10),
                                 'type': 'album',
                                 'price': 'price%02d' % (i * 10),
                                 'artist': 'artist_name%02d' % (i * 10)},
            })
            self.assertEqual(resp[(i * 10) + 11 + i]['result']['title'],
                             'title%02d' % (9 + i * 10))

    def test_surfacing_specific_genre(self):
        app = sevendigital.App(FAKE_CONFIG)
        kwargs = dict(query='', locale='C', department='genre-pop',
                      country='CO', partner=None)
        sdi = SDIMock()
        with patch.object(app, '_get_departments_tree') as mock_deptree:
            with patch.object(sevendigital, 'SevenDigitalInterface',
                              lambda *a, **k: sdi):
                mock_deptree.return_value = 'fake dep tree'
                resp = list(app.search(**kwargs))

        # check that all was called ok
        self.assertEqual(len(sdi.calls), 1)
        self.assertEqual(sdi.calls[0], ('get_album_tagged', ('pop',)))
        mock_deptree.assert_called_with('genre-pop', 'C')

        # check the response, first we have the departments, then the
        # category (no title!), and then all the albums
        self.assertEqual(resp[0]['departments'], 'fake dep tree')
        self.assertEqual(resp[1]['category'], sevendigital.CATEGORY_GRID_SMALL)
        self.assertEqual(resp[2]['result'], {
            'cat_id': sevendigital.CATEGORY_GRID_SMALL['id'],
            'uri': 'url00',
            'art': 'image00',
            'title': 'title00',
            'subtitle': 'artist_name00',
            'attributes': [{'style': 'default',
                            'value': 'price00'}],
            'preview_info': {'album_id': 'id00',
                             'release_date': 'release_date00',
                             'type': 'album',
                             'price': 'price00',
                             'artist': 'artist_name00'},
        })
        self.assertEqual(resp[-1]['result']['title'], 'title09')

    def test_search_generic(self):
        app = sevendigital.App(FAKE_CONFIG)
        kwargs = dict(query=u'test moño', locale='C', department=None,
                      country='CO', partner=None)
        sdi = SDIMock()
        with patch.object(sevendigital, 'SevenDigitalInterface',
                          lambda *a, **k: sdi):
            resp = list(app.search(**kwargs))

        # check that all was called ok
        self.assertEqual(len(sdi.calls), 3)
        self.assertEqual(sdi.calls[0], ('search_artists', (u'test moño',)))
        self.assertEqual(sdi.calls[1], ('search_albums', (u'test moño',)))
        self.assertEqual(sdi.calls[2], ('search_tracks', (u'test moño',)))

        cat = sevendigital.CATEGORY_LIST_SHORT.copy()
        cat['title'] = 'Artists (25 results)'
        cat['id'] = 'artists'
        self.assertEqual(resp[0]['category'], cat)
        cq = ('scope://com.canonical.scopes.sevendigital?'
              'dep=specialsearch-id00&q=test%20mo%C3%B1o')
        self.assertEqual(resp[1]['result'], {
            'cat_id': cat['id'],
            'uri': cq,
            'art': 'image00',
            'title': 'name00',
        })
        self.assertEqual(resp[10]['result']['title'], 'name09')

        cat['title'] = 'Albums (25 results)'
        cat['id'] = 'albums'
        self.assertEqual(resp[11]['category'], cat)
        self.assertEqual(resp[12]['result'], {
            'cat_id': cat['id'],
            'uri': 'url00',
            'art': 'image00',
            'title': 'title00',
            'subtitle': 'artist_name00',
            'attributes': [{'style': 'default',
                            'value': 'price00'}],
            'preview_info': {'album_id': 'id00',
                             'release_date': 'release_date00',
                             'type': 'album',
                             'price': 'price00',
                             'artist': 'artist_name00'},
        })
        self.assertEqual(resp[21]['result']['title'], 'title09')

        cat['title'] = 'Tracks (25 results)'
        cat['id'] = 'tracks'
        self.assertEqual(resp[22]['category'], cat)
        self.assertEqual(resp[23]['result'], {
            'cat_id': cat['id'],
            'uri': 'url00',
            'art': 'image00',
            'title': 'title00',
            'subtitle': 'artist_name00',
            'attributes': [{'style': 'default',
                            'value': 'price00'}],
            'preview_info': {'release_date': 'release_date00',
                             'track_id': 'id00',
                             'type': 'track',
                             'price': 'price00',
                             'artist': 'artist_name00',
                             'length': 'length00'},
        })
        self.assertEqual(resp[32]['result']['title'], 'title09')

    def test_search_special(self):
        app = sevendigital.App(FAKE_CONFIG)
        kwargs = dict(query='test query', locale='C', country='CO',
                      department='specialsearch-artist_id', partner=None)
        sdi = SDIMock()
        with patch.object(sevendigital, 'SevenDigitalInterface',
                          lambda *a, **k: sdi):
            resp = list(app.search(**kwargs))

        # check that all was called ok
        self.assertEqual(len(sdi.calls), 3)
        self.assertEqual(sdi.calls[0], ('get_artist_details', ('artist_id',)))
        self.assertEqual(sdi.calls[1], ('get_artist_albums', ('artist_id',)))
        self.assertEqual(sdi.calls[2], ('get_artist_tracks', ('artist_id',)))

        # check the response, first the image, then the albums and tracks
        self.assertEqual(resp[0]['category'], sevendigital.CATEGORY_GRID_LARGE)
        self.assertEqual(resp[1]['result'], {
            'cat_id': sevendigital.CATEGORY_GRID_LARGE['id'],
            'title': 'Fake Artist',
            'art': 'fakeimage',
            'uri': 'fakeurl',
        })

        cat = sevendigital.CATEGORY_LIST_SHORT.copy()
        cat['title'] = 'Albums'
        cat['id'] = 'albums'
        self.assertEqual(resp[2]['category'], cat)
        self.assertEqual(resp[3]['result'], {
            'cat_id': cat['id'],
            'uri': 'url00',
            'art': 'image00',
            'title': 'title00',
            'subtitle': 'artist_name00',
            'attributes': [{'style': 'default',
                            'value': 'price00'}],
            'preview_info': {'album_id': 'id00',
                             'release_date': 'release_date00',
                             'type': 'album',
                             'price': 'price00',
                             'artist': 'artist_name00'},
        })
        self.assertEqual(resp[12]['result']['title'], 'title09')

        cat['title'] = 'Tracks'
        cat['id'] = 'tracks'
        self.assertEqual(resp[13]['category'], cat)
        self.assertEqual(resp[14]['result'], {
            'cat_id': cat['id'],
            'uri': 'url00',
            'art': 'image00',
            'title': 'title00',
            'subtitle': 'artist_name00',
            'attributes': [{'style': 'default',
                            'value': 'price00'}],
            'preview_info': {'release_date': 'release_date00',
                             'track_id': 'id00',
                             'type': 'track',
                             'price': 'price00',
                             'artist': 'artist_name00',
                             'length': 'length00'},
        })
        self.assertEqual(resp[23]['result']['title'], 'title09')

    @patch.object(sevendigital, 'translator', lambda t, l: 'translated-' + t)
    def test_build_count_justone(self):
        app = sevendigital.App(FAKE_CONFIG)
        resp = app._build_count("Prefix", 1, 'C')
        self.assertEqual(resp, "translated-Prefix translated-(1 result)")

    @patch.object(sevendigital, 'translator', lambda t, l: 'translated-' + t)
    def test_build_count_several(self):
        app = sevendigital.App(FAKE_CONFIG)
        resp = app._build_count("Prefix", 5, 'C')
        self.assertEqual(resp, "translated-Prefix translated-(5 results)")

    def test_preview_album_simple(self):
        kwargs = dict(locale='C', country='US', partner=None, result={
            'uri': 'test url',
            'art': 'test image',
            'title': 'test title',
            'preview_info': {
                'type': 'album',
                'album_id': 'test alb id',
                'release_date': '2000-01-28',
                'price': '$test',
                'artist': 'test artist_name',
            },
        })

        app = sevendigital.App(FAKE_CONFIG)
        sdi = SDIMock()
        with patch.object(sevendigital, 'SevenDigitalInterface',
                          lambda *a, **k: sdi):
            resp = list(app.preview(**kwargs))

        # check that SDI was called ok
        self.assertEqual(len(sdi.calls), 1)
        self.assertEqual(sdi.calls[0], ('get_album_tracks', ('test alb id',)))

        # check the response
        self.assertEqual(resp[0]['widget'], {
            'id': 'art',
            'type': 'image',
            'source': 'test image'
        })
        self.assertEqual(resp[1]['widget'], {
            'id': 'header',
            'type': 'header',
            'uri': 'test url',
            'title': 'test title',
            'subtitle': 'by test artist_name',
            'attributes': [{'style': 'default',
                            'value': u'Released: 28/01/2000'}],
        })
        self.assertEqual(resp[2]['widget'], {
            'id': 'button',
            'type': 'actions',
            'actions': [{'id': 'test url',
                        'label': '$test on 7digital'}],
        })
        should_tracks = []
        for i in range(10):
            should_tracks.append({
                'title': '%02d. title%02d' % (i + 1, i),
                'source': 'signed url for id%02d' % i,
                'length': 'length%02d' % i,
            })
        self.assertEqual(resp[3]['widget'], {
            'id': 'tracks',
            'type': 'audio',
            'tracks': should_tracks,
        })

    def test_preview_distributor_track(self):
        kwargs = dict(locale='C', country='US', partner=None, result={
            'uri': 'test url',
            'art': 'test image',
            'title': 'test title',
            'preview_info': {
                'type': 'track',
                'track_id': 'test trk id',
                'release_date': '2000-01-28',
                'price': '$test',
                'artist': 'test artist_name',
                'length': 'test length',
            },
        })

        app = sevendigital.App(FAKE_CONFIG)
        sdi = SDIMock()
        with patch.object(sevendigital, 'SevenDigitalInterface',
                          lambda *a, **k: sdi):
            resp = list(app.preview(**kwargs))

        # SDI shouldn't get called to get anything
        self.assertFalse(sdi.calls)

        self.assertEqual(resp[0]['widget'], {
            'id': 'header',
            'type': 'header',
            'uri': 'test url',
            'title': 'test title',
            'subtitle': 'by test artist_name',
            'attributes': [{'style': 'default',
                            'value': u'Released: 28/01/2000'}],
            'mascot': 'test image',
        })
        self.assertEqual(resp[1]['widget'], {
            'id': 'button',
            'type': 'actions',
            'actions': [{'id': 'test url',
                        'label': '$test on 7digital'}],
        })
        self.assertEqual(resp[2]['widget'], {
            'id': 'tracks',
            'type': 'audio',
            'tracks': [{
                'title': 'test title',
                'source': 'signed url for test trk id',
                'length': 'test length',
            }],
        })

    def test_preview_nothing(self):
        # the 'preview_info' is missing for the result!!
        kwargs = dict(locale='en_US', country='US', partner=None, result={
            'uri': 'test url',
            'art': 'test image',
            'title': 'test title',
        })

        app = sevendigital.App(FAKE_CONFIG)
        resp = list(app.preview(**kwargs))
        self.assertEqual(resp, [])


class DepartmentTreeTestCase(TestCase):
    """Test how the department tree is built."""

    fake_genres = [
        ('genre-g1', 'Genre 1'),
        ('genre-g2', 'Genre 2'),
    ]
    maxDiff = None

    @patch.object(sevendigital, 'translator', lambda t, l: 'translated-' + t)
    def test_root(self):
        app = sevendigital.App(FAKE_CONFIG)
        tree = app._get_departments_tree('', 'C')
        _base = [
            ('newreleases', 'New releases', False),
            ('topalbums', 'Top albums', False),
            ('toptracks', 'Top tracks', False),
            ('genres', 'Genres', True),
        ]
        subdeps = []
        for did, label, haschild in _base:
            subdeps.append({
                'label': 'translated-' + label,
                'canned_query': 'scope://com.canonical.scopes.sevendigital?'
                                'dep=' + did,
                'has_subdepartments': haschild,
            })
        self.assertEqual(tree, {
            'label': 'translated-All categories',
            'canned_query': 'scope://com.canonical.scopes.sevendigital',
            'subdepartments': subdeps,
        })

    @patch.object(sevendigital, 'GENRES', fake_genres)
    @patch.object(sevendigital, 'translator', lambda t, l: 'translated-' + t)
    def test_base_simple(self):
        app = sevendigital.App(FAKE_CONFIG)
        tree = app._get_departments_tree('toptracks', 'C')
        self.assertEqual(tree, {
            'label': 'translated-All categories',
            'canned_query': 'scope://com.canonical.scopes.sevendigital',
            'subdepartments': [{
                'label': 'translated-Top tracks',
                'canned_query': 'scope://com.canonical.scopes.sevendigital?'
                                'dep=toptracks',
                'subdepartments': [],
            }]
        })

    @patch.object(sevendigital, 'GENRES', fake_genres)
    @patch.object(sevendigital, 'translator', lambda t, l: 'translated-' + t)
    def test_base_complex(self):
        app = sevendigital.App(FAKE_CONFIG)
        tree = app._get_departments_tree('genres', 'C')
        self.assertEqual(tree, {
            'label': 'translated-All categories',
            'canned_query': 'scope://com.canonical.scopes.sevendigital',
            'subdepartments': [{
                'label': 'translated-Genres',
                'canned_query': 'scope://com.canonical.scopes.sevendigital?'
                                'dep=genres',
                'subdepartments': [{
                    'label': 'translated-Genre 1',
                    'canned_query': 'scope://com.canonical.scopes.sevendigital'
                                    '?dep=genre-g1',
                    'has_subdepartments': False,
                    }, {
                    'label': 'translated-Genre 2',
                    'canned_query': 'scope://com.canonical.scopes.sevendigital'
                                    '?dep=genre-g2',
                    'has_subdepartments': False,
                }],
            }]
        })

    @patch.object(sevendigital, 'GENRES', fake_genres)
    @patch.object(sevendigital, 'translator', lambda t, l: 'translated-' + t)
    def test_genre_leaf(self):
        app = sevendigital.App(FAKE_CONFIG)
        tree = app._get_departments_tree('genre-g1', 'C')
        self.assertEqual(tree, {
            'label': 'translated-Genres',
            'canned_query': 'scope://com.canonical.scopes.sevendigital?'
                            'dep=genres',
            'subdepartments': [{
                'label': 'translated-Genre 1',
                'canned_query': 'scope://com.canonical.scopes.sevendigital?'
                                'dep=genre-g1',
                'subdepartments': [],
            }]
        })
