# -*- coding: utf8 -*-

# Copyright 2014-2015 Canonical
# All Rights Reserved

"""Tests for the weatherchannel scope."""

import json

from datetime import datetime

from unittest import TestCase

from mock import patch, Mock

from src.scopes import weatherchannel, BadRequestError, BackendDataError
from src.scopes.tests import get_fixture

FAKE_AUTH = {'api_key': 'bar', 'partner': 'ptest'}
FAKE_CONFIG = {'base_image_url': 'foo', 'secrets': {None: FAKE_AUTH}}


class SearchTestCase(TestCase):
    """Search tests."""
    maxDiff = None

    def test_small_query(self):
        app = weatherchannel.App(FAKE_CONFIG)
        response = list(app.search(query=u'f', locale='en_GB', partner=None))
        self.assertEqual(response, [])

    def _check_simple_landpage(self, response):
        """Check the output for the several 'simple' cases below."""
        # the first category, with the special title
        self.assertEqual(response[0]['category']['id'],
                         weatherchannel.CATEGORY_GRID_MAIN['id'])
        self.assertEqual(response[0]['category']['title'], u'SE1, England, GB')

        # the two big items, the first very detailed for sanity check
        url = 'https://weather.com/weather/today/l/123,456?par=ptest'
        comment = u'Light rain. Low 4\xbaC.'
        self.assertDictEqual(response[1], dict(result={
            'art': 'foo/weatherchannel/suru/weather-showers.svg',
            'cat_id': weatherchannel.CATEGORY_GRID_MAIN['id'],
            'dnd_uri': url,
            'title': u"Tonight, light rain",
            'subtitle': u"13°C / 4°C",
            'uri': url,
            'latitude': 123,
            'longitude': 456,
            'attributes': [
                {'style': 'default', 'value': u'Rain: 40%'},
                {'style': 'default', 'value': u'Humidity: 73%'},
                {'style': 'default', 'value': u'Wind: Winds SW at 10 to 20 mph.'},
                {'style': 'default', 'value': ''},
            ],
            'preview_info': {
                'type': 'full',
                'url': url,
                'comment': comment,
                'art': 'foo/weatherchannel/suru/weather-showers.svg',
                'art_wlogo': 'foo/weatherchannel/suru/weather-showers-wlogo.svg',
                'uv': '0',
                'wind_sentence': u'Winds SW at 10 to 20 mph.',
                'location': u'SE1, England, GB',
                'max_temp': u'13°C',
                'min_temp': u'4°C',
                'pop': u'40%',
                'conditions': u'Light rain',
                'current_temp': u'6°C',
                'date': u'08 Feb',
                'day': u'Tonight',
                'humidity': '73%',
                'dew': u'1°C',
                'feels_like': u'1°C',
                'sunrise': '7:28',
                'sunset': '17:01',
                'visibility': u'6.03 mi',
                'latitude': 123,
                'longitude': 456,
            }
        }))

        resp = response[2]['result']
        self.assertEqual(resp['cat_id'],
                         weatherchannel.CATEGORY_GRID_MAIN['id'])
        self.assertEqual(resp['title'], u'Tomorrow, morning clouds'
                         ' followed by afternoon sun')
        self.assertEqual(resp['subtitle'], u'8°C / 4°C')
        self.assertEqual(resp['art'],
                         'foo/weatherchannel/suru/weather-clouds.svg')

        # the rest of the items
        cat = weatherchannel.CATEGORY_GRID_REST
        self.assertEqual(response[3], dict(category=cat))
        resp = response[4]['result']
        self.assertEqual(resp['cat_id'], cat['id'])
        self.assertEqual(resp['title'], u'Morning clouds followed by afternoon sun')
        self.assertEqual(resp['subtitle'], 'Tomorrow night')
        self.assertEqual(resp['art'],
                         'foo/weatherchannel/suru/weather-clouds-night.svg')
        resp = response[5]['result']
        self.assertEqual(resp['cat_id'], cat['id'])
        self.assertEqual(resp['title'], u'Partly cloudy')
        self.assertEqual(resp['subtitle'], 'Wednesday')
        self.assertEqual(resp['art'],
                         'foo/weatherchannel/suru/weather-clouds.svg')
        resp = response[6]['result']
        self.assertEqual(resp['cat_id'], cat['id'])
        self.assertEqual(resp['title'], u'Partly cloudy')
        self.assertEqual(resp['subtitle'], 'Wednesday night')
        self.assertEqual(resp['art'],
                         'foo/weatherchannel/suru/weather-few-clouds-night.svg')
        resp = response[7]['result']
        self.assertEqual(resp['cat_id'], cat['id'])
        self.assertEqual(resp['title'], u'Generally clear')
        self.assertEqual(resp['subtitle'], 'Thursday')
        self.assertEqual(resp['art'],
                         'foo/weatherchannel/suru/weather-few-clouds.svg')

    def test_landpage_simple(self):
        app = weatherchannel.App(FAKE_CONFIG)

        with patch.object(weatherchannel.TWCInterface, '_get_forecast') as mock:
            with patch.object(weatherchannel.TWCInterface, 'get_location') as mock1:
                mock1.return_value = {
                    u'locality': u'SE1',
                    u'admin_district': u'England',
                    u'country': u'United Kingdom',
                    u'country_code': u'GB',
                }
                mock.return_value = json.loads(get_fixture('weatherchannel-simple.json'))
                response = list(app.search(query=u'', locale='en_GB',
                                platform='desktop', partner=None,
                                latitude=123, longitude=456))

        # got the info as it should
        mock.assert_called_once_with(123, 456)
        mock1.assert_called_once()
        return self._check_simple_landpage(response)

    def test_landpage_simple_noinfo(self):
        app = weatherchannel.App(FAKE_CONFIG)

        with patch.object(weatherchannel.TWCInterface, '_get_forecast') as mock:
            with patch.object(weatherchannel.TWCInterface, 'get_location') as mock1:
                mock1.return_value = {
                    u'locality': u'SE1',
                    u'admin_district': u'England',
                    u'country': u'United Kingdom',
                    u'country_code': u'GB',
                }
                mock.return_value = json.loads(get_fixture('weatherchannel-noforecast.json'))
                response = list(app.search(query=u'', locale='en_GB',
                                platform='desktop', partner=None,
                                latitude=123, longitude=456))

        # no info
        self.assertEqual(response, [])

    def test_landpage_missing_lat_long(self):
        app = weatherchannel.App(FAKE_CONFIG)
        with self.assertRaises(BadRequestError):
            list(app.search(query=u'', locale='en_GB', platform='desktop', partner=None))

    def test_landpage_empty_lat_long(self):
        app = weatherchannel.App(FAKE_CONFIG)
        with self.assertRaises(BadRequestError):
            list(app.search(query=u'', locale='en_GB', platform='desktop', partner=None,
                            latitude=0, longitude=0.0))

    def test_landpage_semiempty_lat_long(self):
        app = weatherchannel.App(FAKE_CONFIG)
        with patch.object(weatherchannel.TWCInterface, '_get_forecast') as mock:
            with patch.object(weatherchannel.TWCInterface, 'get_location') as mock1:
                mock1.return_value = {
                    u'locality': u'SE1',
                    u'admin_district': u'England',
                    u'country': u'United Kingdom',
                    u'country_code': u'GB',
                }
                mock.return_value = json.loads(get_fixture('weatherchannel-simple.json'))
                # doesn't raise an exception, even if one of the lat/long is 0
                list(app.search(query=u'', locale='en_GB', platform='desktop', partner=None,
                                latitude=50, longitude=0.0))

    def test_landpage_chinese(self):
        app = weatherchannel.App(FAKE_CONFIG)

        with patch.object(weatherchannel.TWCInterface, '_get_forecast') as mock_search:
            with patch.object(weatherchannel.TWCInterface, 'get_location') as mock_loc:
                mock_loc.return_value = {
                    u'locality': u'SE1',
                    u'admin_district': u'\u82f1\u683c\u5170',
                    u'country': u'United Kingdom',
                    u'country_code': u'GB',
                }
                mock_search.return_value = json.loads(
                    get_fixture('weatherchannel-chinese.json'))
                response = list(app.search(query=u'', locale='en_US',
                                platform='desktop', partner=None,
                                latitude=123, longitude=456))

        # category
        self.assertEqual(response[0]['category']['id'],
                         weatherchannel.CATEGORY_GRID_MAIN['id'])
        self.assertEqual(response[0]['category']['title'], u'SE1, 英格兰, GB')

        # the result
        url = 'https://weather.com/weather/today/l/123,456?par=ptest'
        comment = u'\u5c11\u4e91\u3002 \u6700\u4f4e 4\xbaC\u3002'
        self.assertDictEqual(response[1], dict(result={
            'art': 'foo/weatherchannel/suru/weather-clouds-night.svg',
            'cat_id': weatherchannel.CATEGORY_GRID_MAIN['id'],
            'subtitle': u'4°C',
            'title': u'\u4eca\u5929\u665a\u4e0a, \u5c11\u4e91\u3002 \u6700\u4f4e 4\xbac\u3002',
            'dnd_uri': url,
            'latitude': 123,
            'longitude': 456,
            'uri': url,
            'attributes': [
                {'style': 'default', 'value': u'Rain: 20%'},
                {'style': 'default', 'value': u'Humidity: 76%'},
                {'style': 'default',
                 'value': (u'Wind: \u897f\u5357\u98a8 \u98ce 15'
                           u' \u5230 30 \u6bcf km / h \u3002')},
                {'style': 'default', 'value': ''},
            ],
            'preview_info': {
                'type': 'full',
                'comment': comment,
                'wind_sentence': u'西南風 风 15 到 30 每 km / h 。',
                'art': 'foo/weatherchannel/suru/weather-clouds-night.svg',
                'art_wlogo': 'foo/weatherchannel/suru/weather-clouds-night-wlogo.svg',
                'uv': '0',
                'url': url,
                'humidity': '76%',
                'location': u'SE1, 英格兰, GB',
                'max_temp': None,
                'min_temp': u'4°C',
                'pop': u'20%',
                'conditions':  u'\u5c11\u4e91\u3002 \u6700\u4f4e 4\xbaC\u3002',
                'current_temp': u'6°C',
                'date': u'08 Feb',
                'day': u'今天晚上',
                'dew': u'1°C',
                'feels_like': u'1°C',
                'sunrise': '7:28am',
                'sunset': '5:01pm',
                'visibility': u'9.7 km',
                'latitude': 123,
                'longitude': 456,
            },
        }))

    def test_search_simple(self):
        app = weatherchannel.App(FAKE_CONFIG)
        fake_locations = [{
            u'country': u'United Kingdom',
            u'name': u'London',
            u'state': u'GLA',
            u'latitude': 123,
            u'longitude': 456,
        }, {
            u'country': u'Canada',
            u'name': u'London',
            u'state': u'ON',
            u'latitude': 321,
            u'longitude': 654,
        }]

        with patch.object(weatherchannel.TWCInterface, 'search_locations') as mock:
            mock.return_value = fake_locations
            response = list(app.search(query=u'foo', locale='en_GB',
                                       partner=None))

        # got the info as it should
        mock.assert_called_once_with(u'foo')

        # category and both result items (one per location)
        self.assertEqual(response[0], dict(
            category=weatherchannel.CATEGORY_LIST))
        self.assertDictEqual(response[1], dict(result={
            'cat_id': weatherchannel.CATEGORY_LIST['id'],
            'uri': 'scope://com.canonical.scopes.weatherchannel?'
                   'dep=123%7C456&q=foo',
            'dnd_uri': 'scope://com.canonical.scopes.weatherchannel?'
                       'dep=123%7C456&q=foo',
            'title': u"London, GLA, United Kingdom",
            'preview_info': {
                'type': 'location',
                'latitude': 123,
                'longitude': 456,
            }
        }))
        self.assertDictEqual(response[2], dict(result={
            'cat_id': weatherchannel.CATEGORY_LIST['id'],
            'uri': 'scope://com.canonical.scopes.weatherchannel?'
                   'dep=321%7C654&q=foo',
            'dnd_uri': 'scope://com.canonical.scopes.weatherchannel?'
                       'dep=321%7C654&q=foo',
            'title': u"London, ON, Canada",
            'preview_info': {
                'type': 'location',
                'latitude': 321,
                'longitude': 654,
            }
        }))

    def test__get_location_search_sorting(self):
        app = weatherchannel.App(FAKE_CONFIG)
        twci = weatherchannel.TWCInterface('urlread', 'foo', FAKE_AUTH, 'en_GB', 'desktop')
        with patch.object(weatherchannel.TWCInterface, 'search_locations') as mock:
            mock.return_value = json.loads(get_fixture('weatherchannel-locations.json'))
            result = list(app._get_location_search(twci, 'foo', 'CO'))
            self.assertEqual(result[1]['result']['title'],
                             u'Santa Fe, Capital District, Colombia')
            result = list(app._get_location_search(twci, 'foo', 'AR'))
            self.assertEqual(result[1]['result']['title'],
                             u'Santa Fe, Santa Fe, Argentina')

    def test__get_location_search_nonascii_query(self):
        app = weatherchannel.App(FAKE_CONFIG)
        query = u"Cañón del Colorado"
        twci = weatherchannel.TWCInterface('urlread', 'foo', FAKE_AUTH, 'en_GB', 'desktop')
        with patch.object(twci, "search_locations") as mock:
            mock.return_value = [dict(name='name', state='state',
                                      country='country',
                                      latitude='123',
                                      longitude='456')]
            resp = list(app._get_location_search(twci, query, 'country'))

        mock.assert_called_with(query)
        self.assertDictEqual(resp[1]['result'], {
            'cat_id': 'cat-list',
            'preview_info':  {
                'latitude': '123',
                'longitude': '456',
                'type': 'location'
            },
            'uri': 'scope://com.canonical.scopes.weatherchannel?'
                   'dep=123%7C456&q=Ca%C3%B1%C3%B3n%20del%20Colorado',
            'dnd_uri': 'scope://com.canonical.scopes.weatherchannel?'
                       'dep=123%7C456&q=Ca%C3%B1%C3%B3n%20del%20Colorado',
            'title': u'name, state, country'
        })

    def test_get_location_with_strings(self):
        "Test get_location converts lat/long to float."
        urlread = Mock(return_value=json.dumps({'addresses': ['foo']}))
        twci = weatherchannel.TWCInterface(urlread, 'foo', FAKE_AUTH, 'en_GB', 'desktop')
        resp = twci.get_location("123", "456")
        self.assertEqual(resp, 'foo')
        urlread.assert_called_once_with(
            'https://api.weather.com/v2/location?geocode=123.00,456.'
            '00&language=en-GB&format=json&apiKey=bar')

    def test_get_location_empty(self):
        "Test get_location handles the no-locations case."
        urlread = Mock(return_value=json.dumps({'addresses': []}))
        twci = weatherchannel.TWCInterface(urlread, 'foo', FAKE_AUTH, 'en_GB', 'desktop')
        resp = twci.get_location("123", "456")
        self.assertEqual(resp, {})
        urlread.assert_called_once_with(
            'https://api.weather.com/v2/location?geocode=123.00,456.'
            '00&language=en-GB&format=json&apiKey=bar')


class PreviewTestCase(TestCase):
    """Tests for the preview function."""
    maxDiff = None

    def _check_simple_output(self, response):
        """Check the output for the several 'simple' cases below."""
        # check the 'image'
        self.assertEqual(response[0], dict(widget={
            'id': 'art',
            'type': 'image',
            'source': 'foo/weatherchannel/weather-showers.svg',
        }))

        # check the 'header'
        self.assertDictEqual(response[1], dict(widget={
            'id': 'header1',
            'type': 'header',
            'uri': 'http://www.weather.com/today?par=ptest#logo',
            'subtitle': u'48°F / 39°F',
            'title': u'Today: Showers',
        }))

        # check the actions
        self.assertDictEqual(response[2], dict(widget={
            'id': 'actions',
            'type': 'actions',
            'actions': [{
                'icon': None,
                'id': 'action1',
                'label': '10 day forecast',
                'uri': 'https://weather.com/weather/10day/l/123,456?par=ptest'
            }],
        }))

        # check the text
        self.assertDictEqual(response[3], dict(widget={
            'id': 'table1',
            'values': [
                (u'Chance of rain', u'40%'),
                (u'Humidity', u'83%'),
                (u'Feels like', u'7°C'),
                (u'Wind', u'S 16 mph'),
                (u'UV index', u'0'),
                (u'Visibility', u'6 km'),
                (u'Dew point', u'7°C'),
                (u'Sunrise', u'4:31am'),
                (u'Sunset', u'1:59pm'),
            ],
            'type': 'table',
        }))

    def test_full(self):
        # fake a previous result with all the info
        result = {
            'art': 'foo/weatherchannel/weather-showers.svg',
            'subtitle': u'Mainly cloudy. Localized flooding is possible.',
            'dnd_uri': 'http://www.weather.com/weather?par=ptest#logo',
            'title': '',
            'url': 'http://www.weather.com/today?par=ptest#logo',
            'preview_info': {
                'type': 'full',
                'wind_sentence': 'S 16 mph',
                'comment': u'Mainly cloudy. Localized flooding is possible.',
                'uv': '0',
                'latitude': 123,
                'longitude': 456,
                'url': 'http://www.weather.com/today?par=ptest#logo',
                'humidity': '83%',
                'art': 'foo/weatherchannel/weather-showers.svg',
                'location': u'London, GLA, UK',
                'max_temp': u'48°F',
                'min_temp': u'39°F',
                'pop': '40%',
                'conditions': u'Showers',
                'current_temp': u'50°F',
                'date': '06 Feb',
                'day': 'Today',
                'dew': u'7°C',
                'feels_like': u'7°C',
                'visibility': u'6 km',
                'sunrise': '4:31am',
                'sunset': '1:59pm',
            }
        }
        app = weatherchannel.App(FAKE_CONFIG)
        response = list(app.preview(result=result, locale='en_US',
                        platform='desktop', partner=None))
        return self._check_simple_output(response)

    def test_location(self):
        # fake a previous result with only the location
        result = {
            'art': 'foo/weatherchannel/weather-showers.svg',
            'subtitle': u'Mainly cloudy. Localized flooding is possible.',
            'dnd_uri': 'http://www.weather.com/weather?par=ptest#logo',
            'title': '',
            'url': 'http://www.weather.com/today?par=ptest#logo',
            'preview_info': {
                'type': 'location',
                'latitude': 123,
                'longitude': 456,
            }
        }

        fake_forecast = {
            'wind_sentence': 'S 16 mph',
            'comment': u'Mainly cloudy. Localized flooding is possible.',
            'uv': '0',
            'url': 'http://www.weather.com/today?par=ptest#logo',
            'humidity': '83%',
            'art': 'foo/weatherchannel/weather-showers.svg',
            'location': u'London, GLA, UK',
            'max_temp': u'48°F',
            'min_temp': u'39°F',
            'pop': '40%',
            'conditions': u'Showers',
            'current_temp': u'50°F',
            'date': '06 Feb',
            'day': 'Today',
            'dew': u'7°C',
            'feels_like': u'7°C',
            'visibility': u'6 km',
            'sunrise': '4:31am',
            'sunset': '1:59pm',
        }
        app = weatherchannel.App(FAKE_CONFIG)
        with patch.object(weatherchannel.TWCInterface, 'get_forecast') as mock_get_f:
            mock_get_f.return_value = [fake_forecast]
            response = list(app.preview(result=result, locale='en_US',
                            platform='desktop', partner=None))
        mock_get_f.assert_called_with(123, 456)
        return self._check_simple_output(response)

    def test_some_info_not_present(self):
        # fake a previous result with all the info
        result = {
            'art': 'foo/weatherchannel/weather-showers.svg',
            'subtitle': u'Mainly cloudy. Localized flooding is possible.',
            'dnd_uri': 'http://www.weather.com/weather?par=ptest#logo',
            'title': '',
            'url': 'http://www.weather.com/today?par=ptest#logo',
            'preview_info': {
                'type': 'full',
                'wind_sentence': 'S 16 mph',
                'comment': u'Mainly cloudy. Localized flooding is possible.',
                'uv': '',   # empty!
                'latitude': 123,
                'longitude': 456,
                'url': 'http://www.weather.com/today?par=ptest#logo',
                'humidity': '83%',
                'art': 'foo/weatherchannel/weather-showers.svg',
                'location': u'London, GLA, UK',
                'max_temp': None,  # in None!
                'min_temp': u'39°',
                'pop': '40%',
                'conditions': u'Showers',
                'current_temp': u'50°',
                'date': '06 Feb',
                'day': 'Today',
                'feels_like': None,
                'sunrise': None,
                'sunset': None,
                'visibility': u'6 km',
                'dew': u'7°C',
            }
        }
        app = weatherchannel.App(FAKE_CONFIG)
        response = list(app.preview(result=result, locale='en_US',
                        platform='desktop', partner=None))

        # check the text, without max temp or UV info
        self.assertDictEqual(response[3], dict(widget={
            'id': 'table1',
            'values': [
                (u'Chance of rain', u'40%'),
                (u'Humidity', u'83%'),
                (u'Wind', u'S 16 mph'),
                (u'Visibility', u'6 km'),
                (u'Dew point', u'7°C'),
            ],
            'type': 'table',
        }))


class TWCInterfaceTestCase(TestCase):
    """Tests for the The Weather Channel Interface."""
    maxDiff = None

    def test_locale_to_language_conversion(self):
        # Supported by TWC, stays
        twci = weatherchannel.TWCInterface('urlread', 'base_url', 'auth', 'en_US', 'formfactor')
        self.assertEqual(twci.language, 'en-US')
        # Unsupported exactly, gets shortened
        twci = weatherchannel.TWCInterface('urlread', 'base_url', 'auth', 'en_CA', 'formfactor')
        self.assertEqual(twci.language, 'en')
        # Unknown, gets replaced by en-US default
        twci = weatherchannel.TWCInterface('urlread', 'base_url', 'auth', 'foo_bar', 'formfactor')
        self.assertEqual(twci.language, 'en-US')

    def test_formattime_pm(self):
        twci = weatherchannel.TWCInterface('urlread', 'base_url', 'auth', 'en_US', 'formfactor')
        resp = twci._format_time(datetime(2000, 1, 1, 15, 12, 4))
        self.assertEqual(resp, "3:12pm")

    def test_formattime_am(self):
        twci = weatherchannel.TWCInterface('urlread', 'base_url', 'auth', 'en_US', 'formfactor')
        resp = twci._format_time(datetime(2000, 1, 1, 15, 12, 4))
        self.assertEqual(resp, "3:12pm")

    def test_formattime_24h(self):
        twci = weatherchannel.TWCInterface('urlread', 'base_url', 'auth', 'es_AR', 'formfactor')
        resp = twci._format_time(datetime(2000, 1, 1, 15, 12, 4))
        self.assertEqual(resp, "15:12")

    def test_formattime_padding(self):
        twci = weatherchannel.TWCInterface('urlread', 'base_url', 'auth', 'es_AR', 'formfactor')
        resp = twci._format_time(datetime(2000, 1, 1, 5, 2, 4))
        self.assertEqual(resp, "5:02")

    def test_forecast_parsing_simple(self):
        twci = weatherchannel.TWCInterface('urlread', 'foo', FAKE_AUTH, 'en_US', 'desktop')
        forecast = json.loads(get_fixture('weatherchannel-simple.json'))
        with patch.object(weatherchannel.TWCInterface, 'get_location') as mock1:
            mock1.return_value = {
                u'locality': u'SE1',
                u'admin_district': u'England',
                u'country': u'United Kingdom',
                u'country_code': u'GB',
            }
            resp = twci._parse_forecast(forecast, 123, 456)

        # check we got them all, and first four in detail
        self.assertEqual(len(resp), 11)

        self.assertDictEqual(resp[0], {
            'url': 'https://weather.com/weather/today/l/123,456?par=ptest',
            'art': 'foo/weatherchannel/suru/weather-showers.svg',
            'art_wlogo': 'foo/weatherchannel/suru/weather-showers-wlogo.svg',
            'day': u'Tonight',
            'comment': u'Light rain. Low 4\xbaC.',
            'date': u'08 Feb',
            'max_temp': u'13°C',
            'min_temp': u'4°C',
            'conditions': u'Light rain',
            'pop': u'40%',
            'wind_sentence': u'Winds SW at 10 to 20 mph.',
            'uv': '0',
            'current_temp': u'6°C',
            'humidity': '73%',
            'location': u'SE1, England, GB',
            'dew': u'1°C',
            'feels_like': u'1°C',
            'visibility': u'6.03 mi',
            'sunrise': '7:28am',
            'sunset': '5:01pm',
        })

        self.assertDictEqual(resp[1], {
            'url': 'https://weather.com/weather/10day/l/123,456?par=ptest',
            'art': 'foo/weatherchannel/suru/weather-clouds.svg',
            'art_wlogo': 'foo/weatherchannel/suru/weather-clouds-wlogo.svg',
            'day': u'Tomorrow',
            'comment': (u'Morning clouds followed by afternoon sun.'
                        u' Highs 7 to 9\xbaC and lows 3 to 5\xbaC.'),
            'date': u'09 Feb',
            'max_temp': u'8°C',
            'min_temp': u'4°C',
            'conditions': u'Morning clouds followed by afternoon sun',
            'pop': u'20%',
            'wind_sentence': u'Winds W at 10 to 20 mph.',
            'uv': '1',
            'current_temp': None,
            'humidity': '69%',
            'location': u'SE1, England, GB',
            'dew': None,
            'feels_like': u'2°C',
            'visibility': None,
            'sunrise': '7:26am',
            'sunset': '5:03pm',
        })

    def test_forecast_parsing_locale(self):
        twci = weatherchannel.TWCInterface('urlread', 'foo', FAKE_AUTH, 'test_locale', 'desktop')
        forecast = json.loads(get_fixture('weatherchannel-simple.json'))
        with patch.object(weatherchannel, 'translator', lambda t, locale: "translated--" + t):
            with patch.object(weatherchannel.TWCInterface, 'get_location') as mock:
                mock.return_value = {}
                resp = twci._parse_forecast(forecast, 123, 456)

        # check only one, as it proves that locale is correctly used
        result = resp[-1]
        self.assertEqual(result['date'], u"13 translated--Feb")

    def test_forecast_parsing_missing_visibility(self):
        twci = weatherchannel.TWCInterface('urlread', 'foo', FAKE_AUTH, 'es_AR', 'desktop')
        forecast = json.loads(get_fixture('weatherchannel-no-visibility.json'))
        with patch.object(weatherchannel.TWCInterface, 'get_location') as mock:
            mock.return_value = {}
            resp = twci._parse_forecast(forecast, 123, 456)

        # check we got them all, and that visibility is in None
        self.assertEqual(len(resp), 11)
        self.assertEqual(resp[0]['visibility'], None)

    def test_forecast_parsing_missing_observation(self):
        twci = weatherchannel.TWCInterface('urlread', 'foo', FAKE_AUTH, 'es_AR', 'desktop')
        forecast = json.loads(get_fixture('weatherchannel-no-stdobserv.json'))
        with patch.object(weatherchannel.TWCInterface, 'get_location') as mock:
            mock.return_value = {}
            resp = twci._parse_forecast(forecast, 123, 456)

        # check we got them all, and that visibility is in None
        self.assertEqual(len(resp), 11)
        self.assertEqual(resp[0]['visibility'], None)

    def test_forecast_parsing_missing_dailyinfo(self):
        twci = weatherchannel.TWCInterface('urlread', 'foo', FAKE_AUTH, 'es_AR', 'desktop')
        forecast = json.loads(get_fixture('weatherchannel-no-dailyinfo.json'))
        with patch.object(weatherchannel.TWCInterface, 'get_location') as mock:
            mock.return_value = {}
            resp = twci._parse_forecast(forecast, 123, 456)

        # It will be missing one forecast because a "day part" is deleted
        # In the older API that would have been generated with missing
        # data but with the new API that makes no sense
        self.assertEqual(len(resp), 10)
        self.assertEqual(resp[0]['max_temp'], u'8°C')
        self.assertEqual(resp[0]['min_temp'], u'4°C')

    def test__get_forecast_ok(self):
        mock = Mock()
        fake_data = json.dumps({
            'forecasts': [{}, {}],
            'observation': 'stuff',
        })
        mock.return_value = fake_data
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'test_locale', 'desktop')
        resp = twci._get_forecast(123, 456)
        mock.assert_called_with(
            'https://api.weather.com/v1/geocode/123/456/observations/current.json?'
            'language=test-locale&units=&apiKey=bar')
        self.assertEqual(resp, {
            'forecasts': [{}, {}],
            'observation': u'stuff',
            'units': 'm'
        })

    def test__get_forecast_empty(self):
        mock = Mock()
        fake_data = json.dumps({})
        mock.return_value = fake_data
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'test_locale', 'desktop')
        with self.assertRaises(BackendDataError) as exc_cm:
            twci._get_forecast(123, 456)
        mock.assert_called_with(
            'https://api.weather.com/v1/geocode/123/456/forecast/daily/5day.json?'
            'language=test-locale&units=&apiKey=bar')
        exc_msg = str(exc_cm.exception)
        self.assertIn('lat=123 long=456', exc_msg)
        self.assertIn('test_locale', exc_msg)

    def test__get_forecast_corrupted(self):
        mock = Mock()
        fake_data = "scrambled data"
        mock.return_value = fake_data
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'test_locale', 'desktop')
        with self.assertRaises(BackendDataError):
            twci._get_forecast(123, 456)
        mock.assert_called_with(
            'https://api.weather.com/v1/geocode/123/456/forecast/daily/5day.json?'
            'language=test-locale&units=&apiKey=bar')

    def test__search_locations(self):
        mock = Mock()
        fake_data = get_fixture('weatherchannel-address-lookup.json')
        mock.return_value = fake_data
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'en_US', 'desktop')
        resp = list(twci.search_locations(u"moño"))
        mock.assert_called_with('https://api.weather.com/v2/location?address'
                                '=mo%C3%B1o&language=en-US&format=json&apiKey=bar')
        self.assertEqual(resp, [
            {
                "name": u"Santa Fe",
                "state": u"NM",
                "country": u"United States",
                "longitude": -105.93795776367188,
                "latitude": 35.691680908203125,
                "country_code": u"US",
            },
            {
                "name": u"Santa Fe",
                "state": u"Santa Fe",
                "country": u"Argentina",
                "longitude": -60.720699310302734,
                "latitude": -31.59589958190918,
                "country_code": u"AR",
            },
            {
                "country_code": u"US",
                "name": u"Santa Fe",
                "state": u"TX",
                "country": u"United States",
                "longitude": -95.08184814453125,
                "latitude": 29.370159149169922,
            }
        ])

    def test__search_locations_empty_json(self):
        mock = Mock()
        mock.return_value = '{"results":[]}'
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'en_US', 'desktop')
        resp = list(twci.search_locations("foo"))
        self.assertEqual(resp, [])

    def test__search_locations_no_results(self):
        mock = Mock()
        mock.return_value = '{}'
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'en_US', 'desktop')
        resp = list(twci.search_locations("foo"))
        self.assertEqual(resp, [])

    def test__get_location_key_simple(self):
        mock = Mock()
        mock.return_value = get_fixture('weatherchannel-address.json')
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'en_GB', 'desktop')
        twci.get_location(1.0, 1.0)
        loc_uri = ('https://api.weather.com/v2/location?geocode=1.00,1.00'
                   '&language=en-GB&format=json&apiKey=bar')
        mock.assert_called_with(loc_uri)

    def test__get_location_key_empty_response(self):
        mock = Mock()
        mock.return_value = ""
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'en_GB', 'desktop')
        self.assertRaises(BackendDataError, twci.get_location, 1.0, 1.0)

    def test__get_location_key_empty_json(self):
        mock = Mock()
        mock.return_value = '{}'
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'en_GB', 'desktop')
        resp = list(twci.search_locations("foo"))
        self.assertEqual(resp, [])

    def test__get_location_key_no_data(self):
        mock = Mock()
        resp_data = {"class": "error", "type": 1000, "cat": "data",
                     "text": "The requested data is not available"}
        mock.return_value = json.dumps(resp_data)
        twci = weatherchannel.TWCInterface(mock, 'foo', FAKE_AUTH, 'en_GB', 'desktop')
        resp = list(twci.search_locations("foo"))
        self.assertEqual(resp, [])

    def test_forecast_parsing_iconset(self):
        twci = weatherchannel.TWCInterface('urlread', 'base', FAKE_AUTH, 'en_US', 'desktop',
                                           iconset='testicons')
        forecast = json.loads(get_fixture('weatherchannel-simple.json'))
        with patch.object(weatherchannel.TWCInterface, 'get_location') as mock:
            mock.return_value = {}
            resp = twci._parse_forecast(forecast, 123, 456)
        self.assertEqual(resp[0]['art'], 'base/weatherchannel/testicons/weather-showers.svg')

    def test_forecast_parsing_missing_sun_rise_set(self):
        twci = weatherchannel.TWCInterface('urlread', 'foo', FAKE_AUTH, 'es_AR', 'desktop')
        forecast = json.loads(get_fixture('weatherchannel-no-sunriseset.json'))
        with patch.object(weatherchannel.TWCInterface, 'get_location') as mock:
            mock.return_value = {}
            resp = twci._parse_forecast(forecast, 123, 456)

        # check we nulled those fields
        self.assertEqual(len(resp), 11)
        self.assertEqual(resp[0]['sunrise'], None)
        self.assertEqual(resp[0]['sunset'], None)

    def test_forecast_parsing_missing_humidity(self):
        twci = weatherchannel.TWCInterface('urlread', 'foo', FAKE_AUTH, 'en_US', 'desktop')
        forecast = json.loads(get_fixture('weatherchannel-no-humid.json'))
        with patch.object(weatherchannel.TWCInterface, 'get_location') as mock:
            mock.return_value = {}
            resp = twci._parse_forecast(forecast, 123, 456)

        # check we nulled those fields
        self.assertEqual(len(resp), 11)
        self.assertEqual(resp[0]['humidity'], None)
