# -*- coding: utf8 -*-

# Copyright 2014-2015 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

import feedparser
import json

from unittest import TestCase

from mock import patch, call

from src.scopes import wikipedia
from src.scopes.tests import get_fixture
from src.utils import localized_category

CONFIG = {'base_image_url': '', 'user_agent': 'test_UA'}


class SearchTestCase(TestCase):
    """The tests for the search part."""
    maxDiff = None

    def test_simple_english(self):
        app = wikipedia.App(CONFIG)
        with patch.object(app, '_get_search') as mock:
            mock.return_value = get_fixture('wikipedia-simple.xml')
            response = list(app.search(query=u'q', locale='en_US', limit=10))

        # got the info as it should
        mock.assert_called_with(u'q', 10, 'en')

        # category
        should_cat = {'category': wikipedia.CATEGORY_DEFAULT}
        self.assertEqual(response[0], should_cat)

        # the result
        should_res = {'result': {
            'cat_id': wikipedia.CATEGORY_DEFAULT['id'],
            'uri': u'http://en.wikipedia.org/wiki/Ubuntu_(operating_system)',
            'dnd_uri':
            u'http://en.wikipedia.org/wiki/Ubuntu_(operating_system)',
            'title': u'Ubuntu (operating system)',
            'art': '/icons/wikipedia.png',
        }}
        self.assertEqual(response[1], should_res)

    def test_simple_otherlang(self):
        app = wikipedia.App(CONFIG)
        with patch.object(app, '_get_search') as mock:
            mock.return_value = get_fixture('wikipedia-otherlang.xml')
            response = list(app.search(query=u'q', locale='fr_FR', limit=10))

        # got the info as it should
        mock.assert_called_with(u'q', 10, 'fr')

        # category
        should_cat = {
            'category': localized_category(wikipedia.CATEGORY_DEFAULT, 'fr')
        }
        self.assertEqual(response[0], should_cat)

        # the result
        uri = u'http://fr.wikipedia.org/wiki/Ubuntu'
        should_res = {'result': {
            'cat_id': wikipedia.CATEGORY_DEFAULT['id'],
            'uri': uri,
            'dnd_uri': uri,
            'title': u'Ubuntu',
            'art': (
                'http://upload.wikimedia.org/wikipedia/commons/thumb/'
                '8/81/Ubuntu_12.04_LTS_%28Fran%C3%A7ais%29.png/'
                '96px-Ubuntu_12.04_LTS_%28Fran%C3%A7ais%29.png'),
        }}
        self.assertEqual(response[1], should_res)

    def test_surfacing_featured(self):
        app = wikipedia.App(CONFIG)
        assert 'fr' in wikipedia.CURATED_SURFACING
        with patch.object(app, '_get_featured') as mock:
            mock.return_value = feedparser.parse(
                get_fixture('wikipedia-featured.rss'))
            response = list(app.search(query='', locale='fr_FR', limit=10))

        # got the info as it should
        mock.assert_called_with(10, 'fr')

        # category
        should_cat = {
            'category': localized_category(wikipedia.CATEGORY_FEATURED, 'fr')
        }
        self.assertEqual(response[0], should_cat)

        # the result
        should_res = {'result': {
            'cat_id': wikipedia.CATEGORY_FEATURED['id'],
            'uri': 'http://fr.wikipedia.org/wiki/Jean_B%C3%A9liveau',
            'dnd_uri': 'http://fr.wikipedia.org/wiki/Jean_B%C3%A9liveau',
            'title':  u'Jean Arthur Béliveau',
            'art': 'http://upload.wikimedia.org/wikipedia/commons/thumb/1/11/'
                   'David_Jacobson_et_Jean_B%C3%A9liveau.jpg/'
                   '96px-David_Jacobson_et_Jean_B%C3%A9liveau.jpg',
        }}
        self.assertEqual(response[1], should_res)

    def test_surfacing_recentchanges(self):
        app = wikipedia.App(CONFIG)
        assert 'es' not in wikipedia.CURATED_SURFACING
        fake_image_url = lambda name, lang: 'http://' + name
        with patch.object(app, '_get_recentchanges') as mock:
            with patch.object(app, '_get_image_from_page', fake_image_url):
                data = get_fixture('wikipedia-recentchanges.rss')
                mock.return_value = feedparser.parse(data)
                response = list(app.search(query='', locale='es_ES', limit=10))

        # got the info as it should
        mock.assert_called_with(10, 'es')

        # category
        cat = localized_category(wikipedia.CATEGORY_FEATURED, 'es')
        self.assertEqual(response[0], dict(category=cat))

        # the results
        self.assertEqual(len(response), 11)  # 1 cat + 10 results
        item = response[1]  # a simple sample
        self.assertEqual(item, dict(result={
            'cat_id': wikipedia.CATEGORY_FEATURED['id'],
            'uri': 'http://es.wikipedia.org/wiki/Optimus%20Prime',
            'dnd_uri': 'http://es.wikipedia.org/wiki/Optimus%20Prime',
            'title':  u'Optimus Prime',
            'art': 'http://Optimus%20Prime',
        }))

        item = response[6]  # other sample, with non-ascii letters
        self.assertEqual(item, dict(result={
            'cat_id': wikipedia.CATEGORY_FEATURED['id'],
            'uri': 'http://es.wikipedia.org/wiki/Sm%C3%B6oy',
            'dnd_uri': 'http://es.wikipedia.org/wiki/Sm%C3%B6oy',
            'title':  u'Smöoy',
            'art': 'http://Sm%C3%B6oy',
        }))

    def test_get_image_frompage_ok(self):
        app = wikipedia.App(CONFIG)
        fake_extract_data = get_fixture('wikipedia-extract_yes.json')
        fake_imageinfo_data = get_fixture('wikipedia-imageinfo.json')
        with patch.object(app, 'urlread') as mock:
            mock.side_effect = [fake_extract_data, fake_imageinfo_data]
            image_url = app._get_image_from_page('pagetitle', 'en')
        call1 = wikipedia.EXTRACT_URL % dict(language='en', title='pagetitle')
        image_name = 'Optimus_Pattaya.jpg'  # from the fixture
        call2 = wikipedia.IMAGEINFO_URL % dict(
            language='en', image=image_name,
            width=wikipedia.IMAGE_DISPLAY_SIZE)
        self.assertEqual(mock.mock_calls, [call(call1), call(call2)])
        self.assertEqual(image_url, (
            'http://upload.wikimedia.org/wikipedia/commons/f/fd/'
            'Madrid_Passeo_de_Castellana-BBVA.jpg'
        ))

    def test_get_image_frompage_notfound(self):
        app = wikipedia.App(CONFIG)
        fake_data = get_fixture('wikipedia-extract_no.json')
        with patch.object(app, 'urlread') as mock:
            mock.return_value = fake_data
            image_url = app._get_image_from_page('pagetitle', 'en')
        extract_url = wikipedia.EXTRACT_URL % dict(language='en',
                                                   title='pagetitle')
        mock.assert_called_with(extract_url)
        self.assertEqual(image_url, None)

    def test_get_image_frompage_unicode_imagename(self):
        app = wikipedia.App(CONFIG)
        fake_extract_data = get_fixture('wikipedia-extract_unicode.json')
        fake_imageinfo_data = get_fixture('wikipedia-imageinfo.json')
        with patch.object(app, 'urlread') as mock:
            mock.side_effect = [fake_extract_data, fake_imageinfo_data]
            app._get_image_from_page('pagetitle', 'en')
        call1 = wikipedia.EXTRACT_URL % dict(language='en', title='pagetitle')
        image_name = 'Optimus_Primo%C3%B1o.jpg'  # from the fixture, fixed
        call2 = wikipedia.IMAGEINFO_URL % dict(
            language='en', image=image_name,
            width=wikipedia.IMAGE_DISPLAY_SIZE)
        self.assertEqual(mock.mock_calls, [call(call1), call(call2)])

    def test_get_image_url_ok(self):
        app = wikipedia.App(CONFIG)
        with patch.object(app, 'urlread') as mock:
            mock.return_value = get_fixture('wikipedia-imageinfo.json')
            resp = app._get_image_url('en', 'imagename', 999)
        should_url = wikipedia.IMAGEINFO_URL % dict(language='en', image='imagename', width=999)
        mock.assert_called_with(should_url)
        self.assertEqual(resp, (
            'http://upload.wikimedia.org/wikipedia/commons/f/fd/'
            'Madrid_Passeo_de_Castellana-BBVA.jpg'
        ))

    def test_get_image_url_nothing(self):
        app = wikipedia.App(CONFIG)
        with patch.object(app, 'urlread') as mock:
            mock.return_value = get_fixture('wikipedia-imageinfo-bad.json')
            resp = app._get_image_url('en', 'imagename', 999)
        self.assertEqual(resp, None)


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_ok(self):
        # fake a previous result
        small_art = (
            "http://upload.wikimedia.org/wikipedia/commons/thumb/"
            "a/af/Xinhua_Gate.jpg/50px-Xinhua_Gate.jpg")
        bigger_art = "http://upload.wikimedia.org/wikipedia/commons/a/af/Xinhua_Gate.jpg"
        result = {
            'title': u'test title moño',
            'art': small_art,
            'uri': 'test uri',
        }
        app = wikipedia.App(CONFIG)
        with patch.object(app, '_get_preview') as mock_preview:
            with patch.object(app, '_get_image_url') as mock_url_image:
                mock_url_image.return_value = bigger_art
                mock_preview.return_value = get_fixture('wikipedia-extraction.json')
                response = list(app.preview(result=result, locale='en_US'))

        # got the info as it should
        mock_preview.assert_called_with(('test%20title%20mo%C3%B1o', 'test uri'), 200, 'en')
        mock_url_image.assert_called_with('en', "Xinhua_Gate.jpg", wikipedia.IMAGE_PREVIEW_SIZE)

        # check the 'image'
        should = dict(widget={
            'id': 'art',
            'type': 'image',
            'source': bigger_art,
        })
        self.assertEqual(response[0], should)

        # check the 'header'
        should = dict(widget={
            'id': 'header1',
            'type': 'header',
            'uri': 'test uri',
            'title': u'test title moño',
            'mascot': None,
        })
        self.assertEqual(response[1], should)

        # check the 'text'
        extract = "<p><b>Argentina</b></p>..."  # from fixture file
        should = dict(widget={
            'id': 'text1',
            'type': 'text',
            'uri': 'test uri',
            'title': None,
            'text': extract,
        })
        self.assertEqual(response[2], should)

        # check the 'actions'
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'action1',
                         'uri': 'test uri',
                         'label': 'View on Wikipedia'}],
        })
        self.assertEqual(response[3], should)

    def test_preview_image_missing(self):
        # fake a previous result
        result = {
            'title': u'test title moño',
            'art': "http://server/static/icons/wikipedia.png",
            'uri': 'test uri',
        }
        app = wikipedia.App(CONFIG)
        with patch.object(app, '_get_preview') as mock:
            with patch.object(app, '_get_image_url') as mock_url_image:
                mock_url_image.return_value = None
                mock.return_value = get_fixture('wikipedia-extraction.json')
                response = list(app.preview(result=result, locale='en_US'))

        # check we don't have an 'image' widget
        for widget in response:
            self.assertNotEqual(widget['widget']['type'], 'image')

    def test_extract_two_results(self):
        # fake a previous result
        result = {
            'title': u'test title',
            'art': "http://server/static/icons/wikipedia.png",
            'uri': 'test_uri',
        }
        preview_texts = json.dumps(dict(query=dict(pages={
            0: {},
            1: {'extract': 'text sample'},
        })))
        app = wikipedia.App(CONFIG)
        with patch.object(app, '_get_preview') as mock:
            with patch.object(app, '_get_image_url') as mock_url_image:
                mock_url_image.return_value = None
                mock.return_value = preview_texts
                response = list(app.preview(result=result, locale='en_US'))

        # got the info as it should
        mock.assert_called_with(('test%20title', 'test_uri'), 200, 'en')

        # check the 'text' widget it produced
        should = dict(widget={
            'id': 'text1',
            'type': 'text',
            'uri': 'test_uri',
            'title': None,
            'text': 'text sample',
        })
        self.assertEqual(response[1], should)

    def test_extract_no_results(self):
        # fake a previous result
        result = {
            'title': u'test title',
            'art': "http://server/static/icons/wikipedia.png",
            'uri': 'test_uri',
        }
        preview_texts = json.dumps(dict(query=dict(pages={})))
        app = wikipedia.App(CONFIG)
        with patch.object(app, '_get_preview') as mock:
            with patch.object(app, '_get_image_url') as mock_url_image:
                mock_url_image.return_value = None
                mock.return_value = preview_texts
                response = list(app.preview(result=result, locale='en_US'))

        # got the info as it should
        mock.assert_called_with(('test%20title', 'test_uri'), 200, 'en')

        # check there is no 'text' widget produced
        self.assertEqual(0, len([x for x in response if x['widget']['type'] == 'text']))


class HelpersTestCase(TestCase):
    """Tests for some helper functions."""

    def test_search_retrieval(self):
        app = wikipedia.App(CONFIG)
        fake_data = 'raw test result'
        with patch.object(app, 'urlread') as mock_urlread:
            mock_urlread.return_value = fake_data
            resp = app._get_search(u'test ñ', 10, 'en')

        self.assertEqual(resp, fake_data)

        should_url = wikipedia.SEARCH_URL % dict(
            language='en', limit=10, query='test%20%C3%B1')
        should_headers = {'User-Agent': 'test_UA', 'Accept': '*/*'}
        mock_urlread.assert_called_with(should_url, headers=should_headers)

    def test_getpreview_simple(self):
        app = wikipedia.App(CONFIG)
        with patch.object(app, 'urlread') as mock_open:
            app._get_preview(('title',), 200, 'en')

        # got the info as it should
        url = ('http://en.wikipedia.org/w/api.php?action=query&prop=extracts'
               '&exchars=200&titles=title&format=json')
        mock_open.assert_called_with(url, headers=app.headers)

    def test_getpreview_several(self):
        app = wikipedia.App(CONFIG)
        with patch.object(app, 'urlread') as mock_open:
            app._get_preview(('tit1', 'tit2'), 200, 'en')

        # got the info as it should
        url = ('http://en.wikipedia.org/w/api.php?action=query&prop=extracts'
               '&exchars=200&titles=tit1|tit2&format=json')
        mock_open.assert_called_with(url, headers=app.headers)
