# -*- coding: utf8 -*-

# Copyright 2014-2015 Canonical
# All Rights Reserved

"""Tests for the Simple App base stuff."""

from unittest import TestCase

from mock import patch

from src.scopes import yahoofinance
from src.scopes.tests import get_fixture

CONFIG = {'secrets': {}}


class SearchTestCase(TestCase):
    """Tests for the search function."""
    maxDiff = None

    def test_surfacing(self):
        app = yahoofinance.App(CONFIG)
        with patch.object(app, '_get_trending_symbols') as trending:
            with patch.object(app, '_get_quotes') as quotes:
                with patch.object(app, '_get_news') as news:
                    trending.return_value = "UBNT"
                    quotes.return_value = [{"symbol": "UBNT",
                                            "change": "+1 +14.10%",
                                            "name": "Ubuntu Corp.",
                                            "value": "1337",
                                            "summary": ""}]
                    news.return_value = [{"uri": "news uri",
                                          "title": "news title",
                                          "date": "news date",
                                          "summary": "news summary",
                                          "image": "art url"}]
                    response = list(app.search(query='', platform='desktop',
                                               limit=app.default_limit,
                                               locale='C'))

        # Ensure we ask for the right number of trends
        trending.assert_called_with(app.default_limit)
        # Ensure the trends are used for the quotes query
        quotes.assert_called_with("http://finance.yahoo.com/d/quotes.cs"
                                  "v?s=UBNT&f=snl1d1t1vc")
        # Ensure we look for global headlines
        news.assert_called_with('C')

        # categories
        should_cat = {'category': yahoofinance.HEADLINES_CATEGORY}
        self.assertEqual(response[0], should_cat)
        should_cat = {'category': yahoofinance.TRENDING_CATEGORY}
        self.assertEqual(response[2], should_cat)

        # Headline result
        should_res = {'result': {
            'cat_id': 'headlines',
            'dnd_uri': 'news uri',
            'uri': 'news uri',
            'title': 'news title',
            'summary': 'news summary',
            'date': 'news date',
            'art': 'art url',
        }}
        self.assertEqual(response[1], should_res)

        # Trending result
        should_res = {'result': {
            'cat_id': 'trending',
            'dnd_uri': 'http://finance.yahoo.com/q?s=UBNT',
            'uri': 'http://finance.yahoo.com/q?s=UBNT',
            'title': 'Ubuntu Corp.',
            'symbol': 'UBNT',
            'change_p': '+14.10%',
            'change_v': '+1',
            'value': '1337',
            'graph': 'http://chart.finance.yahoo.com/z?s=UBNT&z=s&t=7d',
            'attributes': [{'value': '1337'},
                           {'value': '+1'},
                           {'value': '+14.10%'}]
        }}
        self.assertEqual(response[3], should_res)

    def test_search(self):
        app = yahoofinance.App(CONFIG)
        with patch.object(app, '_get_quotes') as quotes:
            with patch.object(app, '_get_symbol_from_name') as name:
                name.return_value = "UBNT", {"UBNT": {
                                             "exchange": "TUX",
                                             "name": "Ubuntu Corp.",
                                             "value": "1337", }
                                             }
                quotes.return_value = [{"symbol": "UBNT",
                                        "change": "+1 +14.10%",
                                        "name": "Ubuntu Corp.",
                                        "value": "1337",
                                        "summary": ""}]
                response = list(app.search(query='ubu', platform='desktop',
                                           limit=app.default_limit,
                                           locale='C'))

        # Ensure the quotes are used for the query
        quotes.assert_called_with("http://finance.yahoo.com/d/quotes.csv"
                                  "?s=UBNT&f=snl1d1t1vc")

        # Custom quotes categories
        QUOTES_CATEGORY = {
            'id': 'TUX',
            'title': 'TUX',
            'render_template': yahoofinance.RENDERER_QUOTES,
        }
        should_cat = {'category': QUOTES_CATEGORY}
        self.assertEqual(response[0], should_cat)

        # Quotes result
        should_res = {'result': {
            'cat_id': 'TUX',
            'dnd_uri': 'http://finance.yahoo.com/q?s=UBNT',
            'uri': 'http://finance.yahoo.com/q?s=UBNT',
            'title': 'Ubuntu Corp.',
            'symbol': 'UBNT',
            'change_p': '+14.10%',
            'change_v': '+1',
            'value': '1337',
            'graph': 'http://chart.finance.yahoo.com/z?s=UBNT&z=s&t=7d',
            'attributes': [{'value': '1337'},
                           {'value': '+1'},
                           {'value': '+14.10%'}]
        }}
        self.assertEqual(response[2], should_res)


class NewsGettingTestCase(TestCase):
    """Tests for the retrieval of news function."""
    maxDiff = None

    def test_selection_standard(self):
        assert 'pa' in yahoofinance.NEWS_URL_STANDARD
        resp = yahoofinance._select_news_url('es_PA')
        self.assertEqual(resp, 'http://pa.yahoo.com/finance/news/rss/')

    def test_selection_default(self):
        test_locale = "weird"
        assert test_locale not in yahoofinance.NEWS_URL_STANDARD
        assert test_locale not in yahoofinance.NEWS_URL_CUSTOM
        resp = yahoofinance._select_news_url(test_locale)
        self.assertEqual(resp, 'http://yahoo.com/finance/news/rss/')

    def test_selection_custom(self):
        assert 'za' not in yahoofinance.NEWS_URL_STANDARD
        should_be = yahoofinance.NEWS_URL_CUSTOM['za']
        resp = yahoofinance._select_news_url("xx_ZA")
        self.assertEqual(resp, should_be)

    def test_selection_spanish(self):
        test_locale = "es_WEIRD"
        assert test_locale not in yahoofinance.NEWS_URL_STANDARD
        assert test_locale not in yahoofinance.NEWS_URL_CUSTOM
        resp = yahoofinance._select_news_url(test_locale)
        self.assertEqual(resp, 'http://es-us.finanzas.yahoo.com/noticias/rss/')

    def test_news_real_sample(self):
        app = yahoofinance.App(CONFIG)
        with patch.object(app, 'urlread') as mock:
            mock.return_value = get_fixture('yahoofinance-news-ar.rss')
            resp = app._get_news('es_AR')

        # assert quantity and the first one in detail
        self.assertEqual(len(resp), 20)
        self.assertDictEqual(resp[0], {
            'date': u'Tue, 23 Sep 2014 15:18:51 -0300',
            'summary': u"La huelga de pilotos de Air France contra la "
                       u"expansión de la filial de bajo coste Transavia "
                       u"continuaba este martes por noveno día consecutivo, "
                       u"con las negociaciones bloqueadas y los huelguistas "
                       u"manifestándose ...",
            'title': u"Las negociaciones siguen bloqueadas en el noveno día "
                     u"de huelga de Air France",
            'uri': u"https://ar.finanzas.yahoo.com/noticias/negociaciones-"
                   u"siguen-bloqueadas-noveno-d%C3%ADa-181618747.html",
            'image': "https://s1.yimg.com/bt/api/res/1.2/dPP8LvKT9929Vb8d5gl4O"
                     "w--/YXBwaWQ9eW5ld3M7Zmk9ZmlsbDtoPTg2O3E9Njk7dz0xMzA-/htt"
                     "p://globalfinance.zenfs.com/images/AFP-LATAM_1/6a618e0b4"
                     "a06fa505080c287eddd371b4ca854e8_original.jpg",
        })

        # also check the second's summary, that has quotes
        self.assertEqual(resp[1]['summary'], (
            u'La Agencia de Recaudación de la provincia de Buenos Aires '
            u'(ARBA) informó hoy que dectectaron, por medio de un "drone '
            u'fiscalizador", que casi nueve de cada diez construcciones de un '
            u'predio cerrado ubicado en la localidad de Quilmes "no estaba '
            u'declarado ante el Fisco". Las viviendas, que conforman el 87%, '
            u'"tributaban como baldío".'
        ))

    def test_getting_empty(self):
        app = yahoofinance.App(CONFIG)
        with patch.object(app, '_read_feed') as mock:
            mock.return_value = ""
            resp = app._get_news('es_AR')
        self.assertEqual(resp, [])

    def test_getting_news_bad_backend_response(self):
        app = yahoofinance.App(CONFIG)
        with patch.object(app, 'urlread') as mock:
            mock.side_effect = ValueError
            resp = app._get_news('es_AR')
        self.assertEqual(resp, [])

    def test_getting_title_missing(self):
        fake_data = dict(entries=[{
            'link': 'test link',
            'published': 'test date',
            'media_content': [{'url': 'test image'}],
            'summary': 'test summary',
        }])
        app = yahoofinance.App(CONFIG)
        with patch.object(app, '_read_feed') as mock:
            mock.return_value = fake_data
            resp = app._get_news('es_AR')
        self.assertEqual(resp, [])

    def test_getting_title_costly(self):
        fake_data = dict(entries=[{
            'title': '[$$] Know how to be rich',
            'link': 'test link',
            'published': 'test date',
            'media_content': [{'url': 'test image'}],
            'summary': 'test summary',
        }])
        app = yahoofinance.App(CONFIG)
        with patch.object(app, '_read_feed') as mock:
            mock.return_value = fake_data
            resp = app._get_news('es_AR')
        self.assertEqual(resp, [])

    def test_getting_no_link(self):
        fake_data = dict(entries=[{
            'title': 'test title',
            'published': 'test date',
            'media_content': [{'url': 'test image'}],
            'summary': 'test summary',
        }])
        app = yahoofinance.App(CONFIG)
        with patch.object(app, '_read_feed') as mock:
            mock.return_value = fake_data
            resp = app._get_news('es_AR')
        self.assertEqual(resp, [])

    def test_getting_no_published(self):
        fake_data = dict(entries=[{
            'title': 'test title',
            'link': 'test link',
            'updated': 'test date',
            'media_content': [{'url': 'test image'}],
            'summary': 'test summary',
        }])
        app = yahoofinance.App(CONFIG)
        with patch.object(app, '_read_feed') as mock:
            mock.return_value = fake_data
            resp = app._get_news('es_AR')
        self.assertEqual(resp[0], {
            'title': 'test title',
            'uri': 'test link',
            'date': 'test date',
            'image': 'test image',
            'summary': 'test summary',
        })

    def test_getting_no_mediacontent(self):
        fake_data = dict(entries=[{
            'title': 'test title',
            'link': 'test link',
            'published': 'test date',
            'summary': 'test summary',
        }])
        app = yahoofinance.App(CONFIG)
        with patch.object(app, '_read_feed') as mock:
            mock.return_value = fake_data
            resp = app._get_news('es_AR')
        self.assertEqual(resp[0], {
            'title': 'test title',
            'uri': 'test link',
            'date': 'test date',
            'image': None,
            'summary': 'test summary',
        })


class PreviewTestCase(TestCase):
    """Tests for the preview function."""

    def test_quote_preview(self):
        # fake a previous quote result
        result = {
            'cat_id': 'TUX',
            'dnd_uri': 'http://finance.yahoo.com/q?s=UBNT',
            'uri': 'http://finance.yahoo.com/q?s=UBNT',
            'title': 'Ubuntu Corp.',
            'symbol': 'UBNT',
            'change_p': '+14.10%',
            'change_v': '+1',
            'value': '1337',
            'graph': 'http://chart.finance.yahoo.com/z?s=UBNT&z=s&t=7d',
        }
        app = yahoofinance.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the graph
        should = dict(widget={
            'id': 'art',
            'type': 'image',
            'source': 'http://chart.finance.yahoo.com/z?s=UBNT&z=s&t=7d',
        })
        self.assertEqual(response[0], should)

        # check the header
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'subtitle': 'UBNT - TUX',
            'title': 'Ubuntu Corp.'
        })
        self.assertEqual(response[1], should)

        # check the text
        should = dict(widget={
            'id': 'text',
            'type': 'text',
            'text': 'Price: 1337\nChange: +1\n% Change: +14.10%',
        })
        self.assertEqual(response[2], should)

        # check the buttons
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'yahoo',
                         'label': 'Yahoo! Finance',
                         'uri': 'http://finance.yahoo.com/q?s=UBNT'}],
        })
        self.assertEqual(response[3], should)

    def test_news_preview(self):
        # fake a previous news result
        result = {
            'cat_id': 'headlines',
            'dnd_uri': 'news uri',
            'uri': 'news uri',
            'title': 'news title',
            'summary': 'news summary',
            'date': 'news date',
        }
        app = yahoofinance.App(CONFIG)
        response = list(app.preview(result=result, locale='C'))

        # check the header
        should = dict(widget={
            'id': 'header',
            'type': 'header',
            'subtitle': 'news date',
            'title': 'news title'
        })
        self.assertEqual(response[0], should)

        # check the buttons
        should = dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'id': 'yahoo',
                         'label': 'Yahoo! Finance',
                         'uri': 'news uri'}],
        })
        self.assertEqual(response[1], should)

        # check the summary
        should = dict(widget={
            'id': 'text',
            'type': 'text',
            'text': 'news summary',
        })
        self.assertEqual(response[2], should)

    def test_get_quotes_ok(self):
        app = yahoofinance.App(CONFIG)
        fake_data = get_fixture('yahoofinance-quotes.csv')
        with patch.object(app, 'urlread') as mock:
            mock.return_value = fake_data
            response = app._get_quotes("url")
        mock.assert_called_with("url")
        self.assertEqual(response[0], {
            'symbol': "TWX",
            'name': "Time Warner Inc. ",
            'value': "71.01",
            'date': "7/15/2014",
            'time': "4:00pm",
            'volume': "6674202",
            'change': "-0.67 - -0.93%",
        })
        self.assertEqual(response[1], {
            'symbol': "JCP",
            'name': "J.C. Penney Compa",
            'value': "8.55",
            'date': "7/15/2014",
            'time': "4:01pm",
            'volume': "9095830",
            'change': "-0.06 - -0.70%",
        })
        self.assertEqual(response[2], {
            'symbol': "ABT",
            'name': "Abbott Laboratori",
            'value': "41.17",
            'date': "7/15/2014",
            'time': "4:03pm",
            'volume': "10958349",
            'change': "-0.65 - -1.55%",
        })

    def test_get_quotes_bad(self):
        app = yahoofinance.App(CONFIG)
        with patch.object(app, 'urlread') as mock:
            mock.return_value = "broken data"
            response = app._get_quotes("url")
        self.assertEqual(len(response), 0)


class HelpersTestCase(TestCase):
    """Tests for some helpers."""
    maxDiff = None

    def test_get_symbol_from_name(self):
        app = yahoofinance.App(CONFIG)
        fake_data = get_fixture('yahoofinance-symbol.json')
        with patch.object(app, 'urlread') as mock:
            mock.return_value = fake_data
            resp = app._get_symbol_from_name('testquery')

        should_uri = dict(query='testquery')
        mock.assert_called_with(yahoofinance.SYMBOL_LOOKUP_URL % should_uri)
        self.assertEqual(resp, (u'IBM IBMI', {
            u'IBM': {'name': u'International Corp', 'exchange': u'NYSE'},
            u'IBMI': {'name': u'iShares iBonds', 'exchange': u'NYSEArca'}
        }))

    def test_get_symbol_no_result(self):
        app = yahoofinance.App(CONFIG)
        fake_data = get_fixture('yahoofinance-no-result.json')
        with patch.object(app, 'urlread') as mock:
            mock.return_value = fake_data
            resp = app._get_symbol_from_name('testquery')

        should_uri = dict(query='testquery')
        mock.assert_called_with(yahoofinance.SYMBOL_LOOKUP_URL % should_uri)
        self.assertEqual(resp, ('', {}))

    def test_trending_symbols(self):
        app = yahoofinance.App(CONFIG)
        fake_data = get_fixture('yahoofinance-trending.json')
        with patch.object(app, 'urlread') as mock:
            mock.return_value = fake_data
            resp = app._get_trending_symbols(10)

        mock.assert_called_with(yahoofinance.STOCKTWITS_URI + "?limit=10")
        self.assertEqual(resp, 'GOOG')
