# -*- coding: UTF-8 -*-

# Copyright 2014-2015 Canonical
# All Rights Reserved

"""The Weather channel scope."""

import json
import logging
import urllib

from datetime import timedelta

from dateutil.parser import parse as parse_date

from ..scopes import SimpleApp, BadRequestError, BackendDataError
from ..translator import translator
from ..utils import localized_category

# limit of the quantity of forecast days, business reasons
MAX_FORECAST_DAYS = 6

# TWC will (sometimes) return some strings in English even if the locale is
# other; so let's force the translation here
translator("Today")
translator("Tonight")
translator("Tomorrow")
translator("Tomorrow night")
translator("Monday")
translator("Tuesday")
translator("Wednesday")
translator("Thursday")
translator("Friday")
translator("Saturday")
translator("Sunday")

# TWC supported languages
LANGUAGES = [
    'ar-AE', 'bn-BD', 'bn-IN', 'ca-ES', 'cs-CZ', 'da-DK', 'de-DE', 'de-CH', 'el-GR', 'en-GB',
    'en-AU', 'en-IN', 'en-US', 'es-AR', 'es-ES', 'es-US', 'es-LA', 'es-MX', 'es-UN', 'es-ES',
    'fa-IR', 'fi-FI', 'fr-CA', 'fr-FR', 'fr-CH', 'he-IL', 'hi-IN', 'hr-HR', 'hu-HU', 'in-ID',
    'it-IT', 'it-CH', 'iw-IL', 'he-IL', 'ja-JP', 'kk-KZ', 'ko-KR', 'ms-MY', 'nl-NL', 'no-NO',
    'nn-NO', 'pl-PL', 'pt-BR', 'pt-PT', 'ro-RO', 'ru-RU', 'sk-SK', 'sv-SE', 'th-TH', 'tl-PH',
    'tr-TR', 'uk-UA', 'ur-PK', 'vi-VN', 'zh-CN', 'zh-HK', 'zh-TW', 'test-locale']
LANGUAGES += [l.split('-')[0] for l in LANGUAGES]
LANGUAGES = set(LANGUAGES)


# we need the abbreviated month name in all the languages
MONTHS = {
    1: translator('Jan'),
    2: translator('Feb'),
    3: translator('Mar'),
    4: translator('Apr'),
    5: translator('May'),
    6: translator('Jun'),
    7: translator('Jul'),
    8: translator('Aug'),
    9: translator('Sep'),
    10: translator('Oct'),
    11: translator('Nov'),
    12: translator('Dec'),
}


_icons_map = [
    ('weather-clear', [32, 36]),
    ('weather-clear-night', [31]),
    ('weather-clouds', [28, 30]),
    ('weather-clouds-night', [27, 29]),
    ('weather-few-clouds', [34]),
    ('weather-few-clouds-night', [33]),
    ('weather-fog', [19, 20, 21, 22, 23, 24]),
    ('weather-overcast', [26]),
    ('weather-showers', [6, 10, 11, 12, 18, 40]),
    ('weather-showers-scattered', [8, 9, 39, 45]),
    ('weather-snow', [5, 7, 13, 14, 15, 16, 41, 42, 43, 46]),
    ('weather-storm', [0, 1, 2, 3, 4, 17, 35, 37, 38, 47]),
]
WEATHER_ICONS_MAP = {}
for icon, wids in _icons_map:
    WEATHER_ICONS_MAP.update(dict.fromkeys(wids, icon))

URL_TEMPLATE = 'https://weather.com/{language}/weather/{hint}/l/{latitude},{longitude}?par={partner}'  # NOQA
ENGLISH_URL_TEMPLATE = 'https://weather.com/weather/{hint}/l/{latitude},{longitude}?par={partner}'

# the categories to use
CATEGORY_GRID_MAIN = {
    'id': 'cat-grid-main',
    'title': None,
    'render_template': json.dumps({
        "schema-version": 1,
        "template": {"category-layout": "grid",
                     "card-size": "medium",
                     "card-layout": "vertical"},
        "components": {"title": "title",
                       "subtitle": "subtitle",
                       "art": "art",
                       "attributes": {"field": "attributes",
                                      "max-count": 4}},
    }),
}
CATEGORY_GRID_REST = {
    'id': 'cat-grid-rest',
    'title': translator('Forecast'),
    'render_template': json.dumps({
        "schema-version": 1,
        "template": {"category-layout": "grid",
                     "card-size": "small",
                     "card-layout": "vertical"},
        "components": {"title": "title",
                       "subtitle": "subtitle",
                       "art": "art"},
    }),
}
CATEGORY_LIST = {
    'id': 'cat-list',
    'title': '',
    'render_template': json.dumps({
        "schema-version": 1,
        'template': {'category-layout': 'grid',
                     'card-size': 'large',
                     'card-layout': 'horizontal',
                     'collapsed-rows': 0},
        'components': {'title': 'title'},
    }),
}

logger = logging.getLogger("restscopes.twc")


class TWCInterface(object):
    """Interface with The Weather Channel.

    This class is not thread-safe; use one per served request.
    """

    _loc_get_uri = (
        'https://api.weather.com/v2/location?geocode=%(lat)s,%(lng)s&language=%(lang)s'
        '&format=json&apiKey=%(key)s'
    )
    _loc_search_uri = (
        'https://api.weather.com/v2/location?address=%(query)s&'
        'language=%(language)s&format=json&apiKey=%(key)s'
    )
    _forecast_uri = (
        'https://api.weather.com/v1/geocode/%(latitude)s/%(longitude)s/forecast/daily/'
        '5day.json?language=%(language)s&units=&apiKey=%(api_key)s'
    )
    _observations_uri = (
        'https://api.weather.com/v1/geocode/%(latitude)s/%(longitude)s/observations/'
        'current.json?language=%(language)s&units=&apiKey=%(api_key)s'
    )

    def __init__(self, urlread, base_image_url, auth, locale, formfactor, iconset=None):
        self.urlread = urlread
        self.base_image_url = base_image_url
        self.auth = auth
        self.locale = locale
        language = locale.replace('_', '-')
        if language in LANGUAGES:
            self.language = language
        elif language.split('-')[0] in LANGUAGES:
            self.language = language.split('-')[0]
        else:
            self.language = 'en-US'
        self.formfactor = formfactor
        self.iconset = 'suru' if iconset is None else iconset

    def _format_temp(self, temp, units):
        """Format a temperature using requested units."""
        temp_unit = {
            'm': u'°C',
            'e': u'°F',
            'h': u'°C',
        }
        if temp is None:  # Happens with new API sometimes
            return None
        return u'%s%s' % (str(temp), temp_unit[units])

    def _format_dist(self, dist, units):
        """Format a distance using requested units."""
        dist_unit = {
            'm': u'km',
            'e': u'mi',
            'h': u'mi',
        }
        return u'%s %s' % (str(dist), dist_unit[units])

    def _feels_like(self, hi, wc, units):
        """Calculate the 'feels_like' value.

        This is the suggested procedure from the TWC docs.
        """
        thresholds = {
            'm': (5, 21),
            'e': (40, 70),
            'h': (5, 21),
        }
        low, high = thresholds[units]
        if hi > high:
            return hi
        elif wc < low:
            return wc
        else:
            return None

    def _format_time(self, dt):
        """Format the datetime to a string according the locale."""
        if self.locale == 'en_US':
            if dt.hour < 12:
                return "{:d}:{:02d}am".format(dt.hour, dt.minute)
            else:
                return "{:d}:{:02d}pm".format(dt.hour - 12, dt.minute)
        else:
            return "{:d}:{:02d}".format(dt.hour, dt.minute)

    def _get_forecast(self, latitude, longitude):
        """Really get the forecast from TWC"""
        results = {}
        logger.debug("Get forecast: locale=%s latitude=%s longitude=%s)",
                     self.locale, latitude, longitude)
        uri = self._forecast_uri % dict(
            latitude=latitude, longitude=longitude, language=self.language,
            api_key=self.auth['api_key'])
        logger.debug("Requesting forecast: '%s'", uri)
        r = self.urlread(uri)
        try:
            data = json.loads(r.decode('utf-8'))
        except ValueError as exc:
            raise BackendDataError(exc, uri)
        if not data.get('forecasts'):
            msg = "Forecast not found for lat=%r long=%r locale=%r" % \
                (latitude, longitude, self.locale)
            raise BackendDataError(msg, uri)
        logger.debug('Got forecast for %s %s', latitude, longitude)
        results['units'] = data.get('metadata', {}).get('units', 'm')
        results['forecasts'] = data['forecasts']

        # The old API included observation data, now needs to be queried separately
        uri = self._observations_uri % dict(
            latitude=latitude, longitude=longitude, language=self.language,
            api_key=self.auth['api_key'])
        logger.debug("Requesting observation: '%s'", uri)
        r = self.urlread(uri)
        try:
            data = json.loads(r.decode('utf-8'))
        except ValueError as exc:
            raise BackendDataError(exc, uri)
        if not data.get('observation'):
            msg = "Observation not found for lat=%r long=%r locale=%r" % \
                (latitude, longitude, self.locale)
            raise BackendDataError(msg, uri)
        logger.debug('Got observation for %s %s', latitude, longitude)

        results['observation'] = data['observation']

        return results

    def get_localized_url(self, hint, latitude, longitude):
        """Build a localized URL with several parameters."""
        if self.language.startswith('en-') or self.language == 'en':
            # TWC URLs english are special
            return ENGLISH_URL_TEMPLATE.format(
                hint=hint,
                latitude=latitude,
                longitude=longitude,
                partner=self.auth['partner'])
        else:
            return URL_TEMPLATE.format(
                hint=hint,
                latitude=latitude,
                longitude=longitude,
                language=self.language,
                partner=self.auth['partner'])

    def _parse_forecast(self, data, latitude, longitude):
        """Get useful info from a forecast."""
        # prepare some info from location
        # Obtain nearest address from TWC
        _loc = self.get_location(latitude, longitude)
        location_info = dict(name=_loc.get('locality'),
                             state=_loc.get('admin_district'),
                             ccode=_loc.get('country_code'))
        units = data['units']
        units_name = {
            'm': 'metric',
            'e': 'imperial',
            'h': 'uk_hybrid',
        }
        # prepare the 'daily' information
        daily_forecast = {}
        for daily in data.get('forecasts', []):
            try:
                max_temp = self._format_temp(daily['max_temp'], units)
            except KeyError:
                max_temp = None
            try:
                min_temp = self._format_temp(daily['min_temp'], units)
            except KeyError:
                min_temp = None

            dt = parse_date(daily['fcst_valid_local'])
            dayparts = daily_forecast[dt.date()] = {}
            for part in ('day', 'night'):
                info = daily.get(part, {})
                phrase = info.get('phrase_22char')
                pop = info.get('pop')
                wind_sentence = info.get('wind_phrase')
                uv = str(info.get('uv_index'))
                humid = info.get('rh')
                if humid is not None:
                    humid = '%i%%' % humid
                dayparts[part] = {
                    'max_temp': max_temp,
                    'min_temp': min_temp,
                    'conditions': phrase,
                    'pop': pop,
                    'wind_sentence': wind_sentence,
                    'uv': uv,
                    'humidity': humid,
                }

        # prepare the info from standard observation, useful for 'now'
        try:
            _visib = self._format_dist(data['observation'][units_name[units]]['vis'], units)
        except KeyError:  # broken data, missing or broken observation
            _visib = None
        std_observ = dict(visibility=_visib)

        # get the info four sun rise and set
        sun_times = {}
        for item in data.get('forecasts', []):
            try:
                rise_raw = item['sunrise']
                set_raw = item['sunset']
            except KeyError:
                continue

            dt_rise = parse_date(rise_raw)
            dt_set = parse_date(set_raw)
            sun_times[dt_rise.date()] = {
                'rise': dt_rise,
                'set': dt_set,
            }

        results = []
        today = None

        for idx, forecast in enumerate(data.get('forecasts', [])):
            f_datetime = parse_date(forecast['fcst_valid_local'])
            f_date = f_datetime.date()

            if today is None:
                # first result is today
                today = f_datetime

            for dpart in ['day', 'night']:
                try:
                    # Consolidate all data for the day/part
                    daily_info = daily_forecast[f_date][dpart]
                    daily_info.update(forecast[dpart])
                except KeyError:
                    continue

                # No need to localize, TWC does it
                _localized_daypart = daily_info['daypart_name']
                # two days limit is 0-hour of past-tomorrow
                two_days_limit = f_date + timedelta(days=2)

                if f_datetime == today:
                    uri_date = "today"
                    day = _localized_daypart

                else:
                    uri_date = "10day"
                    if f_date < two_days_limit:
                        # today and tomorrow have special strings
                        day = _localized_daypart
                    else:
                        day = u"%s, %s" % (_localized_daypart, f_datetime.day)

                icode = daily_info['icon_code']
                if icode in WEATHER_ICONS_MAP:
                    base = "/".join((self.base_image_url, 'weatherchannel', self.iconset))
                    icon = base + '/' + WEATHER_ICONS_MAP[icode] + ".svg"
                    icon_wlogo = base + '/' + WEATHER_ICONS_MAP[icode] + "-wlogo.svg"
                else:
                    icon = "http://s.imwx.com/img/wxicon/130/%s.png" % (icode,)
                    icon_wlogo = None
                location = ', '.join(filter(None, (location_info['name'],
                                                   location_info['state'],
                                                   location_info['ccode'])))

                url = self.get_localized_url(uri_date, latitude, longitude)

                # improve conditions, in case got nothing from TWC
                conditions = daily_info['conditions']
                if not conditions:
                    conditions = forecast['narrative'].split(".")[0].strip()

                # on first forecast, can use some info from the std observation
                if idx == 0:
                    visibility = std_observ['visibility']
                    try:
                        current_temp = self._format_temp(
                            data['observation'][units_name[units]]['temp'], units)
                    except KeyError:  # Broken observation data
                        current_temp = None
                    try:
                        current_dew = self._format_temp(
                            data['observation'][units_name[units]]['dewpt'], units)
                    except KeyError:  # Broken observation data
                        current_dew = None
                else:
                    visibility = current_temp = current_dew = None

                # sun rising and seting
                try:
                    info = sun_times[f_date]
                except KeyError:
                    sunrise = sunset = None
                else:
                    sunrise = self._format_time(info['rise'])
                    sunset = self._format_time(info['set'])

                # the date, nicely
                month = MONTHS[f_date.month]
                nice_date = u"%02d" % f_date.day + u" " + translator(month, self.locale)

                feels_like = self._format_temp(
                    self._feels_like(daily_info['hi'], daily_info['wc'], units), units)

                results.append({
                    'url': url,
                    'art': icon,
                    'art_wlogo': icon_wlogo,
                    'day': day,
                    'comment': forecast['narrative'],
                    'date': nice_date,
                    'max_temp': daily_info['max_temp'],
                    'min_temp': daily_info['min_temp'],
                    'conditions': conditions,
                    'pop': u'%i%%' % daily_info['pop'],
                    'wind_sentence': daily_info['wind_sentence'],
                    'uv': daily_info['uv'],
                    'humidity': daily_info['humidity'],
                    'location': location,
                    'visibility': visibility,
                    'current_temp': current_temp,
                    'dew': current_dew,
                    'feels_like': feels_like,
                    'sunrise': sunrise,
                    'sunset': sunset,
                })
        return results

    def get_forecast(self, latitude, longitude):
        """Get the forecast from TWC"""
        data = self._get_forecast(latitude, longitude)
        return self._parse_forecast(data, latitude, longitude)

    def get_location(self, latitude, longitude):
        """Return a near address for current location parameters."""
        # no search string, use geolocation
        params = dict(lat="%.2f" % float(latitude), lng="%.2f" % float(longitude),
                      key=self.auth['api_key'], lang=self.language)
        loc_uri = self._loc_get_uri % params
        resp = self.urlread(loc_uri)

        try:
            loc_result = json.loads(resp.decode('utf-8'))
        except ValueError as exc:
            raise BackendDataError(exc, loc_uri)
        location = loc_result.get('addresses', [])
        if location:
            location = location[0]
        else:
            location = {}
        return location

    def search_locations(self, query):
        """Search for locations using user query."""
        q = urllib.quote(query.encode('utf8'))
        uri = self._loc_search_uri % dict(
            query=q, key=self.auth['api_key'], language=self.language)
        logger.debug("Requesting place key for city: %r", uri)
        raw = self.urlread(uri)
        data = json.loads(raw.decode('utf8'))

        for item in data.get('addresses', []):
            yield {
                'country_code': item.get('country_code'),
                'country': item.get('country'),
                'state': item.get('admin_district'),
                'name': item.get('locality'),
                'latitude': item['latitude'],
                'longitude': item['longitude'],
            }


class App(SimpleApp):
    """The Weather channel wsgi app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        self.base_image_url = self.config['base_image_url']
        self.auth = self.config['secrets']

    def _get_landpage(self, twci, latitude, longitude, locale):
        """Get the weather for the landpage, using localization parameters."""
        results = twci.get_forecast(latitude, longitude)
        if not results:
            return
        results = results[:MAX_FORECAST_DAYS]

        # send first the top two results
        cat = CATEGORY_GRID_MAIN.copy()
        cat['title'] = results[0]['location']
        yield dict(category=cat)
        for result in results[0:2]:
            result.update(type='full', latitude=latitude, longitude=longitude)
            conditions = result['conditions']
            if conditions is not None:
                conditions = conditions.lower()
            title = u', '.join(filter(None, (result['day'], conditions)))
            max_min_temps = (result['max_temp'], result['min_temp'])
            temperatures = u" / ".join(temp for temp in max_min_temps if temp)
            rain = u"%s: %s" % (translator("Rain", locale), result['pop'])
            wind = u"%s: %s" % (translator("Wind", locale),
                                result['wind_sentence'])
            humm = u"%s: %s" % (translator("Humidity", locale),
                                result['humidity'])
            yield dict(result={
                'cat_id': CATEGORY_GRID_MAIN['id'],
                'uri': result['url'],
                'dnd_uri': result['url'],
                'title': title,
                'subtitle': temperatures,
                'art': result['art'],
                'latitude': latitude,
                'longitude': longitude,
                'attributes': [
                    {'style': 'default', 'value': rain},
                    {'style': 'default', 'value': humm},
                    {'style': 'default', 'value': wind},
                    {'style': 'default', 'value': ""},
                ],
                'preview_info': result,
            })

        # finally the rest
        yield dict(category=localized_category(CATEGORY_GRID_REST, locale))
        for result in results[2:]:
            result.update(type='full', latitude=latitude, longitude=longitude)
            title = u', '.join(filter(None,
                               (result['conditions'], result['current_temp'])))
            yield dict(result={
                'cat_id': CATEGORY_GRID_REST['id'],
                'uri': result['url'],
                'dnd_uri': result['url'],
                'title': title,
                'subtitle': result['day'],
                'art': result['art'],
                'preview_info': result,
            })

    def _get_location_search(self, twci, query, country):
        """Search for locations and present them to the user."""

        locations = twci.search_locations(query)
        yield dict(category=CATEGORY_LIST)
        locations = list(locations)
        # Put those in "country" first
        locations.sort(key=lambda x: x.get('country_code') != country)
        for loc in locations:
            title = ', '.join(filter(None, (loc['name'], loc['state'], loc['country'])))
            # Encode lat/long of locations in de dep parameter
            dep = '%s|%s' % (loc['latitude'], loc['longitude'])
            params = dict(q=query.encode("utf8"), dep=dep)
            url_encoded = urllib.urlencode(params)
            uri_encoded = url_encoded.replace('+', '%20')
            cq = 'scope://com.canonical.scopes.weatherchannel?' + uri_encoded
            yield dict(result={
                'cat_id': CATEGORY_LIST['id'],
                'title': title,
                'uri': cq,
                'dnd_uri': cq,
                'preview_info': {
                    'type': 'location',
                    'latitude': loc['latitude'],
                    'longitude': loc['longitude'],
                }
            })

    def search(self, query=None, locale=None, country=None, department=None,
               latitude=None, longitude=None, platform=None, **kwargs):
        """Search in The Weather Channel backend."""
        auth = self.auth[kwargs['partner']]
        twci = TWCInterface(self.urlread, self.base_image_url, auth, locale, platform)

        if department is not None:
            # lat/long are encoded in de department
            if '|' not in department:
                department = urllib.unquote(department)
            latitude, longitude = department.split('|')
            return self._get_landpage(twci, latitude, longitude, locale)

        if query:
            if len(query) < 2:
                # if the query is really small, just ignore it
                return []
            return self._get_location_search(twci, query, country)

        # normal landpage for current localization; get all the info for the
        # received lat/long
        if not latitude and not longitude:
            # missing, or in 0 (which is most probably an error, loc=0/lat=0
            # is way in the ocean anyway)
            raise BadRequestError("Missing latitude/longitude parameters")
        return self._get_landpage(twci, latitude, longitude, locale)

    def preview(self, **kwargs):
        """The preview"""
        result = kwargs['result']
        locale = kwargs['locale']
        platform = kwargs['platform']
        info = result['preview_info']
        latitude = info['latitude']
        longitude = info['longitude']
        auth = self.auth[kwargs['partner']]
        twci = TWCInterface(self.urlread, self.base_image_url, auth, locale, platform)

        if info['type'] == 'location':
            # actually need to get the forecast
            results = twci.get_forecast(latitude, longitude)
            info = results[0]

        # present the preview
        title = "%s: %s" % (info['day'], info['conditions'])
        max_min_temps = (info['max_temp'], info['min_temp'])
        temperatures = u" / ".join(temp for temp in max_min_temps if temp)

        # yield an 'image' widget
        yield dict(widget={
            'id': 'art',
            'type': 'image',
            'source': info['art'],
        })

        # yield a 'header' widget
        yield dict(widget={
            'id': 'header1',
            'type': 'header',
            'uri': info['url'],
            'title': title,
            'subtitle': temperatures,
        })

        # always add an action for the next days
        uri_10day = twci.get_localized_url("10day", latitude, longitude)
        actions = [{
            "id": "action1",
            "icon": None,
            "label": translator("10 day forecast", locale),
            "uri": uri_10day,
        }]

        yield dict(widget={
            "id": "actions",
            "type": "actions",
            "actions": actions,
        })

        # send all the info as a 'table' widget
        infos = [
            (translator("Chance of rain", locale), info['pop']),
            (translator("Humidity", locale), info['humidity']),
            (translator("Feels like", locale), info['feels_like']),
            (translator("Wind", locale), info['wind_sentence']),
            (translator("UV index", locale), info['uv']),
            (translator("Visibility", locale), info['visibility']),
            (translator("Dew point", locale), info['dew']),
            (translator("Sunrise", locale), info['sunrise']),
            (translator("Sunset", locale), info['sunset']),
        ]
        infos = [(desc, val) for desc, val in infos if val]
        yield dict(widget={
            'id': 'table1',
            'type': 'table',
            'values': infos,
        })
