# Copyright 2014-2015 Canonical
# All Rights Reserved

"""The Wikipedia scope."""

import json
import re
import urllib
import feedparser

from xml.dom.minidom import parseString

from bs4 import BeautifulSoup

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category
from .. import rest_scopes

# We use the opensearch action as that seems to be the only action that
# searches only on page names -- the standard search function performs a
# full-text search and may return less relevant pages first.
SEARCH_URL = (
    'http://%(language)s.wikipedia.org'
    '/w/api.php?action=opensearch&limit=%(limit)s&format=xml&search=%(query)s'
)
PREVIEW_URL = (
    'http://%(language)s.wikipedia.org/w/api.php?action=query&'
    'prop=extracts&exchars=%(text_len)s&titles=%(title)s&format=json'
)
FEATURED_URL = (
    'http://%(language)s.wikipedia.org'
    '/w/api.php?action=featuredfeed&limit=%(limit)s&format=xml&feed=featured'
)
RECENTCHANGES_URL = (
    'http://%(language)s.wikipedia.org/w/api.php?action=feedrecentchanges&'
    'limit=%(limit)s&namespace=0&'
    'hideminor=true&hidebots=true&hideanons&hidepatrolled'
)
IMAGEINFO_URL = (
    'http://%(language)s.wikipedia.org/w/api.php?action=query'
    '&prop=imageinfo&titles=File:%(image)s&format=json'
    '&iiprop=url&iiurlwidth=%(width)s&iilimit=1&continue='
)
EXTRACT_URL = (
    'http://%(language)s.wikipedia.org/w/api.php?action=query'
    '&prop=pageimages&titles=%(title)s&format=json&piprop=name'
)


# the categories to use, with some renderers
_renderer_featured = {
    "schema-version": 1,
    'template': {'category-layout': 'grid', 'card-size': 'small'},
    'components': {'title': 'title',
                   'art': 'art'},
}
CATEGORY_FEATURED = {
    'id': 'featured',
    'title': translator('Featured articles'),
    'render_template': json.dumps(_renderer_featured),
}
_renderer_default = {
    "schema-version": 1,
    "template": {
        "category-layout": "grid",
        "card-size": "large",
        "card-layout": "horizontal",
    },
    "components": {
        "title": "title",
        "art": "art",
    },
}
CATEGORY_DEFAULT = {
    'id': 'default_category',
    'title': None,
    'render_template': json.dumps(_renderer_default),
}

# these are the default image sizes for the search and preview
IMAGE_DISPLAY_SIZE = '96'
IMAGE_PREVIEW_SIZE = '512'

# the regex to replace the image size in the returned URL
RE_IMAGE_SIZE = re.compile(r'(?<=\/)\d+(?=px-)')
RE_FEATURED_TITLE = re.compile(r'\stitle\=\"(.*?)\">[^<]')
RE_FEATURED_IMAGE = re.compile(r'.*(http:\/\/.*(\.jpg|\.png)).*')

# these are the wikipedia languages where there is curated content, so it's
# ok to ask for a featured feed
CURATED_SURFACING = {
    'bg', 'cs', 'de', 'el', 'en', 'fa',
    'fr', 'he', 'hu', 'nb', 'ta', 'vi', 'zh',
}


def get_lang_from_locale(locale):
    """Get the language part from the locale."""
    if locale is None:
        language = 'en'
    else:
        language = locale.split('_', 1)[0]
    return language


def _parse_featured_feed(feed, language):
    """Parse the featured RSS response."""
    for item in reversed(feed['entries']):
        soup = BeautifulSoup(item['summary'])

        # get the image
        img_tag = soup.find('img')
        if img_tag is None:
            art = None
        else:
            img = img_tag['src']
            if img.startswith("//"):
                img = "http:" + img
            art = RE_IMAGE_SIZE.sub(IMAGE_DISPLAY_SIZE, img)

        # get the title and link
        for a_tag in soup.find_all('a'):
            if 'image' not in a_tag.get('class', []):
                title = a_tag.text
                url = a_tag['href']
                if url.startswith('/'):
                    url = 'http://%s.wikipedia.org%s' % (language, url)
                break

        cat_id = CATEGORY_FEATURED['id']
        yield art, title, url, cat_id


def _parse_xml(raw):
    """Parse the search based XML response."""
    xml = parseString(raw)
    for item in xml.getElementsByTagName('Item'):
        img = item.getElementsByTagName('Image')
        if img:
            # the image for the search result
            img_source = img[0].getAttribute("source")
            art = RE_IMAGE_SIZE.sub(IMAGE_DISPLAY_SIZE, img_source)
        else:
            art = None
        title = item.getElementsByTagName('Text')[0].firstChild.data
        url = item.getElementsByTagName('Url')[0].firstChild.data
        cat_id = CATEGORY_DEFAULT['id']
        yield art, title, url, cat_id


class App(SimpleApp):
    """The Wikipedia app."""

    def __init__(self, *args, **kwargs):
        super(App, self).__init__(*args, **kwargs)
        config = rest_scopes.get_scopes_config()['wikipedia']
        self.icon = self.config['base_image_url'] + '/' + config['icon']
        self.headers = {
            'User-Agent': self.config['user_agent'],
            'Accept': '*/*',
        }

    def _get_image_url(self, language, image_name, size):
        """Get the image url for the specific size."""
        url = IMAGEINFO_URL % dict(language=language, width=size, image=image_name)
        raw = self.urlread(url)
        data = json.loads(raw)
        resp = data['query']['pages'].values()[0]
        if 'imageinfo' in resp:
            url = resp['imageinfo'][0]['url']
        else:
            url = None
        return url

    def _get_image_from_page(self, q_title, language):
        """Get the image url for an article."""
        # get an image name for the page
        url = EXTRACT_URL % dict(title=q_title, language=language)
        raw = self.urlread(url)
        data = json.loads(raw.decode("utf8"))

        item = list(data['query']['pages'].values())[0]
        image_name = item.get('pageimage')
        if image_name:
            image_name = urllib.quote(image_name.encode('utf8'))
            url = self._get_image_url(language, image_name, IMAGE_DISPLAY_SIZE)
            return url

    def _parse_recentchanges_feed(self, feed, language):
        """Parse the recent-changes RSS response."""
        for item in feed['entries']:
            title = item['title']
            q_title = urllib.quote(title.encode("utf8"))
            data = dict(title=q_title, locale=language)
            url = 'http://%(locale)s.wikipedia.org/wiki/%(title)s' % data
            cat_id = CATEGORY_FEATURED['id']
            art = self._get_image_from_page(q_title, language)
            yield art, title, url, cat_id

    def _get_search(self, query, limit, language):
        """Hit Wikipedia to do a search."""
        query = urllib.quote(query.encode("utf8"))
        url = SEARCH_URL % dict(language=language, limit=limit, query=query)
        return self.urlread(url, headers=self.headers)

    def _get_preview(self, titles, text_len, language):
        """Hit Wikipedia to do a preview."""
        url = PREVIEW_URL % dict(language=language, text_len=text_len, title="|".join(titles))
        return self.urlread(url, headers=self.headers)

    def _get_recentchanges(self, limit, language):
        """Hit Wikipedia for recent changes."""
        url = RECENTCHANGES_URL % dict(language=language, limit=limit)
        raw = self.urlread(url)
        feed = feedparser.parse(raw)
        return feed

    def _get_featured(self, limit, language):
        """Hit Wikipedia for featured articles."""
        url = FEATURED_URL % dict(language=language, limit=limit)
        raw = self.urlread(url)
        feed = feedparser.parse(raw)
        return feed

    def search(self, **kwargs):
        """Do the search."""
        query = kwargs['query']
        locale = kwargs['locale']
        limit = kwargs['limit']
        language = get_lang_from_locale(locale)

        # do the search, and get the result
        if not query.strip():
            yield dict(category=localized_category(CATEGORY_FEATURED, locale))
            if language in CURATED_SURFACING:
                feed = self._get_featured(limit, language)
                info = _parse_featured_feed(feed, language)
            else:
                feed = self._get_recentchanges(limit, language)
                info = self._parse_recentchanges_feed(feed, language)
        else:
            yield dict(category=localized_category(CATEGORY_DEFAULT, locale))
            raw = self._get_search(query, limit, language)
            info = _parse_xml(raw)

        # Create the result
        for art, title, url, cat_id in info:
            if not art:
                art = self.icon
            yield dict(result={
                'cat_id': cat_id,
                'uri': url,
                'dnd_uri': url,
                'title': title,
                'art': art,
            })

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']
        language = get_lang_from_locale(locale)

        # some info from the previous search result
        title = result['title']
        art = result['art']
        uri = result['uri']
        mascot = art

        # if we get a bigger art from wikipedia, send it as image and cancel the mascot
        image_name = art.split("/")[-2]
        bigger_art = self._get_image_url(language, image_name, IMAGE_PREVIEW_SIZE)
        if bigger_art is not None:
            mascot = None
            yield dict(widget={
                'id': 'art',
                'type': 'image',
                'source': bigger_art,
            })

        # yield a 'header' widget
        yield dict(widget={
            'id': 'header1',
            'type': 'header',
            'uri': uri,
            'title': title,
            'mascot': mascot,
        })

        # get a longer text using the real title and the title used in the uri
        titles = (urllib.quote(title.encode("utf8")), uri.split("/")[-1])
        raw = self._get_preview(titles, 200, language)
        data = json.loads(raw)
        page_info = data['query']['pages'].values()
        extracts = [x['extract'] for x in page_info if 'extract' in x]

        if extracts:
            # send it as a 'text' widget
            yield dict(widget={
                'id': 'text1',
                'type': 'text',
                'uri': uri,
                'title': None,
                'text': extracts[0],
            })

        # add an 'actions' widget
        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'label': translator('View on Wikipedia', locale),
                         'uri': uri,
                         'id': 'action1'}]
        })
