# -*- coding: utf-8 -*-

# Copyright 2014-2015 Canonical
# All Rights Reserved


"""The Yahoo Finance scope."""

import csv
import feedparser
import json
import re
import urllib

from ..scopes import SimpleApp
from ..translator import translator
from ..utils import localized_category

BASE_URI = 'http://finance.yahoo.com'
STOCKTWITS_URI = 'https://api.stocktwits.com/api/2/trending/symbols.json'
SYMBOL_NEWS_URL = "http://finance.yahoo.com/rss/headline?s=%(symbol)s"
SYMBOL_LOOKUP_URL = (
    "http://d.yimg.com/autoc.finance.yahoo.com/autoc?"
    "query=%(query)s&callback=YAHOO.Finance.SymbolSuggest.ssCallback&region=US&lang=en"
)

# the list of countries that have a standard news URL
NEWS_URL_STANDARD = set("ar bg br ca ch de ee es et fi fr hr hu ie it lt lv "
                        "nl no pa pl pt ru sg si sk th tr tw uk".split())

# some countries have a custom URL
NEWS_URL_CUSTOM = {
    'za': "http://za.news.yahoo.com/rss/finance",
}


RENDERER_QUOTES_SURFACING = {
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'medium',
                 'card-layout': 'vertical',
                 'collapsed-rows': 2},
    'components': {'title': 'title',
                   'mascot': 'graph',
                   'subtitle': 'change_p', },
}

RENDERER_QUOTES = json.dumps({
    "schema-version": 1,
    'template': {'category-layout': 'grid',
                 'card-size': 'large',
                 'card-layout': 'horizontal'},
    'components': {'title': 'title',
                   'mascot': 'graph',
                   'subtitle': 'symbol',
                   'attributes': {'field': 'attributes', 'max-count': 3}},
})

TRENDING_CATEGORY = {
    'id': 'trending',
    'title': translator('Trending on Twitter'),
    'render_template': json.dumps(RENDERER_QUOTES_SURFACING),
}

HEADLINES_CATEGORY = {
    'id': 'headlines',
    'title': translator('Headlines'),
    'render_template': json.dumps({
        "schema-version": 1,
        'template': {'category-layout': 'grid',
                     'card-size': 'large',
                     'card-layout': 'horizontal'},
        'components': {'title': 'title',
                       'subtitle': 'date',
                       'art': 'art'},
    }),
}


def _select_news_url(locale):
    """Select the URL to retrieve the news."""
    # get the language and country from the locale
    if '_' in locale:
        language, country = locale.lower().split("_")
    else:
        language = locale.lower()
        country = None

    if country in NEWS_URL_STANDARD:
        # the url is standard for this country
        return "http://{}.yahoo.com/finance/news/rss/".format(country)

    try:
        # return the custom URL, if available
        return NEWS_URL_CUSTOM[country]
    except KeyError:
        pass

    if language == 'es':
        # we have a default for Spanish
        return "http://es-us.finanzas.yahoo.com/noticias/rss/"

    # the really real default
    return "http://yahoo.com/finance/news/rss/"


class App(SimpleApp):
    """The Yahoo Finance app."""

    def _get_symbol_from_name(self, query):
        """Get symbols for a name search."""
        symbols = {}
        uri = SYMBOL_LOOKUP_URL % dict(query=query)
        response = self.urlread(uri).decode('utf-8')
        # Get rid of the callback
        cleaned = response[response.index("(") + 1: response.rindex(")")]
        r = json.loads(cleaned)
        symbols_query = []
        for s in r['ResultSet'].get('Result', []):
            symbols_query.append(s['symbol'])
            try:
                exchange = s['exchDisp']
            except KeyError:
                exchange = "other"
            symbols[s['symbol']] = {'exchange': exchange, 'name': s['name']}
        return " ".join(symbols_query), symbols

    def _get_trending_symbols(self, limit):
        """Get symbols trending on Twitter with StockTwits."""
        uri = STOCKTWITS_URI + "?limit=" + str(limit)
        response = self.urlread(uri)
        r = json.loads(response.decode('utf-8'))
        symbols = ' '.join([s['symbol'] for s in r['symbols']])
        return symbols

    def _get_quotes(self, url):
        """Call the Yahoo Finance API."""
        data = self.urlread(url)
        fnames = ('symbol', 'name', 'value', 'date', 'time', 'volume', 'change')
        reader = csv.DictReader(data.split("\n"), fieldnames=fnames)
        # bad info from server will result in items with Nones, filter them out
        results = [x for x in reader if None not in x.values()]
        return results

    def _read_feed(self, url):
        """Read a feed."""
        try:
            raw = self.urlread(url)
            d = feedparser.parse(raw)
        except:
            # something wrong with backend response
            return
        return d

    def _get_news(self, locale):
        """Get top news from Yahoo! Finance rss feeds."""
        url = _select_news_url(locale)
        d = self._read_feed(url)
        if not d:
            return []

        stories = []
        for entry in d['entries']:
            # discard incomplete or non-free articles
            title = entry.get('title')
            if title is None or title.startswith('[$$]'):
                continue
            link = entry.get('link')
            if link is None:
                continue

            # if no published date, just use the updated one
            date = entry.get('published')
            if date is None:
                date = entry.get('updated')

            # the summary has html tags and quotes!
            summary = re.sub("<.*?>", "", entry['summary']).replace("&quot;", '"')

            # it may be a media content, get the first url
            if "media_content" in entry.keys() and entry['media_content']:
                image = entry['media_content'][0]['url']
            else:
                image = None

            stories.append({
                'title': title,
                'uri': link,
                'date': date,
                'summary': summary,
                'image': image,
            })
        return stories

    def search(self, **kwargs):
        """Do the search."""
        query = kwargs['query'].encode('utf-8')
        limit = kwargs['limit']
        locale = kwargs['locale']
        headlines = []
        surfacing = False
        if not query.strip():
            surfacing = True
            query = self._get_trending_symbols(limit)
            symbol_map = {}
            uri = '%(base_uri)s/d/quotes.csv?s=%(quotes)s&f=snl1d1t1vc' % dict(
                base_uri=BASE_URI, quotes=urllib.quote(query.encode('utf8')))
            quotes = self._get_quotes(uri)
            headlines = self._get_news(locale)
        else:
            quoted_query = urllib.quote(query)
            query_string, symbol_map = self._get_symbol_from_name(quoted_query)
            d = dict(base_uri=BASE_URI, quotes=urllib.quote(query_string))
            uri = '%(base_uri)s/d/quotes.csv?s=%(quotes)s&f=snl1d1t1vc' % d
            quotes = self._get_quotes(uri)
            sent_exchanges = []
            for s in symbol_map:
                if symbol_map[s]['exchange'] and \
                        not symbol_map[s]['exchange'] in sent_exchanges:
                    cat = {
                        'render_template': RENDERER_QUOTES,
                        'id': symbol_map[s]['exchange'],
                        'title': symbol_map[s]['exchange'],
                    }
                    yield dict(category=cat)
                    sent_exchanges.append(symbol_map[s]['exchange'])
            other_quotes = {
                'render_template': RENDERER_QUOTES,
                'id': "other",
                'title': translator("Other exchanges", locale),
            }
            yield dict(category=other_quotes)
        if headlines:
            yield dict(category=localized_category(HEADLINES_CATEGORY, locale))
            for h in headlines:
                yield dict(result={
                    'cat_id': 'headlines',
                    'uri': h['uri'],
                    'dnd_uri': h['uri'],
                    'title': h['title'],
                    'date': h['date'],
                    'summary': h['summary'],
                    'art': h['image'],
                })
        if surfacing:
            yield dict(category=localized_category(TRENDING_CATEGORY, locale))
        for quote in quotes:
            graph = "http://chart.finance.yahoo.com/z?" \
                "s=" + quote['symbol'] + "&z=s&t=7d"
            value_change = quote['change'].split(' ')[0]
            percent_change = quote['change'].split(' ')[-1]
            uri = 'http://finance.yahoo.com/q?s=' + quote['symbol']
            if surfacing:
                cat_id = "trending"
            else:
                if quote['symbol'] in symbol_map:
                    cat_id = symbol_map[quote['symbol']]['exchange']
                else:
                    cat_id = "other"
            yield dict(result={
                'uri': uri,
                'dnd_uri': uri,
                'graph': graph,
                'cat_id': cat_id,
                'title': quote['name'],
                'value': quote['value'],
                'symbol': quote['symbol'],
                'change_v': value_change,
                'change_p': percent_change,
                'attributes': [{'value': quote['value']},
                               {'value': value_change},
                               {'value': percent_change}]
            })

    def preview(self, **kwargs):
        """Do the preview."""
        result = kwargs['result']
        locale = kwargs['locale']
        if result['cat_id'] != 'headlines':
            graph = "http://chart.finance.yahoo.com/z?" \
                    "s=" + result['symbol'] + "&z=s&t=7d"

            # yield an 'image' widget
            yield dict(widget={
                'id': 'art',
                'type': 'image',
                'source': graph,
            })

            # Append the exchange to the symbol when it's not a trending value
            if result['cat_id'] != 'trending':
                subtitle = "%s - %s" % (result['symbol'], result['cat_id'])
            else:
                subtitle = result['symbol']

            # yield a 'header' widget
            yield dict(widget={
                'id': 'header',
                'type': 'header',
                'title': result['title'],
                'subtitle': subtitle,
            })

            # Content for the text widget
            text_for_values = []
            text_for_values.append(
                translator('Price: %s' % result['value'], locale))
            text_for_values.append(
                translator('Change: %s' % result['change_v'], locale))
            text_for_values.append(
                translator('%% Change: %s' % result['change_p'], locale))

            # yield a 'text' widget
            yield dict(widget={
                'id': 'text',
                'type': 'text',
                'text': '\n'.join(text_for_values),
            })
        else:
            # yield a 'header' widget
            yield dict(widget={
                'id': 'header',
                'type': 'header',
                'title': result['title'],
                'subtitle': result['date'],
            })

        # yield an 'actions' widget
        yield dict(widget={
            'id': 'buttons',
            'type': 'actions',
            'actions': [{'uri': result['uri'],
                         'label': translator('Yahoo! Finance', locale),
                         'id': 'yahoo'}]
        })

        if result['cat_id'] == 'headlines':
            # yield an 'text' widget
            yield dict(widget={
                'id': 'text',
                'type': 'text',
                'text': result['summary'],
            })
