# Copyright 2014-2015 Canonical
# All Rights Reserved

"""Tests for the Rest Scopes server."""

import io
import json
import uuid
import os
import urllib2

from unittest import TestCase

from mock import Mock, patch, ANY

from src import rest_scopes
from src.scopes.tests import SetupLogChecker


class InitTestCase(TestCase):
    """Test initialization."""

    def test_set_scope_config(self):
        secrets = {'wikipedia': {'foo': 'bar'}, 'other': ['1', '2']}
        rest_scopes.get_scopes_secrets = Mock(return_value=secrets)
        app = rest_scopes.App()
        wikipedia = app.scope_apps['wikipedia']

        should_config = rest_scopes.config.copy()
        should_config['secrets'] = {'foo': 'bar'}
        self.assertEqual(wikipedia.config, should_config)

    def test_secrets_loading(self):
        from_file = dict(default={'scope1': {'foo': 'bar1', 'bar': 'baz'},
                                  'scope2': {'secret': 'stuff'}},
                         partn1={'scope1': {'foo': 'bar2', 'bar': 'baz'},
                                 'scope2': {'secret': 'stuff'}},
                         partn2={'scope1': {'foo': 'bar3', 'bar': 'baz'},
                                 'scope2': {'secret': 'stuff'}})
        with patch.object(rest_scopes.yaml, 'load') as mock:
            mock.return_value = from_file
            resp = rest_scopes.get_scopes_secrets()
        self.assertEqual(resp, {
            'scope1': {None: {'foo': 'bar1', 'bar': 'baz'},
                       'partn1': {'foo': 'bar2', 'bar': 'baz'},
                       'partn2': {'foo': 'bar3', 'bar': 'baz'}},
            'scope2': {None: {'secret': 'stuff'},
                       'partn1': {'secret': 'stuff'},
                       'partn2': {'secret': 'stuff'}},
        })

    def test_extra_scope_config(self):
        extra = {'wikipedia': {'foo': 'bar'}}
        app = rest_scopes.App(extra)

        # wikipedia config is changed
        wikipedia = app.scope_apps['wikipedia']
        self.assertEqual(wikipedia.config['foo'], 'bar')

        # but the other scopes' isn't
        other_scope = app.scope_apps['ebay']
        self.assertNotIn('foo', other_scope.config)

    def test_real_partners_building(self):
        secrets = {
            'weatherchannel': {None: {'foo': 'bar1', 'bar': 'baz'},
                               'partn1': {'foo': 'bar2', 'bar': 'baz'}},
            'sevendigital': {None: {'secret': 'stuff'},
                             'partn2': {'secret': 'stuff'}},
        }
        with patch.object(rest_scopes, 'get_scopes_secrets') as mock:
            mock.return_value = secrets
            app = rest_scopes.App()
        self.assertEqual(app._real_partners, set([None, 'partn1', 'partn2']))

    def test_oopser_passing(self):
        oopser = 'test oopser'
        app = rest_scopes.App(oopser=oopser)

        # any scope would do
        scope = app.scope_apps.values()[0]
        self.assertEqual(scope.oopser, 'test oopser')


class CallTestCase(TestCase):
    """Test when calling the class."""

    def test_list_remote_scopes(self):
        meter = Mock()
        app = rest_scopes.App(metrics=(meter, ''))  # no base!
        app.list_remote_scopes = mock = Mock()
        mock.return_value = 'response'

        environ = {
            'PATH_INFO': '/remote-scopes',
        }
        resp = app(environ, 'start_response')
        self.assertEqual(resp, 'response')
        mock.assert_called_with(environ, 'start_response')
        meter.timing.assert_called_with('restscopes.remote-scopes.200', ANY)

    def test_scope_not_found(self):
        meter = Mock()
        app = rest_scopes.App(metrics=(meter, 'base'))
        app._error = mock = Mock()
        mock.return_value = 'error response'

        environ = {
            'PATH_INFO': '/not_a_scope',
        }
        resp = app(environ, 'start_response')
        self.assertEqual(resp, 'error response')
        mock.assert_called_with('404 NOT FOUND', 'start_response')
        meter.timing.assert_called_with('base.restscopes.404', ANY)

    def test_scope_with_bad_path(self):
        meter = Mock()
        app = rest_scopes.App(metrics=(meter, 'base'))
        app._error = mock = Mock()
        mock.return_value = 'error response'

        assert 'wikipedia' in app.scope_apps
        environ = {
            'PATH_INFO': '/wikipedia',
        }
        resp = app(environ, 'start_response')
        self.assertEqual(resp, 'error response')
        msg = "Malformed URL: need an action after the scope name"
        mock.assert_called_with('400 BAD REQUEST', 'start_response', msg)
        meter.timing.assert_called_with('base.restscopes.400', ANY)

    def test_scope_ok(self):
        app = rest_scopes.App()

        app.scope_apps['somescope'] = app_mock = Mock()
        app_mock.return_value = "some response from the scope"
        environ = {
            'PATH_INFO': '/somescope/somemethod',
            'FOO': 'bar',
        }
        resp = app(environ, 'start_response')
        self.assertEqual(resp, "some response from the scope")
        new_environ = {
            'PATH_INFO': '/somemethod',
            'FOO': 'bar',
            'USER_AGENT': {},
        }
        app_mock.assert_called_with(new_environ, 'start_response')

    def test_user_agent(self):
        app = rest_scopes.App()
        app.scope_apps['somescope'] = app_mock = Mock()
        environ = {
            'PATH_INFO': '/somescope/somemethod',
            'HTTP_USER_AGENT': 'foo=3&bar=5',
        }
        app(environ, 'start_response')
        new_environ = {
            'PATH_INFO': '/somemethod',
            'USER_AGENT': {'foo': '3', 'bar': '5'},
        }
        app_mock.assert_called_with(new_environ, 'start_response')

    def test_scope_deep_sight_real_partner(self):
        app = rest_scopes.App()
        app.scope_apps['somescope'] = app_mock = Mock()
        app._real_partners = [None, 'foo']

        environ = {
            'PATH_INFO': '/somescope/somemethod',
            'HTTP_USER_AGENT': 'partner=foo',
        }
        app(environ, 'start_response')
        new_environ = {
            'PATH_INFO': '/somemethod',
            'USER_AGENT': {'partner': 'foo'},
        }
        app_mock.assert_called_with(new_environ, 'start_response')

    def test_scope_deep_sight_in_action(self):
        app = rest_scopes.App()
        app.scope_apps['somescope'] = app_mock = Mock()
        app._real_partners = [None, 'foo']

        environ = {
            'PATH_INFO': '/somescope/somemethod',
            'HTTP_USER_AGENT': 'partner=bar',
        }
        app(environ, 'start_response')
        new_environ = {
            'PATH_INFO': '/somemethod',
            'USER_AGENT': {},
            'DEEP_SIGHT': 'bar',
        }
        app_mock.assert_called_with(new_environ, 'start_response')

    def test_scope_deep_sight_in_action_dirty_partner(self):
        app = rest_scopes.App()
        app.scope_apps['somescope'] = app_mock = Mock()
        app._real_partners = [None, 'foo']

        environ = {
            'PATH_INFO': '/somescope/somemethod',
            'HTTP_USER_AGENT': 'partner=bar\n',
        }
        app(environ, 'start_response')
        new_environ = {
            'PATH_INFO': '/somemethod',
            'USER_AGENT': {},
            'DEEP_SIGHT': 'bar',
        }
        app_mock.assert_called_with(new_environ, 'start_response')


class ListRemoteScopesTestCase(TestCase):
    """Test the remote scopes listing."""

    def test_avoid_hidden_scope(self):
        app = rest_scopes.App()
        app._generic_response = Mock()
        app._generic_response.return_value = 'generic response result'

        # set up the original data from the config
        _scopes_config = [{
            'id': '1',
            'description': 'description',
            'name': 'name',
            'hidden': False,
        }, {
            'id': '2',
            'description': 'description',
            'name': 'name',
            'hidden': 'someuser',
        }, {
            'id': '3',
            'description': 'description',
            'name': 'name',
            'hidden': 'otheruser',
        }]

        # set all the scopes config, and which ones we should get later
        assert rest_scopes.config['hide_scopes']
        app.scope_info[None] = _scopes_config
        final_scope_list = [x for x in _scopes_config if x['id'] in (
            '1',  # 'hidden' in False
            '2',  # 'hidden' but indicating that 'someuser' can see it
        )]

        environ = {'DEEP_SIGHT': 'someuser'}  # will see scope 3 but not 4

        app.list_remote_scopes(environ, 'start_response')
        args, kwargs = app._generic_response.call_args
        self.assertEqual(args[0], "200 OK")
        self.assertEqual(args[1], 'start_response')
        self.assertEqual(json.loads(args[2]), final_scope_list)

    def test_dont_hide_hidden(self):
        app = rest_scopes.App()
        app._generic_response = Mock()
        app._generic_response.return_value = 'generic response result'

        # set up the original data from the config
        _scopes_config = [{
            'id': '1',
            'description': 'description',
            'name': 'name',
            'hidden': 'user',
        }]

        # set all the scopes config, and which ones we should get later
        rest_scopes.config['hide_scopes'] = False
        app.scope_info[None] = _scopes_config

        app.list_remote_scopes({}, 'start_response')
        args, kwargs = app._generic_response.call_args
        self.assertEqual(args[0], "200 OK")
        self.assertEqual(args[1], 'start_response')
        scope, = json.loads(args[2])
        self.assertEqual(scope['id'], '1')

    def test_blacklist(self):
        rest_scopes.config['blacklist'] = {'FR': ['foo']}
        app = rest_scopes.App()
        app._generic_response = Mock()
        app._generic_response.return_value = 'generic response result'

        # set up the original data from the config
        app.scope_info[None] = [{
            'id': 'foo',
            'description': 'description',
            'name': 'name',
            'hidden': False,
        }, {
            'id': 'bar',
            'description': 'description',
            'name': 'name',
            'hidden': False,
        }]

        environ = {'QUERY_STRING': 'country=FR'}

        # Ensure blacklisted scopes are not returned
        final_scope_list = [{
            'id': 'bar',
            'description': 'description',
            'name': 'name',
            'hidden': False,
        }]
        app.list_remote_scopes(environ, 'start_response')
        args, kwargs = app._generic_response.call_args
        self.assertEqual(args[0], "200 OK")
        self.assertEqual(args[1], 'start_response')
        self.assertEqual(json.loads(args[2]), final_scope_list)

    def test_call_simple(self):
        app = rest_scopes.App()
        app._generic_response = Mock()
        app._generic_response.return_value = 'generic response result'

        # set up the original data from the config
        app.scope_info[None] = [{
            'foo': 1,
            'description': 'description 1',
            'name': 'name 1',
            'hidden': False,
        }, {
            'foo': 2,
            'description': 'description 2',
            'name': 'name 2',
            'hidden': False,
        }]

        environ = {'QUERY_STRING': 'locale=es_AR'}

        def fake_translator(text, locale):
            """Translate in a funny way."""
            return "translated-%s %s" % (locale, text)

        with patch.object(rest_scopes, 'translator', fake_translator):
            resp = app.list_remote_scopes(environ, 'start_response')

        # check method returned proper info
        final_scope_list = [{
            'foo': 1,
            'description': 'translated-es_AR description 1',
            'name': 'translated-es_AR name 1',
            'hidden': False,
        }, {
            'foo': 2,
            'description': 'translated-es_AR description 2',
            'name': 'translated-es_AR name 2',
            'hidden': False,
        }]
        self.assertEqual(resp, "generic response result")
        app._generic_response.assert_called_with(
            "200 OK", 'start_response', json.dumps(final_scope_list), Vary='User-Agent')

        # verify we had stored the translation for future usage
        self.assertEqual(app.scope_info['es_AR'], final_scope_list)

    def test_call_dont_translate_already_translated(self):
        app = rest_scopes.App()
        app._generic_response = Mock()
        app._generic_response.return_value = 'generic response result'

        # set up the original data from the config, already translated
        scope_list = [{
            'foo': 1,
            'description': 'translated 1',
            'hidden': False,
        }, {
            'foo': 2,
            'description': 'translated 2',
            'hidden': False,
        }]
        app.scope_info['es_AR'] = scope_list

        environ = {'QUERY_STRING': 'locale=es_AR'}

        with patch.object(rest_scopes, 'translator') as trans_mock:
            resp = app.list_remote_scopes(environ, 'start_response')

        self.assertFalse(trans_mock.called)
        self.assertEqual(resp, "generic response result")
        app._generic_response.assert_called_with(
            "200 OK", 'start_response', json.dumps(scope_list), Vary='User-Agent')

    def test_empty(self):
        with patch.object(rest_scopes, 'get_scopes_config', lambda: {}):
            app = rest_scopes.App()
        self.assertEqual(app.scope_info[None], [])

    def test_internal(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': '/somescope',
                'invisible': False,
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            with patch('importlib.import_module') as mock:
                app = rest_scopes.App()
                mock.assert_called_with("src.scopes.somescope")
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url':  "http://localhost:8000/somescope",
            'invisible': False,
            'icon': None,
            'art': None,
            'hidden': False,
            'appearance': {},
            'needs_location_data': False,
            'keywords': [],
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_external(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': None,
            'art': None,
            'hidden': False,
            'appearance': {},
            'needs_location_data': False,
            'keywords': [],
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_fqdn(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'fqdn': 'some.fqdn.name.somescope',
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'some.fqdn.name.somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': None,
            'art': None,
            'hidden': False,
            'appearance': {},
            'needs_location_data': False,
            'keywords': [],
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_strip_desc_name(self):
        config = {
            'somescope': {
                'name': ' name',
                'description': 'desc\n',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'fqdn': 'some.fqdn.name.somescope',
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        returned = app.scope_info[None][0]
        self.assertEqual(returned['name'], 'name')
        self.assertEqual(returned['description'], 'desc')

    def test_internal_otherpath(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': '/somescope',
                'invisible': False,
                'mod_package': 'src.other',
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            with patch('importlib.import_module') as mock:
                app = rest_scopes.App()
                mock.assert_called_with("src.other.somescope")
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url':  "http://localhost:8000/somescope",
            'invisible': False,
            'icon': None,
            'art': None,
            'hidden': False,
            'appearance': {},
            'needs_location_data': False,
            'keywords': [],
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_icon_art(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'icon': 'somedir/foo-icon.png',
                'art': 'somedir/foo-art.png',
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': 'http://localhost:8000/_static/somedir/foo-icon.png',
            'art': 'http://localhost:8000/_static/somedir/foo-art.png',
            'hidden': False,
            'appearance': {},
            'needs_location_data': False,
            'keywords': [],
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_appearance(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'appearance': {
                    'bgcolor': '#ffffff',
                    'page-header': {'logo': 'logo.png'},
                },
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'needs_location_data': False,
            'icon': None,
            'art': None,
            'hidden': False,
            'keywords': [],
            'appearance': {
                'bgcolor': '#ffffff',
                'page-header': {
                    'logo': 'http://localhost:8000/_static/logo.png',
                },
            },
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_needslocationdata(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'needs_location_data': True,
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': None,
            'art': None,
            'hidden': False,
            'appearance': {},
            'needs_location_data': True,
            'keywords': [],
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_hidden_parameter(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'hidden': 'foobar',
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': None,
            'art': None,
            'hidden': 'foobar',
            'appearance': {},
            'needs_location_data': False,
            'keywords': [],
        }]
        self.assertEqual(app.scope_info[None], result)

    def test_keywords(self):
        config = {
            'somescope': {
                'name': 'name',
                'description': 'desc',
                'author': 'John Doe',
                'base_url': 'http://server.com/somescope',
                'invisible': False,
                'keywords': ['tag1', 'tag2'],
            }
        }
        with patch.object(rest_scopes, 'get_scopes_config', lambda: config):
            app = rest_scopes.App()
        result = [{
            'id': 'somescope',
            'name': 'name',
            'description': 'desc',
            'author': 'John Doe',
            'base_url': 'http://server.com/somescope',
            'invisible': False,
            'icon': None,
            'art': None,
            'hidden': False,
            'appearance': {},
            'needs_location_data': False,
            'keywords': ['tag1', 'tag2'],
        }]
        self.assertEqual(app.scope_info[None], result)


class HelpersTestCase(TestCase):
    """Test some helper methods."""

    def setUp(self):
        self.app = rest_scopes.App()
        self.start_response = Mock()

    def test_generic_response_sanity(self):
        resp = self.app._generic_response("200 test", self.start_response,
                                          "test message", header='extra')
        self.assertEqual(resp, ["test message"])
        should_status = "200 test"
        should_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len("test message"))),
            rest_scopes.REVNO_HEADER,
            ('header', 'extra'),
        ]
        self.start_response.assert_called_with(should_status, should_headers)

    def test_error_sanity(self):
        resp = self.app._error("400 test", self.start_response,
                               "test message", header='extra')
        should_error_message = json.dumps(
            {'error': '400 test', 'message': 'test message'}, indent=4)

        self.assertEqual(resp, [should_error_message])
        should_status = "400 test"
        should_headers = [
            ('Content-type', "application/json"),
            ('Content-Length', str(len(should_error_message))),
            rest_scopes.REVNO_HEADER,
            ('header', 'extra'),
        ]
        self.start_response.assert_called_with(should_status, should_headers)

    def test_get_scopes_config_simple(self):
        config_copy = rest_scopes.config.copy()
        test_fname = "test-" + str(uuid.uuid4())
        with open(test_fname, 'wt') as fh:
            fh.write("foo: bar\n")
        self.addCleanup(os.remove, test_fname)

        config_copy['scopes_config'] = test_fname
        with patch.object(rest_scopes, 'config', config_copy):
            loaded = rest_scopes.get_scopes_config()

        self.assertEqual(loaded, {'foo': 'bar'})

    def test_get_scopes_config_composed(self):
        config_copy = rest_scopes.config.copy()
        test_fname_1 = "test-" + str(uuid.uuid4())
        with open(test_fname_1, 'wt') as fh:
            fh.write("aaa: 111\nbbb: 222\n")
        self.addCleanup(os.remove, test_fname_1)
        test_fname_2 = "test-" + str(uuid.uuid4())
        with open(test_fname_2, 'wt') as fh:
            fh.write("bbb: 777\nccc: 333\n")
        self.addCleanup(os.remove, test_fname_2)

        config_copy['scopes_config'] = [test_fname_1, test_fname_2]
        with patch.object(rest_scopes, 'config', config_copy):
            loaded = rest_scopes.get_scopes_config()

        self.assertEqual(loaded, {'aaa': 111, 'bbb': 777, 'ccc': 333})

    def test_dict_merger(self):
        d1 = {'a': {'b': 1, 'h': 5}, 'j': 7}
        d2 = {'a': {'b': 2, 'c': 3}, 'd': None}
        resp = rest_scopes.dict_merger(d1, d2)
        self.assertDictEqual(
            resp, {'a': {'b': 2, 'c': 3, 'h': 5}, 'j': 7, 'd': None})


class URLOpenerTestCase(TestCase):
    """Test the centralized url opener."""

    def setUp(self):
        SetupLogChecker(self, 'restscopes.urlread')

    def test_simple_url(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {}

        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            resp = rest_scopes.urlread(None, None, 'http://blah/')

        self.assertEqual(resp, 'response')

    def test_with_data(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {}

        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            resp = rest_scopes.urlread(None, None, 'http://blah/', data='data')

        self.assertEqual(resp, 'response')
        req = mock_urlopen.call_args[0][0]
        self.assertEqual(req.data, 'data')
        self.assertEqual(req.headers, {})

    def test_with_headers(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {}

        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            resp = rest_scopes.urlread(None, None, 'http://blah/', headers={'A': 3})

        self.assertEqual(resp, 'response')
        req = mock_urlopen.call_args[0][0]
        self.assertEqual(req.data, None)
        self.assertEqual(req.headers, {'A': 3})

    def test_metrics_no_prefix(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {}

        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            rest_scopes.urlread(meter, '', 'http://www.blah.com/foo/bar?a=3')

        meter.timing.assert_called_with('backend.www_blah_com.200.no-cache', ANY)

    def test_with_metrics_ok_nocache(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {}

        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            rest_scopes.urlread(meter, 'base', 'http://www.blah.com/foo/bar?a=3', headers={'A': 3})

        meter.timing.assert_called_with('base.backend.www_blah_com.200.no-cache', ANY)

    def test_with_metrics_ok_cache_miss(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {
            'X-Cache': 'MISS from localhost',
            'X-Cache-Lookup': 'MISS from localhost',
        }

        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            rest_scopes.urlread(meter, 'base', 'http://www.blah.com/foo/bar?a=3', headers={'A': 3})

        meter.timing.assert_called_with('base.backend.www_blah_com.200.miss', ANY)

    def test_with_metrics_ok_cache_expired(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {
            'X-Cache': 'MISS from localhost',
            'X-Cache-Lookup': 'HIT from localhost',
        }

        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            rest_scopes.urlread(meter, 'base', 'http://www.blah.com/foo/bar?a=3', headers={'A': 3})

        meter.timing.assert_called_with('base.backend.www_blah_com.200.miss-expired', ANY)

    def test_with_metrics_ok_cached_simple(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {
            'X-Cache': 'HIT from localhost',
            'X-Cache-Lookup': 'dont care',
        }

        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            rest_scopes.urlread(meter, 'base', 'http://www.blah.com/foo/bar?a=3', headers={'A': 3})

        meter.timing.assert_called_with('base.backend.www_blah_com.200.hit', ANY)

    def test_with_metrics_ok_cached_multiple(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {
            'X-Cache': 'MISS, MISS, HIT from localhost',
        }

        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            rest_scopes.urlread(meter, 'base', 'http://www.blah.com/foo/bar?a=3', headers={'A': 3})

        meter.timing.assert_called_with('base.backend.www_blah_com.200.hit', ANY)

    def test_with_metrics_http_error_nocached(self):
        req_headers = {
            'X-Cache': 'MISS from localhost',
            'X-Cache-Lookup': 'MISS from localhost',
        }
        exc = urllib2.HTTPError(None, 500, "Internal Server Error", req_headers, None)

        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.side_effect = exc
            self.assertRaises(urllib2.HTTPError, rest_scopes.urlread,
                              meter, 'base', 'http://www.blah.com/foo/bar?a=3', headers={'A': 3})

        meter.timing.assert_called_with('base.backend.www_blah_com.500.miss', ANY)

    def test_with_metrics_http_error_cached(self):
        req_headers = {
            'X-Cache': 'HIT from localhost',
        }
        exc = urllib2.HTTPError(None, 500, "Internal Server Error", req_headers, None)

        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.side_effect = exc
            self.assertRaises(urllib2.HTTPError, rest_scopes.urlread,
                              meter, 'base', 'http://www.blah.com/foo/bar?a=3', headers={'A': 3})

        meter.timing.assert_called_with('base.backend.www_blah_com.500.hit', ANY)

    def test_with_metrics_ok_cached_with_hostname(self):
        netfh = Mock()
        netfh.read.return_value = 'response'
        netfh.headers = {
            'X-Cache': 'HIT from juju-env-u1-psearch-machine-1',
            'X-Cache-Lookup': 'dont care',
        }

        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.return_value = netfh
            rest_scopes.urlread(meter, 'base', 'http://www.blah.com/foo/bar?a=3', headers={'A': 3})

        meter.timing.assert_called_with('base.backend.www_blah_com.200.hit', ANY)

    def test_with_metrics_other_error(self):
        meter = Mock()
        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.side_effect = ValueError("something bad")
            self.assertRaises(ValueError, rest_scopes.urlread, meter, 'base', 'http://blah.com')

        meter.timing.assert_called_with('base.backend.blah_com.ValueError', ANY)

    def test_log_httperror_body(self):
        """Checks that it logs on http error, including the body response."""
        body = "yours was a naughty http request!"
        exc = urllib2.HTTPError(None, 500, "Internal Server Error", {}, io.BytesIO(body))
        url = 'http://www.blah.com/foo/bar?a=3'

        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.side_effect = exc
            self.assertRaises(urllib2.HTTPError, rest_scopes.urlread, None, '', url)

        self.assertLogged("HTTPError", "500", "Internal Server Error", url, body)

    def test_log_httperror_no_body(self):
        """Checks that it logs on http error, even if there's no body."""
        exc = urllib2.HTTPError(None, 500, "Internal Server Error", {}, None)
        url = 'http://www.blah.com/foo/bar?a=3'

        with patch.object(urllib2, 'urlopen') as mock_urlopen:
            mock_urlopen.side_effect = exc
            self.assertRaises(urllib2.HTTPError, rest_scopes.urlread, None, '', url)

        self.assertLogged("HTTPError", "500", "Internal Server Error", url)
