# Copyright 2014 Canonical
# All Rights Reserved

"""Tests for some utils."""

import unittest

import utils


class MarkupCleanerTestCase(unittest.TestCase):
    """Test the markup cleaner function."""

    def test_simple(self):
        html = "text1<br><b>text2</b>"
        resp = utils.markup_cleaner(html)
        self.assertEqual(resp, "text1\n\ntext2\n")

    def test_remove_common(self):
        html = "text1<br>XXX<br><b>text2</b><br>XXX<br>XXX<br>XXX<br>XXX<br>"
        resp = utils.markup_cleaner(html)
        self.assertEqual(resp, "text1\n\ntext2\n")

    def test_remove_double_empty(self):
        html = "text1<br><br><br><br><b>text2</b>"
        resp = utils.markup_cleaner(html)
        self.assertEqual(resp, "text1\n\ntext2\n")


class LocaleSanitizerTestCase(unittest.TestCase):
    """Test the markup cleaner function."""

    def test_ok(self):
        resp = utils.sanitize_locale("en_US")
        self.assertEqual(resp, "en_US")

    def test_none(self):
        resp = utils.sanitize_locale(None)
        self.assertEqual(resp, utils.DEFAULT_LOCALE)

    def test_garbage(self):
        resp = utils.sanitize_locale("en/US")
        self.assertEqual(resp, utils.DEFAULT_LOCALE)

    def test_c(self):
        resp = utils.sanitize_locale("C")
        self.assertEqual(resp, "en_US")

    def test_encoding(self):
        resp = utils.sanitize_locale("en_US.UTF-8")
        self.assertEqual(resp, "en_US")


class AvoidableQueryTestCase(unittest.TestCase):
    """Test the avoidable query function."""

    def test_ok(self):
        self.assertFalse(utils.avoidable_query(u"simple"))

    def test_too_long(self):
        q = u"qwertyuiopasdfghjklzxcvbnm" * 10
        assert q > utils.MAX_QUERY_LENGTH
        self.assertTrue(utils.avoidable_query(q))

    def test_not_so_repeated(self):
        self.assertFalse(utils.avoidable_query(u"banana anana"))

    def test_too_repeated(self):
        self.assertTrue(utils.avoidable_query(u"adjskajhxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxdadhjsa"))

    def test_too_repeated_twice(self):
        q = u"adjskajhxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxmb,mb,mb,mb,hkkkkkkkkkkkkkkkkkkkkkkkkkpouiud"
        self.assertTrue(utils.avoidable_query(q))

    def test_simple_repeated_5(self):
        q = u"x" * utils.MAX_REPEATED_IN_A_ROW
        self.assertFalse(utils.avoidable_query(q))

    def test_simple_repeated_6(self):
        q = u"x" * (utils.MAX_REPEATED_IN_A_ROW + 1)
        self.assertTrue(utils.avoidable_query(q))

    def test_simple_repeated_7(self):
        q = u"x" * (utils.MAX_REPEATED_IN_A_ROW + 2)
        self.assertTrue(utils.avoidable_query(q))

    def test_simple_repeated_again(self):
        q = u"x" * utils.MAX_REPEATED_IN_A_ROW
        q += u"yz"
        q += u"x" * (utils.MAX_REPEATED_IN_A_ROW + 1)
        self.assertTrue(utils.avoidable_query(q))
