# Copyright 2014 Canonical Ltd.
# All Rights Reserved

"""A collection of small utilities."""

import collections
import urlparse

from translator import translator

import html2text

# the location of unity icons in the desktop
ICON_DIR = 'file:///usr/share/icons/unity-icon-theme/places/svg/'

# the locale can be formed only by these chars
ALLOWED_LOCALE_CHARS = set("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_")
DEFAULT_LOCALE = "en"

# limits for the avoidable query function
MAX_QUERY_LENGTH = 100
MAX_REPEATED_IN_A_ROW = 5


def full_icon_url(name):
    """Make the full icon url."""
    return urlparse.urljoin(ICON_DIR, name)


def sanitize_locale(locale):
    """Assure a valid locale is returned, a default or a controlled one."""
    if locale is None:
        locale = DEFAULT_LOCALE
    else:
        locale = locale.split(".")[0]
        if set(locale) - ALLOWED_LOCALE_CHARS:
            locale = DEFAULT_LOCALE
        if locale == 'C':
            locale = 'en_US'
    return locale


def localized_category(category, locale):
    """Copy the category and localize its title."""
    c = category.copy()
    c['title'] = translator(c['title'], locale)
    return c


def markup_cleaner(html):
    """Clean up an html from all the markup and leave a nice text."""
    h = html2text.HTML2Text()
    h.ignore_emphasis = True
    h.ignore_links = True
    h.ignore_images = True
    text = h.handle(html)

    # split in lines, calculate which happen too frequent (garbage, surely)
    lines = [line.strip() for line in text.split("\n")]
    c = collections.Counter(lines)
    to_remove = set(line for line, quant in c.items() if line and quant > 4)

    # remove those too-frequent lines, and avoid double empty lines
    cleaned = []
    prv = None
    for line in lines:
        if line in to_remove:
            continue
        if line == '' and prv == '':
            continue
        cleaned.append(line)
        prv = line

    text = "\n".join(cleaned)
    return text


def avoidable_query(query):
    """Decide if the query is to be avoided."""
    # filter by max length
    if len(query) > MAX_QUERY_LENGTH:
        return True

    # detect a "lot of same letter in the row"
    cnt = 1
    prev = None
    for char in query:
        if char == prev:
            cnt += 1
            if cnt > MAX_REPEATED_IN_A_ROW:
                return True
        else:
            cnt = 1
            prev = char
    return False
