#!/usr/bin/env python3

import json
import optparse
import time
import uuid
import urllib.request
import urllib.parse

from pprint import pprint
from xml.etree import ElementTree

BASE_URL = 'http://localhost:8000/'

HEADERS = {
    'Cache-Control': 'no-cache',
}


def get_location():
    resp = urllib.request.urlopen("http://geoip.ubuntu.com/lookup").read()
    root = ElementTree.fromstring(resp)
    latitude = float(root.find("Latitude").text)
    longitude = float(root.find("Longitude").text)
    return latitude, longitude


def interactive(base_url, scope_name, search, locale, verbose,
                department, filters, partner, platform='desktop'):
    """Do a search, show it all, offer a preview."""
    session = str(uuid.uuid1())

    # build params
    lat, lon = get_location()
    params = {
        'q': search,
        'session_id': session,
        'platform': platform,
        'latitude': lat,
        'longitude': lon,
    }
    if locale is not None:
        params["locale"] = locale
        params["country"] = locale.split("_")[1]
    if department is not None:
        params["department"] = department
    if filters is not None:
        print("Using filters:")
        pprint(json.loads(filters))
        params["filters"] = urllib.parse.quote(filters)

    print("\nSearching...")
    tini = time.time()
    url = base_url + scope_name + "/search?" + urllib.parse.urlencode(params)
    if verbose:
        print("  hitting url:", repr(url))

    # put a partner in the User Agent header, if needed
    headers = HEADERS.copy()
    if partner is not None:
        headers['User-Agent'] = "partner=" + urllib.parse.quote(partner)
    if verbose:
        print("  with header:", repr(headers))

    results = []
    try:
        req = urllib.request.Request(url, headers=headers)
        for response in urllib.request.urlopen(req):
            if not response:
                continue
            data = json.loads(response.decode('utf8'))
            tdelta = int(1000 * (time.time() - tini))
            result = data.get('result')
            if result is None:
                if verbose:
                    item_type, item_content = list(data.items())[0]
                    print("\n{} ({:d} msec):".format(item_type.title(),
                                                     tdelta))
                    pprint(item_content)
            else:
                print("\nResult {} ({:d} msec):".format(len(results), tdelta))
                results.append(result)
                pprint(result)

    except urllib.error.HTTPError as exc:
        print("Error:", exc, exc.read())
        return

    if not results:
        print("No result!")
        return

    inp = input("\nPreview? ")
    if not inp.isdigit() or not (0 <= int(inp) < len(results)):
        return

    # build params
    result_chosen = results[int(inp)]
    params = {
        'q': search,
        'session_id': session,
        'platform': platform,
        'widgets_api_version':  1,
        'result': json.dumps(result_chosen),
    }
    if locale is not None:
        params["locale"] = locale
        params["country"] = locale.split("_")[1]


    print("\nPreviewing...")
    tini = time.time()
    url = base_url + scope_name + "/preview?" + urllib.parse.urlencode(params)
    if verbose:
        print("  hitting url:", repr(url))
        print("  with header:", repr(headers))

    try:
        req = urllib.request.Request(url, headers=headers)
        for response in urllib.request.urlopen(req):
            if not response:
                continue
            tdelta = int(1000 * (time.time() - tini))
            data = json.loads(response.decode('utf8'))
            print("\n({:d} msec):".format(tdelta))
            pprint(data)
    except urllib.error.HTTPError as exc:
        print("Error:", exc, exc.read())
        return


def remote_scopes(base_url, locale, verbose, partner):
    """Do a search, show it all, offer a preview."""
    if locale is None:
        endpoint = "remote-scopes"
    else:
        params = {
            "locale": locale,
            "country": locale.split("_")[1],
        }
        endpoint = "remote-scopes?" + urllib.parse.urlencode(params)

    # put a partner in the User Agent header, if needed
    headers = HEADERS.copy()
    if partner is not None:
        headers['User-Agent'] = "partner=" + urllib.parse.quote(partner)

    print("\nListing...")
    url = base_url + endpoint
    if verbose:
        print("  hitting url:", repr(url))
        print("  with header:", repr(headers))
    try:
        req = urllib.request.Request(url, headers=headers)
        u = urllib.request.urlopen(req)
        raw = u.read()
    except urllib.error.HTTPError as exc:
        print("Error:", exc, exc.read())
        return

    for scope in json.loads(raw.decode('utf8')):
        print()
        pprint(scope)


USAGE = """
Usage: check_scope.py [--verbose|-v] [--locale|-l locale]
                      [--partner|-p partner] [--platform|-t platform]

...and choose the mode:
    list all:     --remote-scopes
    hit a scope:  scope_name search_string
        in this case, these can be also used:
            [--filters|-f filters] (already json'ized, e.g.: '{"foo": ["b"]}')
            [--department|-d department]
"""

if __name__ == '__main__':
    parser = optparse.OptionParser()
    parser.add_option("-l", "--locale")
    parser.add_option("-v", "--verbose", action="store_true")
    parser.add_option("-r", "--remote-scopes", action="store_true")
    parser.add_option("-d", "--department", action="store")
    parser.add_option("-f", "--filters", action="store")
    parser.add_option("-p", "--partner", action="store")
    parser.add_option("-t", "--platform", action="store")

    options, args = parser.parse_args()
    if options.remote_scopes:
        remote_scopes(BASE_URL, options.locale, options.verbose,
                      options.partner)
    elif len(args) == 2:
        scope_name, search = args
        interactive(BASE_URL, scope_name, search,
                    options.locale, options.verbose, options.department,
                    options.filters, options.partner, options.platform)
    else:
        print(USAGE)
        exit()

