
import time
import urllib2
import yaml

from codecs import open

import bottlenose

from lxml import etree


SHOPS = [
    ("CA", "u1treatyrslf-ca-20"),
    ("CN", "u1treatyrslf-23"),
    ("DE", "u1treatyrslf-21"),
    ("ES", "u1treatyrslf-es-21"),
    ("FR", "u1treatyrslf-fr-21"),
    ("IT", "u1treatyrslf-it-21"),
    ("JP", "u1treatyrslf-22"),
    ("UK", "u1treatyrslf-uk-21"),
    ("US", "u1treatyrslf-20"),
]

AMZ_VERSION = "2011-08-01"
AMZ_NS = {'x': "http://webservices.amazon.com/AWSECommerceService/2011-08-01"}

MAX_DEPTH = 5

def retrier(func):
    """A decorator that will retry calling if gets HTTP error 503."""
    delays = [1, 3, 5, 10, 30, 30, 30, 60, 120, 600, None]
    def _f(*args, **kwargs):
        for delay in delays:
            try:
                return func(*args, **kwargs)
            except urllib2.HTTPError as err:
                if err.code == 503 and delay is not None:
                    time.sleep(delay)
                    continue
                raise
    return _f


class Amazon(object):
    """The interface to get Amazon info through the API."""

    def __init__(self, shop, tag):
        with open('configs/dev-secrets.yaml', 'rt', encoding='utf8') as fh:
            config = yaml.load(fh)['amazon']
        self.amz = bottlenose.Amazon(config['key'], config['secret'],
                                     Version=AMZ_VERSION,
                                     AssociateTag=tag, Region=shop)

    @retrier
    def get_surfacing_results(self, root_code, dep_id):
        """Get surfacing results for a department."""
        kwargs = dict(
            Keywords='',
            SearchIndex=root_code,
            ResponseGroup="Images,ItemAttributes,OfferSummary,Offers",
            BrowseNode=dep_id,
        )
        raw = self.amz.ItemSearch(**kwargs)
        xml = etree.XML(raw)
        items = xml.xpath("//x:Item", namespaces=AMZ_NS)
        results = []
        for item in items:
            title = item.findtext("x:ItemAttributes/x:Title",
                                  namespaces=AMZ_NS)
            results.append(title)
        return results

    @retrier
    def get_children(self, dep_id):
        """Get the department children for current node."""
        raw = self.amz.BrowseNodeLookup(BrowseNodeId=dep_id,
                                        ResponseGroup='BrowseNodeInfo')
        xml = etree.XML(raw)
        nodes = xml.xpath("//x:BrowseNodes/x:BrowseNode/x:Children",
                          namespaces=AMZ_NS)
        children = []
        if nodes:
            for child in nodes[0]:
                data = {e.tag.split('}')[1]: e.text for e in child}
                child = {'name': data['Name'], 'id': data['BrowseNodeId']}
                children.append(child)
        return children


def explore(amz, root_code, dep_id, dep_label, level=1):
    """Explore a department tree."""
    indent = "    " * level
    result = amz.get_surfacing_results(root_code, dep_id)
    sample = ", ".join(map(repr, result[:3]))
    if len(result) > 3:
        sample += ", ..."
    alert = "" if len(result) == 10 else "WARNING -- "
    print "%s%s%s (%s): %s [%s]" % (indent, alert, dep_label, dep_id,
                                  len(result), sample)
    if level == MAX_DEPTH:
        return
    for child in amz.get_children(dep_id):
        explore(amz, root_code, child['id'], child['name'], level+1)


def main():
    """Main entry point."""
    # open the departments config
    with open('configs/scopes/amazon_departments.yaml', 'rb') as fh:
        departments = yaml.load(fh.read())

    for shop, tag in SHOPS:
        amz = Amazon(shop, tag)
        deps = departments[shop]
        for dep_id, (root_code, dep_label) in deps.items():
            print "\nStarting: %s - %s" % (shop, root_code)
            explore(amz, root_code, dep_id, dep_label)


if __name__ == '__main__':
    main()
